﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirFusion.Dandelion
{
	#region Extensions

	/// <summary>
	/// Extensions.
	/// </summary>
	public class AODException : Exception
	{
		/// <summary>
		/// Gets the status description of the AOD Controller.
		/// </summary>
		public virtual string StatusDescription { get; set; }

		/// <summary>
		/// Gets the status of the AOD Controller.
		/// </summary>
		public virtual AODExceptionStatus StatusCode { get; set; }


		/// <summary>
		/// Initializes a new instance of the Exception class with the
		/// specified error message and status.
		/// </summary>
		/// <param name="message">The text of the error message.</param>
		/// <param name="status">One of the AODControllerExceptionStatus values.</param>
		public AODException(string message, AODExceptionStatus status)
		{
			this.StatusDescription = message;
			this.StatusCode = status;
		}
	}

	#endregion


	#region Definitions

	/// <summary>
	/// Defines status for the AODException class.
	/// </summary>
	public enum AODExceptionStatus : int
	{
		/// <summary>
		/// No error was encountered.
		/// </summary>
		ERR_Success = 0,

		/// <summary>
		/// An exception of unknown type has occurred.
		/// </summary>
		ERR_Default = 1,

		/// <summary>
		/// The external network doesn't work.
		/// </summary>
		ERR_Internet = 2,

		/// <summary>
		/// A WiFi componet doesn't enable.
		/// </summary>
		ERR_WiFi = 3,

		/// <summary>
		/// The operation value is invalid or empty.
		/// </summary>
		ERR_Value = 4,

		/// <summary>
		/// The remote service point could not be contacted.
		/// </summary>
		ERR_Server = 5
	}

	#endregion
}
