﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Win32;

namespace AirFusion.Dandelion
{
	/// <summary>
	/// Represents the Windows registry.
	/// </summary>
	class RegistryUtils
	{
		/// <summary>
		/// Creates a new subkey or opens an existing subkey for write access.
		/// </summary>
		/// <param name="subKey1"> The name or path of the subkey to open as read-only. </param>
		/// <param name="subKey2"> The name or path of the subkey to create or open. </param>
		public RegistryKey CreateSubKey(string subKey1, string subKey2)
		{
			RegistryKey tmpReg = null;

			try
			{
				tmpReg = Registry.CurrentUser.OpenSubKey(subKey1, true);
				if (tmpReg.GetSubKeyNames().Contains(subKey2) == false)
				{
					tmpReg.CreateSubKey(subKey2);
				}
			}
			catch { }

			return tmpReg;
		}

		/// <summary>
		/// Closes the key and flushes it to disk if its contents have been modified.
		/// </summary>
		/// <param name="reg"> A key-level node in the Windows registry.</param>
		public void CloseRegistryKey(RegistryKey reg)
		{
			if (reg != null)
			{
				reg.Close();
			}
		}

		/// <summary>
		/// Sets the value of a name/value pair in the registry key.
		/// </summary>
		/// <param name="keyName"> The full registry path of the key. </param>
		/// <param name="valueName"> The name of the name/value pair. </param>
		/// <param name="defaultValue"> The data to be stored. </param>
		/// <param name="valueKind"> The registry data type to use when storing the data. </param>
		/// <exception cref="Exception"></exception>
		public void SetRegistryValue(string keyName, string valueName, object defaultValue,
																					RegistryValueKind valueKind)
		{
			try
			{
				Registry.SetValue(keyName, valueName, defaultValue, valueKind);
			}
			catch
			{
				throw;
			}
		}

		/// <summary>
		/// Retrieves the value associated with the specified name, in the specified registry key.
		/// </summary>
		/// <param name="keyName"> The full registry path of the key. </param>
		/// <param name="valueName"> The name of the name/value pair. </param>
		/// <param name="defaultValue"> The value to return if valueName does not exist. </param>
		/// <exception cref="Exception"></exception>
		public object GetRegistryValue(string keyName, string valueName, object defaultValue)
		{
			object o = null;

			try
			{
				o = Registry.GetValue(keyName, valueName, defaultValue);
			}
			catch
			{
				throw;
			}

			return o;
		}

	}
}
