﻿using AirFusion.Common;
using AirFusion.ConnectionManager;
using AirFusion.Manager;
using AirFusion.Manager.DeviceManager;
using AirFusion.Manager.FeatureHelper;

//using AirFusion.PipeHelper; // 2015-05-20 Seoker: remove Auto-login functions since it's defeatured
using AirFusion.WebService.Adapters;
using AirFusion.Windows;
using AirFusionCustomToast;
using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media.Imaging;

namespace AirFusion
{
	/// <summary>
	/// Control logic for App.cs
	/// </summary>
	public partial class App
	{
		#region Definitions & Members

		// For instance
		private const string _instanceMutexGUID = "{2F312437-EB37-4596-8D64-634087172B9A}";

		private SingleInstance _singleInstance;

		// For background services
		private delegate InitialStatus ConnectingWorkDelegate(AppDeviceInfo deviceinfo);

		private AirFusionManager _manager = null;
		private GeneralNotificationManager _generalNotificationManager = null;
		//private PhoneCallToastManager _phoneCallToastManager = null; // 2015-05-20 Seoker: remove incoming call functions since it's defeatured

		// For file transferring records
		private int _nTotalFileCount = 0;

		private int _nCurrentFinishFileCount = 0;

		// For auto login service
		//private ALUIPipeManager _pipeManager = null; // 2015-05-20 Seoker: remove Auto-Login functions since it's defeatured

		// For lock code block with function name and object
		private Dictionary<object, object> _functionObjectLockObj = new Dictionary<object, object>();

		// For QR Code
		private QRCodeGenerator _qRCodeGenerator = null;

		// For blocking the timer of waiting for receiving data and event "CheckReceiveData"
		private ManualResetEvent _signalOfReceiveDataTimerStarted = new ManualResetEvent(false);

		// For ADB commands
		private Process _runInputDaemonProcess = null;

		private Process _runScreenCastingDaemonProcess = null;

		//Johnson add for set KeyFrameInterval
		public uint _mKeyFrameInterval = 4;

		public uint _mBitRate = 5000000;
		public bool _mManualReset = false;
		public int _mScreenCastWidth = 0, _mScreenCastHeight = 0;

		#endregion Definitions & Members

		#region Events

		public event EventHandler<AppEventArgs> AppEvent;

		#endregion Events

		#region Properties

		public string ApplicationVersion { get; private set; }

		public string MinimumTargetVersion { get; private set; }

		// For controller and container
		private object _vncControllerLockObj = new object();

		private VNCController _vncController = null;

		public VNCController VNCController
		{
			get
			{
				lock (_vncControllerLockObj)
				{
					return _vncController;
				}
			}

			set
			{
				lock (_vncControllerLockObj)
				{
					_vncController = value;
				}
			}
		}

		private object _rtspControllerLockObj = new object();
		private RTSPController _rtspController = null;

		public RTSPController RTSPController
		{
			get
			{
				lock (_rtspControllerLockObj)
				{
					return _rtspController;
				}
			}

			set
			{
				lock (_rtspControllerLockObj)
				{
					_rtspController = value;
				}
			}
		}

		private object _isStartConnectionLockObj = new object();
		private bool _isStartConnection = false;

		public bool IsStartConnection
		{
			get
			{
				lock (_isStartConnectionLockObj)
				{
					return _isStartConnection;
				}
			}

			set
			{
				lock (_isStartConnectionLockObj)
				{
					_isStartConnection = value;
				}
			}
		}

		private object _connectingdeviceInfoLockObj = new object();
		private AppDeviceInfo _connecingdeviceInfo = null;

		public AppDeviceInfo ConnectingDeviceInfo
		{
			get
			{
				lock (_connectingdeviceInfoLockObj)
				{
					return _connecingdeviceInfo;
				}
			}

			set
			{
				lock (_connectingdeviceInfoLockObj)
				{
					_connecingdeviceInfo = value;

					if (null != _connecingdeviceInfo)
					{
						this.SetLastLaunchDeviceID(_connecingdeviceInfo.Device.InstanceID);
					}
				}
			}
		}

		private object _pairedDevicesLockObj = new object();
		private ObservableCollection<AppDeviceInfo> _pairedDevices = null;

		public ObservableCollection<AppDeviceInfo> PairedDevices
		{
			get
			{
				lock (_pairedDevicesLockObj)
				{
					return _pairedDevices;
				}
			}

			set
			{
				lock (_pairedDevicesLockObj)
				{
					_pairedDevices = value;
				}
			}
		}

		private object _lastLaunchedIDLockObj = new object();
		private Guid _lastLaunchedID = Guid.Empty;

		public Guid LastLaunchID
		{
			get
			{
				lock (_lastLaunchedIDLockObj)
				{
					return _lastLaunchedID;
				}
			}

			set
			{
				lock (_lastLaunchedIDLockObj)
				{
					_lastLaunchedID = value;
				}
			}
		}

		private object _newPairedDeviceLockObj = new object();
		private AFUserDevice _newPairedDevice = null;

		public AFUserDevice NewPairedDevice
		{
			get
			{
				lock (_newPairedDeviceLockObj)
				{
					return _newPairedDevice;
				}
			}

			set
			{
				lock (_newPairedDeviceLockObj)
				{
					_newPairedDevice = value;
				}
			}
		}

		#region For Audio Disable check

		// For audio function
		private bool _isAudioDisabled = false;

		public bool IsAudioDisabled
		{
			get
			{
				return _isAudioDisabled;
			}
		}

		#endregion For Audio Disable check

		// For locking code block of restarting screen casting
		private object _restartingScreenCastingLockObj = new object();

		private Task<InitialStatus> _connectingWorker = null;
		private CancellationTokenSource _tokenSource = null;
		private AutoResetEvent _resetEventForWorkCancenlled = new AutoResetEvent(false);
		private Thread _doWorkerThread = null;

		private object _isNetworkGoodLockObj = new object();
		private bool _isNetworkGood = true;

		public bool IsNetworkGood
		{
			get
			{
				lock (_isNetworkGoodLockObj)
				{
					return _isNetworkGood;
				}
			}

			set
			{
				lock (_isNetworkGoodLockObj)
				{
					if (_isNetworkGood != value)
					{
						_isNetworkGood = value;
						this.OnPropertyChanged("IsNetworkGood");
					}
				}
			}
		}

		// For the timer of waiting for pairing
		private object _waitForPairingTimerLockObj = new object();

		private System.Timers.Timer _waitForPairingTimer = null;

		public System.Timers.Timer WaitForPairingTimer
		{
			get
			{
				lock (_waitForPairingTimerLockObj)
				{
					return _waitForPairingTimer;
				}
			}

			set
			{
				lock (_waitForPairingTimerLockObj)
				{
					_waitForPairingTimer = value;
				}
			}
		}

		// For the timer of waiting for receiving data
		private object _waitForReceiveDataTimerLockObj = new object();

		private System.Timers.Timer _waitForReceiveDataTimer = null;

		public System.Timers.Timer WaitForReceiveDataTimer
		{
			get
			{
				lock (_waitForReceiveDataTimerLockObj)
				{
					return _waitForReceiveDataTimer;
				}
			}

			set
			{
				lock (_waitForReceiveDataTimerLockObj)
				{
					_waitForReceiveDataTimer = value;
				}
			}
		}

		private GlobalSettings _settings = null;

		public GlobalSettings Settings
		{
			get
			{
				return this._settings;
			}

			private set
			{
				this._settings = value;
			}
		}

		#endregion Properties

		#region Private Methods

		#region Project Common Mehtod

		/// <summary>
		/// For Check only one application is running, need to check single instance
		/// </summary>
		/// <returns></returns>
		private bool SingleInstance()
		{
			_singleInstance = new SingleInstance(_instanceMutexGUID);

			if (!_singleInstance.IsSingleInstance)
			{
				return false;
			}

			return true;
		}

		private static Random random = new Random((int)DateTime.Now.Ticks);

		private string RandomString(int size)
		{
			StringBuilder builder = new StringBuilder();
			char ch;
			for (int i = 0; i < size; i++)
			{
				ch = Convert.ToChar(Convert.ToInt32(Math.Floor(26 * random.NextDouble() + 65)));
				builder.Append(ch);
			}

			return builder.ToString();
		}

		#endregion Project Common Mehtod

		#region For AcerEXTEND Common Method

		/// <summary>
		/// Gets the last launch remote device ID from registry
		/// </summary>
		private void GetLastLaunchDeviceID()
		{
			// Get last launched device ID from registry
			try
			{
				LogtoFile.TrackEnter();

				bool isX86;
				object oRK = RegistryUtility.GetRegistry(RegistryHive.CurrentUser, Configs.RegUIGeneralPath, Configs.RegLastLaunchIDKey, RegistryView.Registry64, out isX86);

				if (oRK != null)
				{
					this._lastLaunchedID = Guid.Parse((string)oRK);
				}
			}
			catch
			{
				this._lastLaunchedID = Guid.Empty;
			}
			finally
			{
				LogtoFile.TrackLeave();
			}
		}

		/// <summary>
		/// Sets the last launch Remote Device ID to Registry
		/// </summary>
		/// <param name="id">The identifier.</param>
		/// <returns>
		/// True is represent success set value to registry
		/// False is represent fail set value to registry
		/// </returns>
		private bool SetLastLaunchDeviceID(Guid id)
		{
			this.LastLaunchID = id;
			bool boo = RegistryUtility.SetRegistry(RegistryHive.CurrentUser, Configs.RegUIGeneralPath, Configs.RegLastLaunchIDKey, id.ToString(), RegistryView.Registry64);

			return boo;
		}

		/// <summary>
		/// Deletes the last launch Remote Device ID to Registry
		/// </summary>
		private void DeleteLastLaunchDeviceID()
		{
			this.LastLaunchID = Guid.Empty;
			RegistryUtility.DeleteRegistry(RegistryHive.CurrentUser, Configs.RegUIGeneralPath, Configs.RegLastLaunchIDKey, RegistryView.Registry64, false);
		}

		#endregion For AcerEXTEND Common Method

		#region initial value

		/// <summary>
		/// Initializes System Event
		/// </summary>
		/// The function will listen system event about Session Change, Suspend Event.
		private void InitializeEvents()
		{
			try
			{
				LogtoFile.TrackEnter();

				SystemEvents.SessionSwitch += this.SystemEventsSessionSwitchCallback;
				SystemEvents.PowerModeChanged += this.SystemEventsPowerModeChangedCallback;
				SystemEvents.EventsThreadShutdown += this.SystemEventsEventsThreadShutdownCallback;
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
			finally
			{
				LogtoFile.TrackLeave();
			}
		}

		/// <summary>
		/// Initializes the values.
		/// </summary>
		private void InitializeValues()
		{
			try
			{
				LogtoFile.TrackEnter();

				#region Inital Device Manager about paired Device

				try
				{
					this.PairedDevices = new ObservableCollection<AppDeviceInfo>();
					this.PairedDevices.CollectionChanged += delegate(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
					{
						var target = sender as ObservableCollection<AppDeviceInfo>;

						if (null != target)
						{
							this.SendAppEventAsync(new AppEventArgs(AppEventType.PairedDeviceListShow, null, target.Count() == 0 ? false : true));
						}
					};
				}
				catch (Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "InitializeValues PairedDevices has some exception : " + ex.ToString(), "");
				}

				#endregion Inital Device Manager about paired Device

				// Get Last Launch Device Info from registry
				this.GetLastLaunchDeviceID();

				// For special purpose(debug use), Get audio disable from registry
				this._isAudioDisabled = AERegSetting.GetIsDisableAudioSetting();
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
			finally
			{
				LogtoFile.TrackLeave();
			}
		}

		/// <summary>
		/// Initializes the AirFusion manager and Some setting.
		/// </summary>
		/// <param name="isStartUpLaunch">if set to <c>true</c> It will not call screencast windows,
		/// represent it just windows boot launch</param>
		private void InitializeAFManager(bool isStartUpLaunch = false)
		{
			LogtoFile.TrackEnter();
			try
			{
				#region Initial some handler

				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Initial AirFusionManager", "");
				this._manager = new AirFusionManager();

				//0609 Johnon add
				this._manager.MixHelper.SettingSupportVersion(this.ApplicationVersion, this.MinimumTargetVersion /*"1.00.2014"*/);

				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Initial QRCodeGenerator", "");
				this._qRCodeGenerator = new QRCodeGenerator();

				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Initial GeneralNotificationManager", "");
				this._generalNotificationManager = new GeneralNotificationManager(this._manager.NotificationManager);

				#endregion Initial some handler

				#region Event Handle

				//For Notification Event Handle
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Initial Notification Event", "");
				this.NotificationEventManager();

				//For AirFusion Web Service Event Handle
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Initial AFManager Event", "");
				this.AirFusionEventManager();

				#endregion Event Handle

				// initialize
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Initial AFManager", "");
				this._manager.Initialize();

				// initialize global settings
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Initial GlobalSettings", "");

				// Get settings
				this.Settings = new GlobalSettings(this._generalNotificationManager)
				{
					IsMute = AERegSetting.GetBoolGlobalSettings(Configs.RegMute, false),
					IsAutoUpdate = AERegSetting.GetBoolGlobalSettings(Configs.RegAutoUpdate, true),
					IsStartUpLaunch = AERegSetting.GetBoolGlobalSettings(Configs.RegStartUpLaunch, false),
				};

				//0422 Johnson add, if application launch by windows boots, don't need to show window, just show tray icon
				if (!isStartUpLaunch)
				{
					// Before loading paired devices, navigate to correct window
					this.PrepareScreenConnection();
				}

				//ToDoIt need to check VNC Event is OK for new flow
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Initial VNC Event", "");
				// Set VNC Event
				VNC.VNCClient.FileTransferUpdateEvent += VNCClientFileTransferUpdateEventCallback;
				VNC.VNCClient.VNCMessageEvent += VNCClientVNCMessageEventCallback;

				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Initial InitializePairedDeviceList", "");

				// This function must be called after initializing AFManager
				this.CheckPairedDeviceList();

				//For auto Update
				if (AERegSetting.GetBoolGlobalSettings(Configs.RegAutoUpdate, false))
				{
					Task.Run(() =>
					{
						UpgradeTool.SelfCheckUpdate(UpgradeTool.LaunchType.AutoCheckUpdate);
					});
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
			finally
			{
				LogtoFile.TrackLeave();
			}
		}

		#endregion initial value

		private async Task CreateNewConnection(AppDeviceInfo deviceInfo, bool isResetDevice)
		{
			this.Dispatcher.Invoke(delegate()
			{
				FlowControlManager.Instance.StartNewConnection(deviceInfo, this._appScreenWindow);
			});

			//// Before assign the new connetion device information, to stop screen casting daemon if need.
			//if (null != this.ConnectingDeviceInfo)
			//{
			//	// Call DoStopScreenCastingDaemonAsync, need to await
			//	await this.DoStopScreenCastingDaemonAsync(this.ConnectingDeviceInfo.Device).ConfigureAwait(false);
			//}

			////0514 Johnson add for resolve difference Aspect Ratio phone not reset screen resolution issue
			//if (null == this.ConnectingDeviceInfo || deviceInfo != this.ConnectingDeviceInfo)
			//{
			//	Configs.ApplicationReference.ResetScreencastResolution();
			//	this._appScreenWindow.RemoveVQOptionToVQList();
			//}

			////0513 Johnon add for save current device Info
			//SaveCurrentDeviceInfo(deviceInfo);

			//LogtoFile.WriteLog("", Configs.DebugLevelValue, "Assign ConnectingDeviceInfo", "");

			//#region Run task work

			//this._tokenSource = new CancellationTokenSource();

			//// Task work
			//this._connectingWorker = Task.Run(async () =>
			//{
			//	LogtoFile.WriteLog("", Configs.DebugLevelValue, "Enter this._connectingWorker work", "");

			//	try
			//	{
			//		LogtoFile.WriteLog("", Configs.DebugLevelValue, "Assign do work thread", "");
			//		this._doWorkerThread = Thread.CurrentThread;

			//		this.CloseConnection();

			//		//For change resoultion use, if the user want to change resoultion,
			//		//the screen height and width value is not zero.
			//		if (this._mScreenCastHeight != 0 || this._mScreenCastWidth != 0)
			//		{
			//			Thread.Sleep(400);
			//		}

			//		//Johnson modify it
			//		//Because belown function "DoWorkBeforeConnection" will call wake up by default IP and WebService provide IP
			//		//So we should not call this function to improve user expreience, when phone can't connect to network,
			//		//it will cast more time.

			//		//Before starting new connection, to wake up target device
			//		//LogtoFile.WriteLog("", Configs.DebugLevelValue, "Before calling wake up function, deviceInfo.ServerIP: " + deviceInfo.ServerIP, "");
			//		//LogtoFile.WriteLog("", Configs.DebugLevelValue, "Call wake up before connection", "");
			//		//bool isWakeUpSuccess = false;
			//		//if (null != deviceInfo.Device)
			//		//{
			//		//	try
			//		//	{
			//		//		//Johnson moidfy it
			//		//		isWakeUpSuccess = ConnectingDeviceInfo.Device.WakeUp();
			//		//	}
			//		//	catch (Exception ex)
			//		//	{
			//		//		LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			//		//	}
			//		//}
			//		//else
			//		//{
			//		//	LogtoFile.WriteLog("", Configs.DebugLevelValue, "deviceInfo.Device is null", "");
			//		//}

			//		//if (!isWakeUpSuccess)
			//		//{
			//		//	LogtoFile.WriteLog("", Configs.DebugLevelValue, "Failed to wake up !", "");
			//		//}
			//		//else
			//		//{
			//		//	LogtoFile.WriteLog("", Configs.DebugLevelValue, "Wake up successfully!", "");
			//		//}

			//		// Do work before connection

			//		LogtoFile.WriteLog("", Configs.DebugLevelValue, "_._ >>>>>> Start DoworkBeforeConnection", "");
			//		DeviceInfoStatus status = DeviceInfoStatus.Unconnectable;
			//		try
			//		{
			//			//status = this.DoWorkBeforeConnection(ConnectingDeviceInfo);
			//			status = await this.DoWorkBeforeConnection(ConnectingDeviceInfo).ConfigureAwait(false);

			//			if (this._appScreenWindow.CurrentScreenCastdeviceInfo.Device.Services.Screencast.SupportedResolutions != null)
			//				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Initial CurrentScreencastDeviceInfo support Screen count" + this._appScreenWindow.CurrentScreenCastdeviceInfo.Device.Services.Screencast.SupportedResolutions.Count.ToString(), "");
			//		}
			//		catch { }
			//		LogtoFile.WriteLog("", Configs.DebugLevelValue, "_._ >>>>>> End DoworkBeforeConnection", "");

			//		LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Final result status: " + status, "");
			//		if (status == DeviceInfoStatus.Unpair)
			//		{
			//			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Device is unpaired in offline", "");

			//			// Inform UI the unpaired device information
			//			this.SendAppEventAsync(new AppEventArgs(AppEventType.PairAgain, deviceInfo.Device));
			//			return InitialStatus.DirectlyAborted;
			//		}
			//		else if (status == DeviceInfoStatus.Unconnectable)
			//		{
			//			return InitialStatus.InitializeInputControlFailed;
			//		}

			//		#region 0420 Johnson add more fail log for debug use

			//		else if (status == DeviceInfoStatus.ConfirmIPFail)
			//		{
			//			return InitialStatus.DandelionReconfirmIPFail;
			//		}
			//		else if (status == DeviceInfoStatus.WakeUpFail)
			//		{
			//			return InitialStatus.InitializeWakeUpFail;
			//		}
			//		else if (status == DeviceInfoStatus.DandelionFail)
			//		{
			//			return InitialStatus.DandelionGetIPFail;
			//		}
			//		else if (status == DeviceInfoStatus.CheckDeviceStatusFail)
			//		{
			//			return InitialStatus.InitializeGetDeviceStatusFail;
			//		}
			//		else if (status == DeviceInfoStatus.CheckDeviceStatusTimeout)
			//		{
			//			return InitialStatus.InitializeGetDeviceStatusTimeout;
			//		}
			//		else if (status == DeviceInfoStatus.GetDeviceInfoFail)
			//		{
			//			return InitialStatus.InitializeGetDeviceStatusFail;
			//		}
			//		else if (status == DeviceInfoStatus.GetDeviceInfoTimeout)
			//		{
			//			return InitialStatus.InitializeGetDeviceInformationTimeout;
			//		}

			//		#endregion 0420 Johnson add more fail log for debug use

			//		if (!String.IsNullOrEmpty(deviceInfo.ServerIP))
			//		{
			//			LogtoFile.WriteLog("", Configs.DebugLevelValue, "deviceInfo.ServerIP: " + deviceInfo.ServerIP.ToString(), "");

			//			try
			//			{
			//				PhoneSystemPermissionDetectionRule rule = this.GetCurrentPhoneSystemPermissionDetectionRule();

			//				// Inform UI to connect screen casting
			//				LogtoFile.WriteLog("", Configs.DebugLevelValue, "isResetDevice: " + isResetDevice, "");
			//				// Prepare event args
			//				object[] args = new object[2];
			//				args[0] = rule;
			//				args[1] = isResetDevice;

			//				this.SendAppEventAsync(new AppEventArgs(AppEventType.ConnectScreenCasting, deviceInfo, args));

			//				// Do connecting based on phone system permission
			//				InitialStatus connectingStatusResult = await this.ConnectingWorkDetection(deviceInfo, rule).ConfigureAwait(false);
			//				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Finishing connecting work, connectingStatusResult: " + connectingStatusResult, "");

			//				return connectingStatusResult;
			//			}
			//			catch (Exception ex)
			//			{
			//				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			//				return InitialStatus.InitializeInputControlFailed;
			//			}
			//		}
			//		else
			//		{
			//			LogtoFile.WriteLog("", Configs.DebugLevelValue, "The IP address of device for new connection is empty", "");
			//			return InitialStatus.InitializeInputControlFailed;
			//		}
			//	}
			//	catch (Exception ex)
			//	{
			//		ex.ToString();

			//		LogtoFile.WriteLog("", Configs.DebugLevelValue, "To set completed signal", "");
			//		// Set the signal to notify that the canceling work is finished.
			//		//this._resetEventForWorkCancenlled.Set();
			//		LogtoFile.WriteLog("", Configs.DebugLevelValue, "CreateNewConnection _connectingWorker exception" + ex.ToString(), "");
			//		this.SendAppEventAsync(new AppEventArgs(AppEventType.InitialStatus, InitialStatus.InitializeInputControlFailed));

			//		return InitialStatus.InitializeInputControlFailed;
			//	}
			//}, this._tokenSource.Token);

			//#endregion Run task work

			//#region Run task ContinueWith work

			//// ContinueWith work
			//await this._connectingWorker.ContinueWith(antecendent =>
			//{
			//	LogtoFile.WriteLog("", Configs.DebugLevelValue, "Enter this._connectingWorker ContinueWith", "");

			//	try
			//	{
			//		InitialStatus result = antecendent.Result;

			//		if (result == InitialStatus.InitailizationsCompleted)
			//		{
			//			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Input control and screen casting are finished", "");

			//			// 2015-05-06 Seoker: the port number is not required to specify since the port number is already specified in ControlScreencastAsync()
			//			// if need, specify by following statement
			//			//this._manager.UserDeviceManager.LocalDevice.Services.Screencast.SetPort(30);
			//			/*
			//			if (null != this._manager)
			//			{
			//				if (null != this._manager.UserDeviceManager)
			//				{
			//					if (null != this._manager.UserDeviceManager.LocalDevice)
			//					{
			//						if (null != this._manager.UserDeviceManager.LocalDevice.Services)
			//						{
			//							if (null != this._manager.UserDeviceManager.LocalDevice.Services.Screencast)
			//							{
			//								this._manager.UserDeviceManager.LocalDevice.Services.Screencast.Port = Configs.DefaultRTPPort;
			//							}
			//						}
			//					}
			//				}
			//			}
			//			*/

			//			this.WaitForReceiveDataTimer.Stop();
			//			this._signalOfReceiveDataTimerStarted.Reset();
			//			if (null != this.AppScreenWindow)
			//			{
			//				// Start to timer for wairing for receving display data
			//				this.WaitForReceiveDataTimer.Interval = Configs.WaitForReceiveDataTimeoutPeriod;
			//				this.WaitForReceiveDataTimer.Start();
			//			}
			//			else
			//			{
			//				LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.AppScreenWindow is null", "");
			//			}
			//		}
			//		else if (result == InitialStatus.ConnectToRemoteServerFailed)
			//		{
			//			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Can't connect to remote server for RTSP", "");

			//			this.ClearAllConnection();
			//		}
			//		else if (result == InitialStatus.DirectlyAborted)
			//		{
			//			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Connecting work is completed with InitialStatus.DirectlyAborted", "");

			//			this.ClearAllConnection();

			//			// Set the signal to notify that the canceling work is finished.
			//			//this._resetEventForWorkCancenlled.Set();

			//			return;
			//		}

			//		this.SendAppEventAsync(new AppEventArgs(AppEventType.InitialStatus, result));
			//	}
			//	catch (Exception ex)
			//	{
			//		LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			//	}
			//	finally
			//	{
			//		this._doWorkerThread = null;
			//		this._connectingWorker = null;
			//	}

			//	LogtoFile.WriteLog("", Configs.DebugLevelValue, "End this._connectingWorker.ContinueWith", "");
			//}, this._tokenSource.Token).ConfigureAwait(false);

			//#endregion Run task ContinueWith work
		}

		private async Task<InitialStatus> ConnectingWorkDetection(AppDeviceInfo deviceInfo, PhoneSystemPermissionDetectionRule permissionDetection)
		{
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "phone system permission detection rule: " + permissionDetection.GetStringValue(), "");

			ConnectionType type = this.GetCurrentVideoProtocol();
			//Based on Meeting Discussion, we default set TCP keyframe interval is 15 and UDP keyframe interval is 4
			if (_mManualReset == false)
				SetADBScreenRecoderParameter(BiteRateType.Others, KeyFrameValueType.Others);

			switch (permissionDetection)
			{
				case PhoneSystemPermissionDetectionRule.AutoDetect:
					{
						bool isHasPermission = Utils.DetectPhoneHasSystemPermission(deviceInfo.Device);
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "isHasPermission: " + isHasPermission, "");

						if (isHasPermission)
						{
							// Do the initialization work for phone with system permission
							if (Configs.IsWithoutMiracast)
							{
								#region Connection by WebService and ADB

								// Do VNC and without Miracast
								//InitialStatus connectingResult = await this.ConnectingWorkWithoutMiracast(deviceInfo, type).ConfigureAwait(false);
								//LogtoFile.WriteLog("", Configs.DebugLevelValue, "Connecting status result: " + connectingResult, "");

								//return connectingResult;
								return await this.ConnectingWorkWithoutMiracast(deviceInfo, type).ConfigureAwait(false);

								#endregion Connection by WebService and ADB
							}
							else
							{
								#region Connection work by Miracast

								// Start receiving network package based on the specific protocol
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Send the event of start receiving", "");
								this.SendAppEventAsync(new AppEventArgs(AppEventType.StartReceiving, ConnectionType.UDP));

								// Do VNC and with Miracast
								return this.ConnectingWorkWithMiracast(deviceInfo);

								#endregion Connection work by Miracast
							}
						}
						else
						{
							// Start receiving network package based on the specific protocol
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Send the event of start receiving", "");
							this.SendAppEventAsync(new AppEventArgs(AppEventType.StartReceiving, type));

							// Do the initialization work of ADB commands
							return await this.ConnectingWorkWithADB(deviceInfo, type).ConfigureAwait(false);
						}
					}

				case PhoneSystemPermissionDetectionRule.ForceHas:
					{
						// Do the initialization work for phone with system permission
						if (Configs.IsWithoutMiracast)
						{
							// Start receiving network package based on the specific protocol
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Send the event of start receiving", "");
							this.SendAppEventAsync(new AppEventArgs(AppEventType.StartReceiving, type));

							// Do VNC and without Miracast
							return await this.ConnectingWorkWithoutMiracast(deviceInfo, type).ConfigureAwait(false);
						}
						else
						{
							// Start receiving network package based on the specific protocol
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Send the event of start receiving", "");
							this.SendAppEventAsync(new AppEventArgs(AppEventType.StartReceiving, ConnectionType.UDP));

							// Do VNC and with Miracast
							return this.ConnectingWorkWithMiracast(deviceInfo);
						}
					}

				case PhoneSystemPermissionDetectionRule.ForceNotHas:
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Send the event of start receiving", "");
						this.SendAppEventAsync(new AppEventArgs(AppEventType.StartReceiving, type));

						// Do the initialization work of ADB commands
						return await this.ConnectingWorkWithADB(deviceInfo, type).ConfigureAwait(false);
					}
			}

			return InitialStatus.DirectlyAborted;
		}

		private InitialStatus ConnectingWorkWithMiracast(AppDeviceInfo deviceinfo)
		{
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start", "");

			try
			{
				// Do VNC initialization work
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Initialize VNCController", "");
				InitialStatus initializeResult = InitialStatus.InitializeInputControl;
				this.SendAppEventAsync(new AppEventArgs(AppEventType.InitialStatus, initializeResult));

				// Set registry for LAV resolution
				this.SetCurrentVideoResolution(Configs.DefaultVideoResolutionLongSize, Configs.DefaultVideoResolutionShortSize);

				// try to connect VNC
				for (int i = 0; i < Configs.RetryTimesForInputControl; i++)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Try to connect to server for VNC, time: " + (i + 1), "");

					if (null == VNCController)
					{
						VNCController = new VNCController(deviceinfo.ServerIP, deviceinfo.ServerVNCPort, deviceinfo.ServerVNCPassword, Configs.DownloadFileFolderPath);
					}

					initializeResult = VNCController.InitializeVNC();
					if (VNCController.IsInitializationSuccess && VNCController.IsRunSuccess && initializeResult == InitialStatus.ConnectToInputControlCompleted)
					{
						break;
					}

					Thread.Sleep(Configs.RetryTimePeriodForInputControl);
				}

				if (initializeResult != InitialStatus.ConnectToInputControlCompleted)
				{
					return initializeResult;
				}

				// Do RTSP work
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Initialize RTSPController", "");
				this.SendAppEventAsync(new AppEventArgs(AppEventType.InitialStatus, InitialStatus.InitializeScreenCasting));

				if (null == RTSPController)
				{
					RTSPController = new RTSPController(deviceinfo.ServerIP, deviceinfo.ServerRTSPPort, deviceinfo.ServerRTPPort);
					RTSPController.RTSPEvent += this.RTSPControllerRTSPEventCallback;
				}

				// try to connect RTSP
				for (int i = 0; i < Configs.RetryTimesForScreenCasting; ++i)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Try to connect to server for RTSP, time: " + (i + 1), "");

					try
					{
						RTSPController.InitializeSocket();

						// If the RTSP socket is created successfully
						if (RTSPController.IsConnected == true)
						{
							if (RTSPController.Send(VideoResolutionOptions.LandscapeResolution1280x720) == string.Empty)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Sending RTSP is failed", "");
							}
							else
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Sending RTSP is successful", "");
								return InitialStatus.InitailizationsCompleted;
							}
						}
						else
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "RTSPController.IsConnected = false, Connecting Socket is failed", "");
						}
					}
					catch (Exception ex)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
					}

					// If failture, to close existed socket
					try
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Try to close socket", "");
						RTSPController.CloseSocket();
					}
					catch (Exception ex)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
					}

					Thread.Sleep(Configs.RetryTimePeriodForScreenCasting);
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}

			return InitialStatus.ConnectToRemoteServerFailed;
		}

		private async Task<InitialStatus> ConnectingWorkWithADB(AppDeviceInfo deviceinfo, ConnectionType type)
		{
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start", "");

			try
			{
				// Do adb input control initialization work
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Initialize adb input controls", "");
				InitialStatus initializeResult = InitialStatus.InitializeInputControl;
				this.SendAppEventAsync(new AppEventArgs(AppEventType.InitialStatus, initializeResult));

				//Try to read registry width and height
				try
				{
					GetRegistryVideoSizeProtocol(deviceinfo.Device, ref _mScreenCastWidth, ref _mScreenCastHeight);

					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Registry assign Screen Cast Width: " + _mScreenCastWidth, "");
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Registry assign Screen Cast Height: " + _mScreenCastHeight, "");
				}
				catch
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "GetCurrentVideoResolution() failed!", "");
				}

				// Set specific video width and height
				//int assignWidth = 0;
				//int assignHeight = 0;
				//if (!this.GetCurrentVideoResolution(deviceinfo.Device, out _mScreenCastWidth, out _mScreenCastHeight))
				//{
				//	LogtoFile.WriteLog("", Configs.DebugLevelValue, "GetCurrentVideoResolution() failed!", "");
				//	return InitialStatus.InitializeInputControlFailed;
				//}

				//LogtoFile.WriteLog("", Configs.DebugLevelValue, "assignScreenCastWidth: " + _mScreenCastWidth, "");
				//LogtoFile.WriteLog("", Configs.DebugLevelValue, "assignScreenCastHeight: " + _mScreenCastHeight, "");

				//0511 Johnson modify it. Move to below function to set LAV Resolution
				// Set registry for LAV resolution
				//this.SetCurrentVideoResolution(_mScreenCastWidth, _mScreenCastHeight);

				try
				{
					// Firsty reset process for ADB commands
					if (null != _runInputDaemonProcess)
					{
						try
						{
							_runInputDaemonProcess.Kill();
						}
						catch { }

						_runInputDaemonProcess = null;
					}
					_runInputDaemonProcess = new Process();

					// The configurations of ADB command process
					_runInputDaemonProcess.StartInfo.CreateNoWindow = true;
					_runInputDaemonProcess.StartInfo.UseShellExecute = false;
					_runInputDaemonProcess.StartInfo.WindowStyle = ProcessWindowStyle.Hidden;
					_runInputDaemonProcess.StartInfo.RedirectStandardOutput = true;
					_runInputDaemonProcess.StartInfo.RedirectStandardError = true;
					_runInputDaemonProcess.StartInfo.FileName = Configs.ADBFilePath;

					// Firsty reset process for ADB commands
					if (null != _runScreenCastingDaemonProcess)
					{
						try
						{
							_runScreenCastingDaemonProcess.Kill();
						}
						catch { }

						_runScreenCastingDaemonProcess = null;
					}
					_runScreenCastingDaemonProcess = new Process();

					// The configurations of ADB command process
					_runScreenCastingDaemonProcess.StartInfo.CreateNoWindow = true;
					_runScreenCastingDaemonProcess.StartInfo.UseShellExecute = false;
					_runScreenCastingDaemonProcess.StartInfo.WindowStyle = ProcessWindowStyle.Hidden;
					_runScreenCastingDaemonProcess.StartInfo.RedirectStandardOutput = true;
					_runScreenCastingDaemonProcess.StartInfo.RedirectStandardError = true;
					_runScreenCastingDaemonProcess.StartInfo.FileName = Configs.ADBFilePath;

					// The output/error message of ADB commands
					string[] outputMessage;
					string[] errorMessage;

					// The counter for running commands
					int commandCounter = 0;

					#region The ADB commmands for checking available device if input daemon is not running

					bool isInputDaemonAlreadyRunning = false;
					try
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call GetInputDaemonStatusAsync()", "");
						isInputDaemonAlreadyRunning = await this.GetInputDaemonStatusAsync(deviceinfo.Device).ConfigureAwait(false);
					}
					catch (Exception ex)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
					}

					LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>> Before running daemon, isInputDaemonRunning: " + isInputDaemonAlreadyRunning, "");

					if (!isInputDaemonAlreadyRunning)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "To check if the device has plugin to PC", "");

						// Run the adb command for listing down available device
						++commandCounter;
						_runInputDaemonProcess.StartInfo.Arguments = Configs.DeviceCommand ?? String.Empty;
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "StartInfo.Arguments: " + _runInputDaemonProcess.StartInfo.Arguments, "");

						_runInputDaemonProcess.Start();
						outputMessage = _runInputDaemonProcess.StandardOutput.ReadToEnd().Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
						errorMessage = _runInputDaemonProcess.StandardError.ReadToEnd().Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
						_runInputDaemonProcess.WaitForExit();
						if (errorMessage.Length != 0)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Error in command " + commandCounter + " : run the adb command for listing down available device failed", "");
							foreach (var error in errorMessage)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, error, "");
							}

							return InitialStatus.InitializeInputControlFailed;
							//return InitialStatus.InitializeADBDeviceFailed;
						}
						else
						{
							if (outputMessage.Length == 1)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Warning: No any available device", "");
								return InitialStatus.InitializeInputControlFailed;
								//return InitialStatus.NoAvailableDevice;
							}
							else
							{
								bool isHasOnlineDevice = false;
								foreach (var device in outputMessage)
								{
									string[] deviceInfo = device.Split(new string[] { "\t" }, StringSplitOptions.RemoveEmptyEntries);
									if (deviceInfo.Length == 1)
									{
										continue;
									}
									else
									{
										if (deviceInfo[1].Equals("device"))
										{
											isHasOnlineDevice = true;
										}
									}
								}

								if (!isHasOnlineDevice)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, "Warning: No any online device", "");
									return InitialStatus.InitializeInputControlFailed;
									//return InitialStatus.NoOnlineDevice;
								}
							}
						}
					}

					#endregion The ADB commmands for checking available device if input daemon is not running

					#region Run VNC before input controls

					// try to connect VNC
					for (int i = 0; i < Configs.RetryTimesForInputControl; i++)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Try to connect to server for VNC, time: " + (i + 1), "");

						if (null == VNCController)
						{
							VNCController = new VNCController(deviceinfo.ServerIP, deviceinfo.ServerVNCPort, deviceinfo.ServerVNCPassword, Configs.DownloadFileFolderPath);
						}

						initializeResult = VNCController.InitializeVNC();
						if (VNCController.IsInitializationSuccess && VNCController.IsRunSuccess && initializeResult == InitialStatus.ConnectToInputControlCompleted)
						{
							break;
						}

						Thread.Sleep(Configs.RetryTimePeriodForInputControl);
					}

					if (initializeResult != InitialStatus.ConnectToInputControlCompleted)
					{
						return initializeResult;
					}

					#endregion Run VNC before input controls

					#region The ADB commands for input control

					try
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call GetInputDaemonStatusAsync()", "");
						isInputDaemonAlreadyRunning = await this.GetInputDaemonStatusAsync(deviceinfo.Device).ConfigureAwait(false);
					}
					catch (Exception ex)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
					}

					LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>> At beginning, isInputDaemonRunning: " + isInputDaemonAlreadyRunning, "");

					// Try to run input daemon with the specific times
					for (int i = 0; i < Configs.RetryTimesForRunInputDaemon; ++i)
					{
						if (!isInputDaemonAlreadyRunning)
						{
							// Run the adb command for pull input daemon file
							++commandCounter;
							_runInputDaemonProcess.StartInfo.Arguments = Configs.PullInputDaemonCommand ?? String.Empty;
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "StartInfo.Arguments: " + _runInputDaemonProcess.StartInfo.Arguments, "");

							_runInputDaemonProcess.Start();
							outputMessage = _runInputDaemonProcess.StandardOutput.ReadToEnd().Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
							errorMessage = _runInputDaemonProcess.StandardError.ReadToEnd().Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
							_runInputDaemonProcess.WaitForExit();
							if (errorMessage.Length != 0)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Error in command " + commandCounter + " : run the adb command for pull input daemon file failed", "");
								foreach (var error in errorMessage)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, error, "");
									// The possible error will be incorrect folder name, the specific folder length is bigger than 60.
									if (error.Length > 60)
									{
										return InitialStatus.InitializeInputControlFailed;
									}
								}
							}
							else
							{
								if (outputMessage.Length != 0)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, outputMessage[0], "");
								}
							}

							// Run the adb command for pushing input daemon file
							++commandCounter;
							_runInputDaemonProcess.StartInfo.Arguments = Configs.PushInputDaemonCommand ?? String.Empty;
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "StartInfo.Arguments: " + _runInputDaemonProcess.StartInfo.Arguments, "");

							_runInputDaemonProcess.Start();
							outputMessage = _runInputDaemonProcess.StandardOutput.ReadToEnd().Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
							errorMessage = _runInputDaemonProcess.StandardError.ReadToEnd().Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
							_runInputDaemonProcess.WaitForExit();
							if (errorMessage.Length != 0)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Error in command " + commandCounter + " : run the adb command for pushing input daemon file failed", "");
								foreach (var error in errorMessage)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, error, "");
									// The possible error will be incorrect folder name, the specific folder length is bigger than 60.
									if (error.Length > 60)
									{
										return InitialStatus.InitializeInputControlFailed;
									}
								}
							}
							else
							{
								if (outputMessage.Length != 0)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, outputMessage[0], "");
								}
							}

							// Run the adb command for shelling set privilege of input daemon file
							++commandCounter;
							_runInputDaemonProcess.StartInfo.Arguments = Configs.ShellSetPrivilegeInputDaemonCommand ?? String.Empty;
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "StartInfo.Arguments: " + _runInputDaemonProcess.StartInfo.Arguments, "");

							_runInputDaemonProcess.Start();
							outputMessage = _runInputDaemonProcess.StandardOutput.ReadToEnd().Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
							errorMessage = _runInputDaemonProcess.StandardError.ReadToEnd().Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
							_runInputDaemonProcess.WaitForExit();
							if (errorMessage.Length != 0)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Error in command " + commandCounter + " : run the adb command for shelling set privilege of input daemon file failed", "");
								foreach (var error in errorMessage)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, error, "");
								}

								return InitialStatus.InitializeInputControlFailed;
							}
							else
							{
								if (outputMessage.Length != 0)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, outputMessage[0], "");
								}
							}

							// Run the adb command for shelling input daemon file
							++commandCounter;
							_runInputDaemonProcess.StartInfo.Arguments = Configs.ShellInputDaemonCommand ?? String.Empty;
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "StartInfo.Arguments: " + _runInputDaemonProcess.StartInfo.Arguments, "");

							_runInputDaemonProcess.Start();

							// Wait a while
							Thread.Sleep(Configs.RetryTimePeriodForWaitInputDaemonReady);
							try
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call GetInputDaemonStatusAsync()", "");
								isInputDaemonAlreadyRunning = await this.GetInputDaemonStatusAsync(deviceinfo.Device).ConfigureAwait(false);
							}
							catch (Exception ex)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
							}

							LogtoFile.WriteLog("", Configs.DebugLevelValue, "<<<<< After running input daemon, isInputDaemonRunning: " + isInputDaemonAlreadyRunning, "");
							if (!isInputDaemonAlreadyRunning)
							{
								try
								{
									_runInputDaemonProcess.Kill();
								}
								catch { }
							}
						}
						else
						{
							// If the input daemon is running, to exit the retry loop
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "input daemon is already running", "");
							break;
						}
					}

					if (!isInputDaemonAlreadyRunning)
					{
						return InitialStatus.InitializeInputControlFailed;
					}

					#endregion The ADB commands for input control

					#region The ADB commands for screen casting

					DaemonStatus screenCastDaemonStatus = DaemonStatus.NotRunning;

					try
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call GetScreencastDaemonStatusAsync", "");
						screenCastDaemonStatus = await this.GetScreencastDaemonStatusAsync(deviceinfo.Device).ConfigureAwait(false);
					}
					catch (Exception ex)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
					}

					LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>> At beginning, screenCastDaemonStatus: " + screenCastDaemonStatus, "");

					// Try to run screen cast daemon with the specific times
					for (int i = 0; i < Configs.RetryTimesForRunScreencastingDaemon; ++i)
					{
						if (screenCastDaemonStatus == DaemonStatus.NotRunning)
						{
							// Run the adb command for pull screen casting daemon file
							++commandCounter;
							_runScreenCastingDaemonProcess.StartInfo.Arguments = Configs.PullScreenCastingCommand ?? String.Empty;
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "StartInfo.Arguments: " + _runScreenCastingDaemonProcess.StartInfo.Arguments, "");

							_runScreenCastingDaemonProcess.Start();
							outputMessage = _runScreenCastingDaemonProcess.StandardOutput.ReadToEnd().Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
							errorMessage = _runScreenCastingDaemonProcess.StandardError.ReadToEnd().Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
							_runScreenCastingDaemonProcess.WaitForExit();
							if (errorMessage.Length != 0)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Error in command " + commandCounter + " : run the adb command for pull input daemon file failed", "");
								foreach (var error in errorMessage)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, error, "");
									// The possible error will be incorrect folder name, the specific folder length is bigger than 60.
									if (error.Length > 60)
									{
										return InitialStatus.InitializeInputControlFailed;
									}
								}
							}
							else
							{
								if (outputMessage.Length != 0)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, outputMessage[0], "");
								}
							}

							// Run the adb command for pushing screen casting daemon file
							++commandCounter;
							_runScreenCastingDaemonProcess.StartInfo.Arguments = Configs.PushScreenCastingCommand ?? String.Empty;
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "StartInfo.Arguments: " + _runScreenCastingDaemonProcess.StartInfo.Arguments, "");

							_runScreenCastingDaemonProcess.Start();
							outputMessage = _runScreenCastingDaemonProcess.StandardOutput.ReadToEnd().Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
							errorMessage = _runScreenCastingDaemonProcess.StandardError.ReadToEnd().Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
							_runScreenCastingDaemonProcess.WaitForExit();
							if (errorMessage.Length != 0)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Error in command " + commandCounter + " : run the adb command for pushing input daemon file failed", "");
								foreach (var error in errorMessage)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, error, "");
									// The possible error will be incorrect folder name, the specific folder length is bigger than 60.
									if (error.Length > 60)
									{
										return InitialStatus.InitializeInputControlFailed;
									}
								}
							}
							else
							{
								if (outputMessage.Length != 0)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, outputMessage[0], "");
								}
							}

							// Run the adb command for shelling set privilege of screen casting daemon file
							++commandCounter;
							_runScreenCastingDaemonProcess.StartInfo.Arguments = Configs.ShellSetPrivilegeScreenCastingDaemonCommand ?? String.Empty;
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "StartInfo.Arguments: " + _runScreenCastingDaemonProcess.StartInfo.Arguments, "");

							_runScreenCastingDaemonProcess.Start();
							outputMessage = _runScreenCastingDaemonProcess.StandardOutput.ReadToEnd().Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
							errorMessage = _runScreenCastingDaemonProcess.StandardError.ReadToEnd().Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
							_runScreenCastingDaemonProcess.WaitForExit();
							if (errorMessage.Length != 0)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Error in command " + commandCounter + " : run the adb command for shelling set privilege of screen casting daemon file failed", "");
								foreach (var error in errorMessage)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, error, "");
								}

								return InitialStatus.InitializeInputControlFailed;
							}
							else
							{
								if (outputMessage.Length != 0)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, outputMessage[0], "");
								}
							}

							// Run the adb command for shelling screen casting daemon file
							++commandCounter;
							_runScreenCastingDaemonProcess.StartInfo.Arguments = Configs.ShellscreenCastingDaemonCommand ?? String.Empty;
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "StartInfo.Arguments: " + _runScreenCastingDaemonProcess.StartInfo.Arguments, "");

							_runScreenCastingDaemonProcess.Start();

							// Wait a while
							Thread.Sleep(Configs.RetryTimePeriodForWaitScreencastingDaemonReady);
							try
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call GetScreencastDaemonStatusAsync", "");
								screenCastDaemonStatus = await this.GetScreencastDaemonStatusAsync(deviceinfo.Device).ConfigureAwait(false);
							}
							catch (Exception ex)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
							}

							LogtoFile.WriteLog("", Configs.DebugLevelValue, "<<<<< After running screen castin daemon, screenCastDaemonStatus: " + screenCastDaemonStatus, "");
							if (screenCastDaemonStatus != DaemonStatus.NotRunning)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "screen casting daemon is already running", "");
							}
							else
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "screen casting daemon is not already running", "");
								try
								{
									_runScreenCastingDaemonProcess.Kill();
								}
								catch { }
							}
						}
						else
						{
							// If the screen casting daemon is running, to exit the retry loop
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "screen casting daemon is already running", "");
							break;
						}
					}

					if (screenCastDaemonStatus == DaemonStatus.NotRunning ||
						screenCastDaemonStatus == DaemonStatus.Unknown)
					{
						return InitialStatus.InitializeInputControlFailed;
					}

					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start to call ControlScreencast", "");
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "ConnectionType: " + type.GetStringValue(), "");

					//0511 Johnson modify it to check play screen cast resoltuion
					if (!CheckScreencastResoultion(deviceinfo, ref _mScreenCastWidth, ref _mScreenCastHeight))
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Check Screen Cast value fail. reset to default screen cast value 720p", "");
					}
					this.SetCurrentVideoResolution(_mScreenCastWidth, _mScreenCastHeight);

					ScreencastControlInfo control = new ScreencastControlInfo()
					{
						DestinationPort = (ushort)deviceinfo.ServerRTPPort,
						Protocol = type == ConnectionType.UDP ? ScreencastControlInfo.ConnectionProtocol.Udp : ScreencastControlInfo.ConnectionProtocol.Tcp,
						Resolution = new ScreencastControlInfo.ResolutionInfo
						{
							Width = _mScreenCastWidth,
							Height = _mScreenCastHeight,
						},
						KeyFrameInterval = _mKeyFrameInterval,
						Bitrate = _mBitRate
					};

					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Set ScreencastControlInfo width:" + _mScreenCastWidth.ToString() + " Height:" + _mScreenCastHeight.ToString(), "");

					LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call play async", "");

					InitialStatus playScreencastStatus = InitialStatus.PlayScreencastError;

					for (int i = 0; i < Configs.RetryTimesForPlayScreencast; i++)
					{
						this.SendAppEventAsync(new AppEventArgs(AppEventType.InitialStatus, InitialStatus.InitializePlayScreencast, i));

						LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call play async", "");
						playScreencastStatus = await this.PlayScreenCastingDaemonAsync(deviceinfo.Device, control).ConfigureAwait(false);

						if (playScreencastStatus == InitialStatus.PlayScreencastSuccess)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call play Screencast Success", "");
							return InitialStatus.InitailizationsCompleted;
						}
						else if (playScreencastStatus == InitialStatus.PlayScreencastTimeout)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call play Screencast Timeout", "");
							playScreencastStatus = InitialStatus.PlayScreencastTimeout;
						}
						else if (playScreencastStatus == InitialStatus.PlayScreencastError)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call play Screencast Error", "");
							playScreencastStatus = InitialStatus.PlayScreencastError;
							break;
						}
					}

					return playScreencastStatus;

					#endregion The ADB commands for screen casting
				}
				catch (Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}

			return InitialStatus.InitailizationsCompleted;
		}

		private async Task<InitialStatus> ConnectingWorkWithoutMiracast(AppDeviceInfo deviceinfo, ConnectionType type)
		{
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start", "");

			try
			{
				// Do adb input control initialization work
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Initialize input controls", "");
				InitialStatus initializeResult = InitialStatus.InitializeInputControl;
				this.SendAppEventAsync(new AppEventArgs(AppEventType.InitialStatus, initializeResult));

				//Try to read registry width and height
				try
				{
					GetRegistryVideoSizeProtocol(deviceinfo.Device, ref _mScreenCastWidth, ref _mScreenCastHeight);

					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Registry assign Screen Cast Width: " + _mScreenCastWidth, "");
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Registry assign Screen Cast Height: " + _mScreenCastHeight, "");
				}
				catch
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "GetCurrentVideoResolution() failed!", "");
				}

				try
				{
					//try
					//{
					//	LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call GetInputDaemonStatusAsync()", "");
					//	isInputDaemonAlreadyRunning = await this.GetInputDaemonStatusAsync(deviceinfo.Device).ConfigureAwait(false);
					//}
					//catch (Exception ex)
					//{
					//	LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
					//}

					//LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>> Before running VNC, isInputDaemonRunning: " + isInputDaemonAlreadyRunning, "");

					//try
					//{
					//	LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call GetInputDaemonStatusAsync()", "");
					//	isInputDaemonAlreadyRunning = await this.GetInputDaemonStatusAsync(deviceinfo.Device).ConfigureAwait(false);
					//}
					//catch (Exception ex)
					//{
					//	LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
					//}

					//LogtoFile.WriteLog("", Configs.DebugLevelValue, "<<<<<< After running VNC, isInputDaemonRunning: " + isInputDaemonAlreadyRunning, "");

					#region Connect to Remote device VNC

					// try to connect VNC
					for (int i = 0; i < Configs.RetryTimesForInputControl; i++)
					{
						this.SendAppEventAsync(new AppEventArgs(AppEventType.InitialStatus, InitialStatus.InitializeConnectTOVNC, i));

						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Try to connect to server for VNC, time: " + i, "");

						if (null == VNCController)
						{
							VNCController = new VNCController(deviceinfo.ServerIP, deviceinfo.ServerVNCPort, deviceinfo.ServerVNCPassword, Configs.DownloadFileFolderPath);
						}

						initializeResult = VNCController.InitializeVNC();
						if (VNCController.IsInitializationSuccess && VNCController.IsRunSuccess && initializeResult == InitialStatus.ConnectToInputControlCompleted)
						{
							break;
						}

						Thread.Sleep(Configs.RetryTimePeriodForInputControl);
					}

					if (initializeResult != InitialStatus.ConnectToInputControlCompleted)
					{
						//0420 Johnson modify for debug use
						//return initializeResult;
						return InitialStatus.InitializeConnectToVNCFail;
					}

					#endregion Connect to Remote device VNC

					#region Check Input Daemon Status

					//0514 Johnon add for record input daemon status
					InitialStatus inputDaemonStatusCheck = InitialStatus.InputDaemonNotRunning;
					bool isInputDaemonAlreadyRunning = false;
					// Try to check input daemon status with the specific times
					for (int i = 0; i < Configs.RetryTimesForCheckInputDaemonStatus; i++)
					{
						this.SendAppEventAsync(new AppEventArgs(AppEventType.InitialStatus, InitialStatus.InitializeCheckVNCDaemonStatus, i));

						try
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call GetInputDaemonStatusAsync()", "");
							isInputDaemonAlreadyRunning = await this.GetInputDaemonStatusAsync(deviceinfo.Device).ConfigureAwait(false);
						}
						catch (TimeoutException)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call GetInputDaemonStatusAsync() Timeout", "");
							inputDaemonStatusCheck = InitialStatus.InitializeInputControlTimeout;
						}
						catch (WebServiceException Wex)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call GetInputDaemonStatusAsync() WebServiceException" + Wex.ToString(), "");
							inputDaemonStatusCheck = InitialStatus.InitializeInputControlWebServiceException;
							break;
						}
						catch (Exception ex)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
						}

						if (isInputDaemonAlreadyRunning)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "input daemon is already running", "");
							break;
						}
						else
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "input daemon is not running, wait 1s", "");
							Thread.Sleep(Configs.RetryTimePeriodForWaitInputDaemonReady);
						}

						//if (!isInputDaemonAlreadyRunning)
						//{
						//	// Wait a while
						//	Thread.Sleep(Configs.RetryTimePeriodForWaitInputDaemonReady);
						//	try
						//	{
						//		LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call GetInputDaemonStatusAsync()", "");
						//		isInputDaemonAlreadyRunning = await this.GetInputDaemonStatusAsync(deviceinfo.Device).ConfigureAwait(false);
						//	}
						//	catch (Exception ex)
						//	{
						//		LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
						//	}

						//	LogtoFile.WriteLog("", Configs.DebugLevelValue, "<<<<<< After checking status, isInputDaemonRunning: " + isInputDaemonAlreadyRunning, "");
						//}
						//else
						//{
						//	// If the input daemon is running, to exit the retry loop
						//	LogtoFile.WriteLog("", Configs.DebugLevelValue, "input daemon is already running", "");
						//	break;
						//}
					}

					if (!isInputDaemonAlreadyRunning)
					{
						//0420 Johnson Modify for debug use
						//return InitialStatus.InitializeInputControlFailed;

						return inputDaemonStatusCheck;
						//return InitialStatus.InitializeVNCDaemonFail;
					}

					#endregion Check Input Daemon Status

					#region Check screen casting Daemon Status

					DaemonStatus screenCastDaemonStatus = DaemonStatus.NotRunning;
					InitialStatus screencastDaemonStatusCheck = InitialStatus.InitializeScreenCastDaemonFail;
					// Try to run screen cast daemon with the specific times
					for (int i = 0; i < Configs.RetryTimesForRunScreencastingDaemon; i++)
					{
						this.SendAppEventAsync(new AppEventArgs(AppEventType.InitialStatus, InitialStatus.InitializeCheckScreencastDaemonStatus, i));

						try
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call GetScreencastDaemonStatusAsync", "");
							screenCastDaemonStatus = await this.GetScreencastDaemonStatusAsync(deviceinfo.Device).ConfigureAwait(false);
						}
						catch (TimeoutException ex)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call GetScreencastDaemonStatusAsync TimeoutException" + ex.ToString(), "");
							screencastDaemonStatusCheck = InitialStatus.InitializeScreencastDaemonTimeout;
						}
						catch (WebServiceException ex)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call GetScreencastDaemonStatusAsync WebServiceException" + ex.ToString(), "");
							screencastDaemonStatusCheck = InitialStatus.InitializeScreencastDaemonWebServiceException;
							break;
						}
						catch (Exception ex)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
						}
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "<<<<< After checking screen castin daemon status, screenCastDaemonStatus: " + screenCastDaemonStatus, "");

						if (screenCastDaemonStatus != DaemonStatus.NotRunning)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "screen casting daemon is already running", "");
							break;
						}

						Thread.Sleep(Configs.RetryTimePeriodForWaitScreencastingDaemonReady);

						//if (screenCastDaemonStatus == DaemonStatus.NotRunning)
						//{
						//	// Wait a while
						//	Thread.Sleep(Configs.RetryTimePeriodForWaitScreenCastingDaemonReady);
						//	try
						//	{
						//		LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call GetScreencastDaemonStatusAsync", "");
						//		screenCastDaemonStatus = await this.GetScreencastDaemonStatusAsync(deviceinfo.Device).ConfigureAwait(false);
						//	}
						//	catch (Exception ex)
						//	{
						//		LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
						//	}

						//	LogtoFile.WriteLog("", Configs.DebugLevelValue, "<<<<< After checking screen castin daemon status, screenCastDaemonStatus: " + screenCastDaemonStatus, "");
						//}
						//else
						//{
						//	// If the screen casting daemon is running, to exit the retry loop
						//	LogtoFile.WriteLog("", Configs.DebugLevelValue, "screen casting daemon is already running", "");
						//	break;
						//}
					}

					if (screenCastDaemonStatus == DaemonStatus.NotRunning ||
						screenCastDaemonStatus == DaemonStatus.Unknown)
					{
						return screencastDaemonStatusCheck;
					}

					#endregion Check screen casting Daemon Status

					#region Add soure filter to graph and prepare to receiving data

					// Start receiving network package based on the specific protocol
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Send the event of start receiving", "");
					this.SendAppEventAsync(new AppEventArgs(AppEventType.StartReceiving, type));

					#endregion Add soure filter to graph and prepare to receiving data

					#region By WebService to send play screencast controlInfo

					//0511 Johnson modify it to check play screen cast resoltuion
					if (!CheckScreencastResoultion(deviceinfo, ref _mScreenCastWidth, ref _mScreenCastHeight))
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Check Screen Cast value fail. reset to default screen cast value 720p", "");
					}
					this.SetCurrentVideoResolution(_mScreenCastWidth, _mScreenCastHeight);

					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start to call ControlScreencast", "");
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "ConnectionType: " + type.GetStringValue(), "");

					ScreencastControlInfo control = new ScreencastControlInfo()
					{
						//DestinationIP = AFCommonUtils.GetLocalIpV4Address(), // this property is internal and is specified by AirFusionManager
						DestinationPort = (ushort)deviceinfo.ServerRTPPort,
						Protocol = type == ConnectionType.UDP ? ScreencastControlInfo.ConnectionProtocol.Udp : ScreencastControlInfo.ConnectionProtocol.Tcp,
						Resolution = new ScreencastControlInfo.ResolutionInfo
						{
							Width = _mScreenCastWidth,
							Height = _mScreenCastHeight,
						},
						KeyFrameInterval = _mKeyFrameInterval,
						Bitrate = _mBitRate
					};

					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Set ScreencastControlInfo width:" + _mScreenCastWidth.ToString() + " Height:" + _mScreenCastHeight.ToString(), "");

					InitialStatus playScreencastStatus = InitialStatus.PlayScreencastError;

					for (int i = 0; i < Configs.RetryTimesForPlayScreencast; i++)
					{
						this.SendAppEventAsync(new AppEventArgs(AppEventType.InitialStatus, InitialStatus.InitializePlayScreencast, i));

						LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call play async", "");
						playScreencastStatus = await this.PlayScreenCastingDaemonAsync(deviceinfo.Device, control).ConfigureAwait(false);

						if (playScreencastStatus == InitialStatus.PlayScreencastSuccess)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call play Screencast Success", "");
							return InitialStatus.InitailizationsCompleted;
						}
						else if (playScreencastStatus == InitialStatus.PlayScreencastTimeout)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call play Screencast Timeout", "");
							playScreencastStatus = InitialStatus.PlayScreencastTimeout;
						}
						else if (playScreencastStatus == InitialStatus.PlayScreencastError)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call play Screencast Error", "");
							playScreencastStatus = InitialStatus.PlayScreencastError;
							break;
						}

						Thread.Sleep(Configs.RetryTimePeriodForWaitPlayScreencastingDaemonReady);
					}

					return playScreencastStatus;

					#endregion By WebService to send play screencast controlInfo
				}
				catch (Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
					return InitialStatus.InitializeInputControlFailed;
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				return InitialStatus.InitializeInputControlFailed;
			}
			return InitialStatus.InitailizationsCompleted;
		}

		private async Task<DeviceInfoStatus> DoWorkBeforeConnection(AppDeviceInfo deviceInfo)
		{
			DeviceInfoStatus result = DeviceInfoStatus.Unconnectable;

			this.SendAppEventAsync(new AppEventArgs(AppEventType.InitialStatus, InitialStatus.PhoneWakeUp));

			#region create task for device wakeup by origin IP and Reconfirm IP(By network or bluetooth update)

			try
			{
				List<Task<DeviceInfoStatus>> tasks = new List<Task<DeviceInfoStatus>>();
				tasks.Add(DoWakeUpBeforeConnection(deviceInfo, true));
				tasks.Add(DoWakeUpBeforeConnection(deviceInfo, false));

				while (true)
				{
					if (tasks.Count == 0)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "No any task need to be waited!!", "");
						break;
					}

					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Wait for any task completed", "");

					var task = await Task.WhenAny(tasks);//.WaitAny(tasks.ToArray());
					//var target = tasks.ToArray()[index];

					result = task.Result;
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Get result: " + result, "");

					tasks.Remove(task);

					//if (result == DeviceInfoStatus.Connectable || result == DeviceInfoStatus.Unpair)
					if (result == DeviceInfoStatus.WakeUpSuccess)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "WakeUp Success no need to wait any task", "");
						//LogtoFile.WriteLog("", Configs.DebugLevelValue, "No need to wait last task", "");
						break;
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Wake up fail, need to wait last task", "");
						//LogtoFile.WriteLog("", Configs.DebugLevelValue, "Need to wait last task", "");
					}
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				return result;
			}

			#endregion create task for device wakeup by origin IP and Reconfirm IP(By network or bluetooth update)

			#region After Wakeup phone, need to check Device Status and information

			if (result == DeviceInfoStatus.WakeUpSuccess)
			{
				#region Get DeviceStatus

				for (int i = 0; i < 3; i++)
				{
					// To get remote status
					this.SendAppEventAsync(new AppEventArgs(AppEventType.InitialStatus, InitialStatus.GetDeviceStatus, i));
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Call RequestPairingStatus()", "");
					result = await this.CheckRemoteDeviceStatus(deviceInfo.Device).ConfigureAwait(false);
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "status: " + result, "");

					if (result == DeviceInfoStatus.Connectable)
						break;
				}

				if (result != DeviceInfoStatus.Connectable)
				{
					return result;
				}

				#endregion Get DeviceStatus

				#region Get DeviceInformation

				// To request information with blocking code
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Call RequestInformation()", "");
				try
				{
					for (int i = 0; i < 3; i++)
					{
						this.SendAppEventAsync(new AppEventArgs(AppEventType.InitialStatus, InitialStatus.GetDeviceInformation, i));

						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Call RequestInformationAsync()", "");
						result = await this.RequestInformationAsync(deviceInfo.Device).ConfigureAwait(false);
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "status: " + result, "");

						if (result == DeviceInfoStatus.Connectable)
							return DeviceInfoStatus.Connectable;

						//if (await this.RequestInformationAsync(deviceInfo.Device).ConfigureAwait(false))
						//{
						//	LogtoFile.WriteLog("", Configs.DebugLevelValue, "Call RequestInformation() Success", "");
						//	return DeviceInfoStatus.Connectable;
						//}
					}
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "End call RequestInformation()", "");
					return result;
				}
				catch (Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "deviceInfo.Device.RequestInformation() error: " + ex.ToString(), "");
					return DeviceInfoStatus.GetDeviceInfoFail;
				}
				finally
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "End call RequestInformation()", "");
				}

				#endregion Get DeviceInformation
			}
			else
			{
				return result;
			}

			#endregion After Wakeup phone, need to check Device Status and information
		}

		private async Task<DeviceInfoStatus> DoWakeUpBeforeConnection(AppDeviceInfo deviceInfo, bool isDoReconfirmIP)
		{
			//Reconfirm IP address by WebService

			#region Reconfirm IP Address

			if (isDoReconfirmIP)
			{
				ReconfirmAddressResult reconfirmIPResult = null;

				try
				{
					reconfirmIPResult = await this.ReconfirmAddressAsync(deviceInfo.Device).ConfigureAwait(false);

					if (reconfirmIPResult != null)
					{
						deviceInfo.ReconfirmAddressResult = reconfirmIPResult;

						if (reconfirmIPResult.Addresses == null)
						{
							return DeviceInfoStatus.DandelionFail;
						}
						else if (reconfirmIPResult.Addresses.Count == 0)
						{
							return DeviceInfoStatus.DandelionFail;
						}

						if (reconfirmIPResult.FirstConnectableAddress != null)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> reconfirmed IP: " + reconfirmIPResult.FirstConnectableAddress, "");

							LogtoFile.WriteLog("", Configs.DebugLevelValue, "deviceInfo.ServerIP: " + deviceInfo.ServerIP, "");
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "deviceInfo.LastServerIP: " + deviceInfo.LastServerIP, "");

							if (!deviceInfo.ServerIP.Equals(deviceInfo.LastServerIP))
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "deviceInfo.ServerIP is different with deviceInfo.LastServerIP", "");
								deviceInfo.LastServerIP = reconfirmIPResult.FirstConnectableAddress.ToString();
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Update LastServerIP", "");
							}
							else
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "deviceInfo.ServerIP is the same as deviceInfo.LastServerIP", "");
							}
						}
						else
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> reconfirmIPResult.FirstConnectableAddress value is null", "");
							return DeviceInfoStatus.ConfirmIPFail;
						}
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> reconfirmIPResult value is null", "");
						Debugger.Break();
						return DeviceInfoStatus.ConfirmIPFail;
					}
				}
				catch (Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
					return DeviceInfoStatus.ConfirmIPFail;
				}
			}

			#endregion Reconfirm IP Address

			// Do wake up

			#region WakeUp Device

			bool isWakeUpSuccess = false;
			if (null != deviceInfo.Device)
			{
				try
				{
					isWakeUpSuccess = await deviceInfo.Device.WakeUpAsync().ConfigureAwait(false);
				}
				catch (Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				}
			}
			else
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "deviceInfo.Device is null", "");
			}

			if (!isWakeUpSuccess)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Failed to wake up !", "");

				//0412 Johnson add, when wakeup faile, we should not to do anything.
				return DeviceInfoStatus.WakeUpFail;
			}
			else
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Wake up successfully!", "");
				return DeviceInfoStatus.WakeUpSuccess;
			}

			#endregion WakeUp Device
		}

		private async Task<DeviceInfoStatus> CheckRemoteDeviceStatus(AFUserDevice device)
		{
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Device name: " + device.DeviceName, "");

			// If the device isn't connectable
			try
			{
				AFUserDevice.PairingStatus originalPairingStatus = device.PairStatus;
				AFUserDevice.PairingStatus newPairingStatus = await this.RequestPairingStatusAsync(device).ConfigureAwait(false);  // Request remote status

				if (newPairingStatus == AFUserDevice.PairingStatus.UnPaired)
				{
					return DeviceInfoStatus.Unpair;
				}
				else
				{
					return DeviceInfoStatus.Connectable;
				}
			}
			catch (TimeoutException)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "device.RequestPairingStatus() Timeout", "");
				return DeviceInfoStatus.CheckDeviceStatusTimeout;
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "device.RequestPairingStatus() error: " + ex.ToString(), "");
				return DeviceInfoStatus.CheckDeviceStatusFail;
			}
		}

		private void ClearAllConnection(bool isFromApplicationExit = false)
		{
			try
			{
				//Johnson 0416 Add for debug Close window deadlock issue
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Enter ClearAllConnection and prepare to stop remote ADB Screen Casting", "");

				#region Stop remote ADB screen casting

				if (null != this.ConnectingDeviceInfo && null != this.ConnectingDeviceInfo.Device)
				{
					//Johnson 0416 Add for debug Close window deadlock issue
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "ClearAllConnection Call DoStopScreenCastingDaemonAsync", "");

					// Call DoStopScreenCastingDaemonAsync
					this.DoStopScreenCastingDaemonAsync(this.ConnectingDeviceInfo.Device).ContinueWith((a) =>
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> DoStopScreenCastingDaemonAsync.ContinueWith", "");

						#region Close connections

						// Clear the current device information
						this.ConnectingDeviceInfo = null;
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Set ConnectingDeviceInfo to null", "");

						if (isFromApplicationExit)
						{
							//Johnson 0416 Add for debug Close window deadlock issue
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "ClearAllConnection to AbortConnectingThread", "");
							this.AbortConnectingThread();
						}

						//Johnson 0416 Add for debug Close window deadlock issue
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "ClearAllConnection to CloseConnection", "");
						this.CloseConnection();

						// If the request is from application exit, to call the application.Shutdown()
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "isFromApplicationExit: " + isFromApplicationExit, "");
						if (isFromApplicationExit)
						{
							this.Dispatcher.Invoke(delegate()
							{
								try
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, "Call this.Shutdown()", "");
									this.Shutdown();
								}
								catch (Exception ex)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
								}
							});
						}

						LogtoFile.WriteLog("", Configs.DebugLevelValue, "<<<<<< DoStopScreenCastingDaemonAsync.ContinueWith", "");

						#endregion Close connections
					}).ConfigureAwait(false);
				}
				else
				{
					#region Close connections

					// Clear the current device information
					this.ConnectingDeviceInfo = null;
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Set ConnectingDeviceInfo to null", "");

					if (isFromApplicationExit)
					{
						//Johnson 0416 Add for debug Close window deadlock issue
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "ClearAllConnection to AbortConnectingThread", "");
						this.AbortConnectingThread();
					}
					//Johnson 0416 Add for debug Close window deadlock issue
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "ClearAllConnection to CloseConnection", "");
					this.CloseConnection();

					// If the request is from application exit, to call the application.Shutdown()
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "isFromApplicationExit: " + isFromApplicationExit, "");
					if (isFromApplicationExit)
					{
						this.Dispatcher.Invoke(delegate()
						{
							try
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Call this.Shutdown()", "");
								this.Shutdown();
							}
							catch (Exception ex)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
							}
						});
					}

					#endregion Close connections
				}

				#endregion Stop remote ADB screen casting
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		private void CloseConnection()
		{
			try
			{
				if (null != this._appScreenWindow && null != this._appScreenWindow.dsElement)
				{
					//Johnson 0416 Add for debug Close window deadlock issue
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "CloseConnection dsElement Close Filter", "");

					this._appScreenWindow.dsElement.Close();
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}

			//Johnson 0416 Add for debug Close window deadlock issue
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "CloseConnection Stop the timer of receiving data", "");

			// Stop the timer of receiving data
			try
			{
				this.WaitForReceiveDataTimer.Stop();
				this._signalOfReceiveDataTimerStarted.Reset();
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}

			try
			{
				// Close socket
				if (null != this.RTSPController)
				{
					if (this.RTSPController.IsConnected)
					{
						//this.RTSPController.TearDown();
					}

					this.RTSPController.CloseSocket();
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Clear RTSPController", "");
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
			finally
			{
				this.RTSPController = null;
			}

			try
			{
				//Johnson 0416 Add for debug Close window deadlock issue
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "CloseConnection Close VNC", "");

				// Close VNC
				if (null != this.VNCController)
				{
					if (null != this.VNCController.VNCClient)
					{
						this.VNCController.VNCClient.CloseVNC();
					}

					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Clear VNCController", "");
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
			finally
			{
				this.VNCController = null;
			}

			this.KillADBProcesses();
		}

		private void AbortConnectingThread()
		{
			try
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Call _tokenSource cancel", "");
				if (null != _tokenSource)
				{
					try
					{
						_tokenSource.Cancel();
					}
					catch
					{ }
				}

				if (null != _doWorkerThread)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Call do work thead abort", "");
					try
					{
						_doWorkerThread.Abort();
					}
					catch
					{ }
				}

				_doWorkerThread = null;
				this._connectingWorker = null;

				//Johnson 0416 Add for debug Close window deadlock issue
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Exit AbortConnectingThread", "");
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		private void KillADBProcesses()
		{
			try
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Kill input daemon processes", "");

				// Kill the ADB process
				if (null != _runInputDaemonProcess)
				{
					_runInputDaemonProcess.Kill();
				}
			}
			catch { }
			finally
			{
				_runInputDaemonProcess = null;
			}

			try
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Kill screen casting processes", "");

				if (null != this._runScreenCastingDaemonProcess)
				{
					this._runScreenCastingDaemonProcess.Kill();
				}
			}
			catch { }
			finally
			{
				this._runScreenCastingDaemonProcess = null;
			}

			try
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start Kill root adb processes", "");

				// Find the processes with the specific name
				var targetProcesses = Process.GetProcesses().Where(pr => pr.ProcessName == Configs.ADBProcessName);
				foreach (Process process in targetProcesses)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "UserName: " + process.StartInfo.UserName, "");
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "FileName: " + process.StartInfo.FileName, "");
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "WorkingDirectory: " + process.StartInfo.WorkingDirectory, "");

					try
					{
						if (!process.HasExited)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Process is not exited, to kill", "");
							process.Kill();
						}
						else
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Process is exited", "");
						}
					}
					catch (Exception ex)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
					}
				}

				LogtoFile.WriteLog("", Configs.DebugLevelValue, "End Kill root adb processes", "");
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		private void DeviceUnpairedWork(AFUserDevice unpairedDevice)
		{
			try
			{
				// Check if the device is existed in UI paired list
				if (this.PairedDevices.Count() != 0)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.PairedDevices.Count() != 0", "");

					AppDeviceInfo deviceInfo = this.PairedDevices.Where(x => x.Device.InstanceID.Equals(unpairedDevice.InstanceID)).FirstOrDefault();

					// To inform UI
					if (this.ConnectingDeviceInfo == null || unpairedDevice.InstanceID.Equals(this.ConnectingDeviceInfo.Device.InstanceID))
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Unpaired device is screencasting, need to close window", "");
						this.SendAppEventAsync(new AppEventArgs(AppEventType.Unpair, unpairedDevice));
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Unpaired device is not screencasting, don't close window", "");
					}

					if (null != deviceInfo)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Find unpaired device, to remove it from list", "");

						this.Dispatcher.Invoke(delegate()
						{
							this.PairedDevices.Remove(deviceInfo);
						});
					}

					// If last launched device ID is equal to unpair device ID, to remove record
					if (this.LastLaunchID.Equals(unpairedDevice.InstanceID))
					{
						this.DeleteLastLaunchDeviceID();
					}

					// To clear all settings
					RegistryUtility.DeleteRegistry(RegistryHive.CurrentUser, Configs.RegUISettingsPath, unpairedDevice.InstanceID.ToString(), RegistryView.Registry64, true);

					// 2015-05-20 Seoker: remove Auto-login functions since it's defeatured
					/*
					// To delete record for auto login
					if (null != _pipeManager)
					{
						this._pipeManager.DeleteALInfo(unpairedDevice.Bluetooth.Address);
					}
					*/
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.PairedDevices.Count() == 0", "");

					LogtoFile.WriteLog("", Configs.DebugLevelValue, "To clear last launched device ID data", "");

					// If last launched device ID is equal to unpair device ID, to remove record
					if (this.LastLaunchID.Equals(unpairedDevice.InstanceID))
					{
						this.DeleteLastLaunchDeviceID();
					}

					// To clear all settings
					RegistryUtility.DeleteRegistry(RegistryHive.CurrentUser, Configs.RegUISettingsPath, unpairedDevice.InstanceID.ToString(), RegistryView.Registry64, true);

					// 2015-05-20 Seoker: remove Auto-login functions since it's defeatured
					/*
					// To delete record for auto login
					if (null != _pipeManager)
					{
						this._pipeManager.DeleteALInfo(unpairedDevice.Bluetooth.Address);
					}
					*/
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		private object GetLockObject(object keyObject)
		{
			lock (_functionObjectLockObj)
			{
				if (_functionObjectLockObj.ContainsKey(keyObject))
				{
					return _functionObjectLockObj[keyObject];
				}
				else
				{
					object createdObject = new object();
					_functionObjectLockObj.Add(keyObject, createdObject);
					return createdObject;
				}
			}
		}

		private PhoneSystemPermissionDetectionRule GetCurrentPhoneSystemPermissionDetectionRule()
		{
			// Get phone system permission detection rule from registry
			try
			{
				bool isX86;
				object oRK = RegistryUtility.GetRegistry(RegistryHive.CurrentUser, Configs.RegVideoPath, Configs.RegPhoneSystemPermissionDetectionRuleKey, RegistryView.Registry64, out isX86);

				if (oRK != null)
				{
					return (PhoneSystemPermissionDetectionRule)oRK;
				}
			}
			catch
			{
				return PhoneSystemPermissionDetectionRule.AutoDetect;
			}

			return PhoneSystemPermissionDetectionRule.AutoDetect;
		}

		private void SetCurrentVideoResolution(int videoWidth, int videoHeight)
		{
			try
			{
				Configs.ApplicationReference._appScreenWindow.dsElement.VideoWidth = videoWidth;
				Configs.ApplicationReference._appScreenWindow.dsElement.VideoHeight = videoHeight;
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		private void SaveCurrentDeviceInfo(AppDeviceInfo appdeviceInfo)
		{
			// Set the current device information for connecting
			this.ConnectingDeviceInfo = appdeviceInfo;

			//0511 Johnson add avoid connecting device info in null
			this._appScreenWindow.CurrentScreenCastdeviceInfo = appdeviceInfo;
		}

		private ConnectionType GetCurrentVideoProtocol()
		{
			// Get video protocol from registry
			try
			{
				bool isX86;
				object oRK = RegistryUtility.GetRegistry(RegistryHive.CurrentUser, Configs.RegVideoPath, Configs.RegVideoProtocolKey, RegistryView.Registry64, out isX86);

				if (oRK != null)
				{
					return (ConnectionType)oRK;
				}
			}
			catch
			{
				return ConnectionType.TCP;
			}

			return ConnectionType.TCP;
		}

		private void GetRegistryVideoSizeProtocol(AFUserDevice device, ref int videoWidth, ref int videoHeight)
		{
			//set default video quality value to 720p
			//videoWidth = device.Display.Width.Value;
			//videoHeight = device.Display.Height.Value;

			bool isX86;
			// Get video protocol from registry
			try
			{
				object oRwK = RegistryUtility.GetRegistry(RegistryHive.CurrentUser, Configs.RegVideoPath, Configs.RegVideoResolutionWidthKey, RegistryView.Registry64, out isX86);
				if (oRwK != null)
				{
					videoWidth = (int)oRwK;
				}
			}
			catch
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "GetRegistryVideoSizeProtocol width fail : can't get value from registry", "");
			}

			try
			{
				object oRhK = RegistryUtility.GetRegistry(RegistryHive.CurrentUser, Configs.RegVideoPath, Configs.RegVideoResolutionHeightKey, RegistryView.Registry64, out isX86);
				if (oRhK != null)
				{
					videoHeight = (int)oRhK;
				}
			}
			catch
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "GetRegistryVideoSizeProtocol Height fail : can't get value from registry", "");
			}
		}

		private bool CheckScreencastResoultion(AppDeviceInfo deviceInfo, ref int videoWidth, ref int videoHeight)
		{
			if (videoWidth != 0 && videoHeight != 0)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "CheckScreenCastResoultion: video Width and Height has been assign value, don't need to change.", "");
				return true;
			}
			else if (deviceInfo != null)
			{
				if (deviceInfo.Device.Services.Screencast.SupportedResolutions != null)
				{
					int _iDeviceSupportResolution = deviceInfo.Device.Services.Screencast.SupportedResolutions.Count;
					if (_iDeviceSupportResolution > 0)
					{
						videoWidth = deviceInfo.Device.Services.Screencast.SupportedResolutions[_iDeviceSupportResolution - 1].Width;
						videoHeight = deviceInfo.Device.Services.Screencast.SupportedResolutions[_iDeviceSupportResolution - 1].Height;
						return true;
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "CheckScreenCastResoultion: deviceInfo.Device.Services.Screencast.SupportedResolutions is null", "");
					}
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "CheckScreenCastResoultion: prepare set default width and height value", "");
				}
			}

			LogtoFile.WriteLog("", Configs.DebugLevelValue, "CheckScreenCastResoultion: prepare set default width and height value", "");

			int _iHeight = deviceInfo.Device.Display.Height > deviceInfo.Device.Display.Width ? deviceInfo.Device.Display.Height.Value : deviceInfo.Device.Display.Width.Value;
			int _iWidth = deviceInfo.Device.Display.Width < deviceInfo.Device.Display.Height ? deviceInfo.Device.Display.Width.Value : deviceInfo.Device.Display.Height.Value;

			//Set default to 720p
			videoWidth = 720;
			videoHeight = (720 * _iHeight) / _iWidth;
			return false;
		}

		//0609 Johnson modify method for avoid some potential issue
		private void CheckPairedDeviceList()
		{
			LogtoFile.TrackEnter();

			try
			{
				var pairedDevices = this._manager.UserDeviceManager.DeviceStore.GetPairedDevices().ToList();

				if (pairedDevices.Count() != 0)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Get paired devices", "");

					foreach (var device in pairedDevices)
					{
						#region double check Device info is right

						if (null != device && null != device.SessionID && null != device.Address)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Device name: " + device.DeviceName, "");

							try
							{
								AppDeviceInfo deviceInfo = InitialAppDeviceInfo(device);

								CheckRemoteDeviceStatus(device).ContinueWith(delegate(Task<DeviceInfoStatus> RemoteStatustask)
								{
									DeviceInfoStatus resultStatus = RemoteStatustask.Result;

									LogtoFile.WriteLog("", Configs.DebugLevelValue, "Device status: " + resultStatus.GetStringValue(), "");

									#region Handle Device status result

									///If the result value such as Paired, Timeout, Exception is do nothing.
									///If the result value is unpaired, need to consider the current connection device
									///if the current connection is unpaired, need to stop the connection
									if (resultStatus != DeviceInfoStatus.Unpair)
									{
										// After checking device, to add the device
										// in list if it's the paired device.
										this.Dispatcher.Invoke(delegate()
										{
											LogtoFile.WriteLog("", Configs.DebugLevelValue, "Add to UI paired device list", "");
											this.PairedDevices.Insert(0, deviceInfo);
										});
									}
									else
									{
										LogtoFile.WriteLog("", Configs.DebugLevelValue, "Device is unpaired in offline", "");

										if (null != this.ConnectingDeviceInfo && null != this.ConnectingDeviceInfo.Device)
										{
											if (this.ConnectingDeviceInfo.Device.InstanceID.Equals(device.InstanceID))
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.ConnectingDeviceInfo.Device.InstanceID = device.InstanceID, to send event", "");

												// If the devices are the same, to inform UI the unpaired device information.
												// This logic is to inform UI earlier for connecting device which is unpaired in offline.
												this.SendAppEventAsync(new AppEventArgs(AppEventType.PairAgain, device));
											}
											else
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.ConnectingDeviceInfo.Device.InstanceID != device.InstanceID, to do nothing", "");
											}
										}
										else
										{
											if (null == this._isStartConnection)
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.ConnectingDeviceInfo.Device is null", "");
											}
											else
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.ConnectingDeviceInfo.Device is null", "");
											}
										}
									}

									#endregion Handle Device status result
								}).ConfigureAwait(continueOnCapturedContext: false);
							}
							catch (Exception ex)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
							}
						}
						else///for debug use, record some error case
						{
							if (device == null)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Device is null", "");
							}
							else
							{
								if (device.SessionID == null)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, "Device SessionID is null", "");
								}

								if (device.Address == null)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, "Device Address is null", "");
								}

								device.UnPair();
							}
						}

						#endregion double check Device info is right
					}
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "No paired device, to start discovering", "");
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
			finally
			{
				LogtoFile.TrackLeave();
			}
		}

		//0609 Johnson add for Combine New AppDeviceInfo Method
		private AppDeviceInfo InitialAppDeviceInfo(AFUserDevice device)
		{
			LogtoFile.TrackEnter();
			AppDeviceInfo appdeviceInfo = null;
			try
			{
				appdeviceInfo = new AppDeviceInfo(this._generalNotificationManager)
				{
					Device = device,
					ServerRTSPPort = Configs.DefaultRTSPPort,
					ServerRTPPort = Configs.DefaultRTPPort,
					ServerVNCPort = Configs.DefaultVNCPort,
					ServerVNCPassword = System.Text.Encoding.ASCII.GetString(device.SessionID.GetByteArray()), // Configs.DefaultVNCPassword,
					LastServerIP = null != device.Address ? device.Address.ToString() : string.Empty,
					IsSendNotificationOn = AERegSetting.GetBoolSettings(Configs.RegNotificationKey, device.InstanceID.ToString(), true),

					// 2015-05-20 Seoker: remove Auto-login & incoming call functions since it's defeatured
					/*
					IsAnswerPhoneCallOn = this.GetBoolSettings(Configs.RegAnswerPhoneCallKey, device.InstanceID.ToString(), true),
					IsAutoUnlockOn = this.GetBoolSettings(Configs.RegUnlockKey, device.InstanceID.ToString(), false),
					LoginAccount = this.GetStringSettings(Configs.RegLoginAccountKey, device.InstanceID.ToString()),
					LoginPassword = this.GetStringSettings(Configs.RegLoginPWDKey, device.InstanceID.ToString()),
					*/
				};

				//For Save Current Connecting Device Info
				//this.ConnectingDeviceInfo = appdeviceInfo;

				LogtoFile.WriteLog("", Configs.DebugLevelValue, "VNC Password String is : " + appdeviceInfo.ServerVNCPassword, "");
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "InitialAppDeviceInfo has some exception : " + ex.ToString(), "");
			}
			finally
			{
				LogtoFile.TrackLeave();
			}
			return appdeviceInfo;
		}

		private string ReadingResoureceWording(object resourekey)
		{
			try
			{
				return Application.Current.TryFindResource(resourekey) as String;
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "ReadingResoureceWording exception : " + ex.ToString(), "");
				return string.Empty;
			}
		}

		#region Event Handle

		//0609 Johnon add for Notification Event Handle
		/// <summary>
		/// Notifications the event manager.
		/// </summary>
		/// ToDoIt Johnon need to modify it.
		private void NotificationEventManager()
		{
			#region For Notification

			#region NotificationEvent

			//Custom event register
			this._generalNotificationManager.NotificationEvent += delegate(object sender, ToastEventArgs e)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "e.EventType: " + e.EventType.GetStringValue(), "");
				switch (e.EventType)
				{
					case ToastEventType.ToastActivated:
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "e.Intent: " + e.Intent.GetStringValue(), "");
							if (e.Intent == ToastIntent.ShowScreenCasting)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "e.Intent == ToastIntent.ShowScreenCasting", "");

								if (null != e.Data)
								{
									string notificationKey = e.Data as string;

									LogtoFile.WriteLog("", Configs.DebugLevelValue, "notificationKey: " + notificationKey, "");

									if (null != e.Args)
									{
										try
										{
											Guid deviceInstanceID = (Guid)e.Args;
											if (null != deviceInstanceID)
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "deviceInstanceID: " + deviceInstanceID, "");

												AppDeviceInfo deviceInfo = this.PairedDevices.Where(x => x.Device.InstanceID.Equals(deviceInstanceID)).FirstOrDefault();

												if (null != deviceInfo)
												{
													if (null != this.ConnectingDeviceInfo)
													{
														if (this.ConnectingDeviceInfo.Device.InstanceID.Equals(deviceInfo.Device.InstanceID))
														{
															this.Dispatcher.Invoke(delegate()
															{
																if (null != this.AppScreenWindow)
																{
																	this.AppScreenWindow.WindowState = WindowState.Normal;
																	this.AppScreenWindow.Activate();
																}
																else
																{
																	LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.AppScreenWindow is null", "");
																}
															});
														}
														else
														{
															LogtoFile.WriteLog("", Configs.DebugLevelValue, "!this.ConnectingDeviceInfo.Device.InstanceID.Equals(deviceInfo.Device.InstanceID", "");

															//ToDoIt Need to Check the method is ok for Notification
															//this.StartConnectRemoteDevice(deviceInfo);
															this.Dispatcher.Invoke(delegate()
															{
																if (FlowControlManager.Instance.IsAcceptNewConnection())
																	this._appScreenWindow.ConnectNewScreencast(deviceInfo);
															});
															//this.InitialShowScreencastWindow();
															//this.RestartScreen(deviceInfo, true);
														}
													}
													else
													{
														LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.ConnectingDeviceInfo is null", "");
														//ToDoIt Need to Check the method is ok for Notification
														//this.StartConnectRemoteDevice(deviceInfo);
														this.Dispatcher.Invoke(delegate()
														{
															if (FlowControlManager.Instance.IsAcceptNewConnection())
																this._appScreenWindow.ConnectNewScreencast(deviceInfo);
														});
														//this.InitialShowScreencastWindow();
														//this.RestartScreen(deviceInfo, true);
													}
												}
												else
												{
													LogtoFile.WriteLog("", Configs.DebugLevelValue, "paired device info is null", "");
												}
											}
											else
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "device instance ID with Guid type is null", "");
											}
										}
										catch (Exception ex)
										{
											LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
										}
									}
									else
									{
										LogtoFile.WriteLog("", Configs.DebugLevelValue, "e.Args is null", "");
									}
								}
								else
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, "e.Data is null", "");
								}
							}
						}
						break;
				}
			};

			#endregion NotificationEvent

			#endregion For Notification
		}

		/// <summary>
		/// AirFusion WebService event manager.
		/// </summary>
		private void AirFusionEventManager()
		{
			#region For AFManager

			#region PairingRequested Event Handler

			/// When DeviceManager send Pairing Requested evnet, we need to check AcerEXTEND Device Manager
			/// about Pairing page has been show and no show any warning dialog. If all condiction is success,
			/// We need to return Pairing Success event to WebService.
			/// If can't do PairingRequested, we need to return PairingResponse(false) event to WebService.
			this._manager.UserDeviceManager.PairingRequested += async delegate(object sender, PairingRequestedEventArgs eventArgs)
			{
				LogtoFile.TrackEnter();

				try
				{
					if (null != this.AppDeviceManagerWindow)
					{
						// If the paring window is opened and no show any warning dialog, trigger event to it
						if (this.AppDeviceManagerWindow.Visibility == Visibility.Visible &&
							this.AppDeviceManagerWindow.grdOneDlgBackground.Visibility != Visibility.Visible &&
							this.AppDeviceManagerWindow.grdTwoDlgBackground.Visibility != Visibility.Visible)
						{
							bool isChecked = false;
							this.Dispatcher.Invoke(delegate()
							{
								isChecked = (bool)this.AppDeviceManagerWindow.rdbtnPairing.IsChecked;
							});

							if (isChecked)
							{
								// To inform window that the pairing proceduce is starting
								this.SendAppEventAsync(new AppEventArgs(AppEventType.Pairing, eventArgs.Device));

								try
								{
									var response = PairingResponse.CreatePairingResponse(true);
									response.SessionId = PairingSessionId.CreateSessionId(System.Text.Encoding.ASCII.GetBytes(RandomString(8)));
									await eventArgs.Device.PairingResponseAsync(response);
								}
								catch (Exception ex)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");

									// To inform window that the pairing proceduce is finished and the result is failed
									this.SendAppEventAsync(new AppEventArgs(AppEventType.PairingFinished, null, false));
								}
							}
							else
							{
								try
								{
									await eventArgs.Device.PairingResponseAsync(PairingResponse.CreatePairingResponse(false));
								}
								catch (Exception ex)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
								}
							}
						}
						else
						{
							// If the paring window is closed
							try
							{
								await eventArgs.Device.PairingResponseAsync(PairingResponse.CreatePairingResponse(false));
							}
							catch (Exception ex)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
							}
						}
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.AppDeviceManagerWindow is null", "");

						// If the paring window is closed
						try
						{
							await eventArgs.Device.PairingResponseAsync(PairingResponse.CreatePairingResponse(false));
						}
						catch (Exception ex)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
						}
					}
				}
				catch (Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				}
				finally
				{
					LogtoFile.TrackLeave();
				}
			};

			#endregion PairingRequested Event Handler

			#region DevicePaired Event Handler

			/// When DeviceManager send DevicePaired evnet, We need to check the device is in
			/// previous paired device list or not. If not, we should start a new connection.
			this._manager.UserDeviceManager.DevicePaired += delegate(object sender, DevicePairedEventArgs eventArgs)
			{
				LogtoFile.TrackEnter();

				try
				{
					if (null != eventArgs && null != eventArgs.Device)
					{
						AppDeviceInfo deviceInfo = null;

						#region Check the Paired Device is in PairedDevice List

						try
						{
							deviceInfo = this.PairedDevices.Where(x => x.Device.InstanceID.Equals(eventArgs.Device.InstanceID)).FirstOrDefault();
						}
						catch
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "New paired device isn't in paired list", "");
						}

						#endregion Check the Paired Device is in PairedDevice List

						//Represent the device in not exist in Paired Device List, need to create a new connection
						if (null == deviceInfo)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Create AppDeviceInfo for new paired device", "");

							try
							{
								deviceInfo = InitialAppDeviceInfo(eventArgs.Device);

								// To add device to list
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "There is an new paied device to be add", "");
								this.Dispatcher.BeginInvoke(new Action(() =>
								{
									this.PairedDevices.Insert(0, deviceInfo);
								}));

								this.SendAppEventAsync(new AppEventArgs(AppEventType.PairingFinished, eventArgs.Device, true));

								// Make sure all filter are closed

								//ToDoIt by Johnson
								//try
								//{
								//	LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>> Start release all filters before start", "");
								//	this._appScreenWindow.dsElement.Close();
								//	LogtoFile.WriteLog("", Configs.DebugLevelValue, "<<<<< End release all filters before start", "");
								//}
								//catch (Exception ex)
								//{
								//	LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
								//}

								Thread.Sleep(700);

								this.Dispatcher.Invoke(delegate()
								{
									if (FlowControlManager.Instance.IsAcceptNewConnection())
										this._appScreenWindow.ConnectNewScreencast(deviceInfo);
								});
								//this.StartConnectRemoteDevice(deviceInfo);
							}
							catch (Exception ex)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
							}
						}
						else
						{
							this.Dispatcher.Invoke(delegate()
							{
								if (FlowControlManager.Instance.IsAcceptNewConnection())
									this._appScreenWindow.ConnectNewScreencast(deviceInfo);
							});

							LogtoFile.WriteLog("", Configs.DebugLevelValue, "UserDeviceManager.DevicePaired the device is already in Paired List, do nothing", "");
						}
					}
				}
				catch (Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				}
				finally
				{
					LogtoFile.TrackLeave();
				}
				// no need to request for information here or start connection here
				// since we call StartMonitor, a DeviceFound event will be triggered and we're going to call RequestInformation() and create connection then
			};

			#endregion DevicePaired Event Handler

			#region DeviceUnPaired Event Handler

			/// When DeviceManager send DeviceUnPaired evnet, We need to unpaird the device.
			this._manager.UserDeviceManager.DeviceUnPaired += delegate(object sender, DeviceUnPairedEventArgs eventArgs)
			{
				LogtoFile.TrackEnter();

				try
				{
					this.DeviceUnpairedWork(eventArgs.Device);
				}
				catch (Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				}
				finally
				{
					LogtoFile.TrackLeave();
				}
			};

			#endregion DeviceUnPaired Event Handler

			#region DisplayConfigurationUpdated Event Handler

			/// When ConfigurationHelper send DisplayConfigurationUpdated evnet, We need to handle rotation and screenoff event
			this._manager.ConfigurationHelper.DisplayConfigurationUpdated += delegate(object sender, DisplayConfigurationUpdatedEventArgs eventArgs)
			{
				LogtoFile.TrackEnter();

				// For connecting device update use
				if (null != this.ConnectingDeviceInfo)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Screen casting program is already in connected", "");

					try
					{
						//If the event device is the same with current connecting device, we need to handle it
						if (eventArgs.Device.InstanceID.Equals(this.ConnectingDeviceInfo.Device.InstanceID))
						{
							// Do work if the screen of display is off
							if (eventArgs.Device.Display.ScreenOn.HasValue)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "ScreenOn: " + eventArgs.Device.Display.ScreenOn.Value, "");

								if (!eventArgs.Device.Display.ScreenOn.Value)
								{
									this.SendAppEventAsync(new AppEventArgs(AppEventType.ScreenOff, eventArgs.Device));
									return; // if the screen of display is off, no need to update rotation
								}
							}

							// Rotate if the rotation of display is changed
							if (eventArgs.Device.Display.Rotation.HasValue)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Device video rotation (clockwise) : " + eventArgs.Device.Display.Rotation, "");

								this.SendAppEventAsync(new AppEventArgs(AppEventType.RotateDirection, eventArgs.Device, this.GetCurrentPhoneSystemPermissionDetectionRule()));
							}

							// Disable Rotate if the AutoRotate is disable
							if (eventArgs.Device.Display.AutoRotation.HasValue)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Device AutoRotation status : " + eventArgs.Device.Display.AutoRotation, "");

								this.SendAppEventAsync(new AppEventArgs(AppEventType.RotateButtonStatus, eventArgs.Device.Display.AutoRotation.Value));

								//this.SendAppEventAsync(new AppEventArgs(AppEventType.RotateDirection, eventArgs.Device, this.GetCurrentPhoneSystemPermissionDetectionRule()));
							}
						}
					}
					catch (Exception ex)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
					}
					finally
					{
						LogtoFile.TrackLeave();
					}
				}
			};

			#endregion DisplayConfigurationUpdated Event Handler

			#region LaunchRequested Event Handler

			this._manager.MixHelper.LaunchRequested += delegate(object sender, LaunchRequestedEventArgs eventArgs)
			{
				try
				{
					LogtoFile.TrackEnter();

					if (null != eventArgs.Device)
					{
						AppDeviceInfo deviceInfo = this.PairedDevices.Where(item => item.Device.InstanceID == eventArgs.Device.InstanceID).FirstOrDefault();

						if (null != deviceInfo && null == this._connectingWorker)
						{
							if (null != this.ConnectingDeviceInfo &&
								this.ConnectingDeviceInfo.Device.InstanceID.Equals(deviceInfo.Device.InstanceID))
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "The device has been connecting, don't need to do something", "");
							}
							else
							{
								this.Dispatcher.Invoke(delegate()
								{
									if (FlowControlManager.Instance.IsAcceptNewConnection())
										this._appScreenWindow.ConnectNewScreencast(deviceInfo);
								});
								//this.StartConnectRemoteDevice(deviceInfo);
								//ToDoIt by Johnson
								//this.RestartScreenCastingActions(deviceInfo, false);
							}
						}
						else
						{
							if (null == deviceInfo)
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "deviceInfo is null", "");
							if (null != this._connectingWorker)
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "connecting work is not null, please wait", "");
						}
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "eventArgs.Device is null", "");
					}
				}
				catch (Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				}
				finally
				{
					LogtoFile.TrackLeave();
				}
			};

			#endregion LaunchRequested Event Handler

			#region Just for debug use Event, do nothing

			#region DeviceFound Event Handler

			///The function just for log device found event. Don't need to anything.
			this._manager.UserDeviceManager.DeviceFound += delegate(object sender, DeviceFoundEventArgs eventArgs)
			{
				LogtoFile.TrackEnter();
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Get DeviceFound", "");
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Device Name: " + eventArgs.Device.DeviceName, "");
				LogtoFile.TrackLeave();
			};

			#endregion DeviceFound Event Handler

			#region DeviceUpdate Event Handler

			///The function just for log Device Update event. Don't need to anything.
			this._manager.UserDeviceManager.DeviceUpdate += delegate(object sender, DeviceUpdateEventArgs eventArgs)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Get DeviceUpdate", "");

				//Johnson modify it. Becaseu we should not handle the case, just to disconnect is ok.

				//// For connecting device update use
				//if (null != ConnectingDeviceInfo)
				//{
				//	LogtoFile.WriteLog("", Configs.DebugLevelValue, "Screen casting program is already in connecting", "");

				//	try
				//	{
				//		// Re-connect if the IP changed
				//		if (eventArgs.Device.InstanceID.Equals(this.ConnectingDeviceInfo.Device.InstanceID))
				//		{
				//			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Update device is connecting device, to do checking IP address", "");

				//			if (null == eventArgs.Device.Address)
				//			{
				//				LogtoFile.WriteLog("", Configs.DebugLevelValue, "eventArgs.Device.Address is null", "");
				//			}
				//			else
				//			{
				//				LogtoFile.WriteLog("", Configs.DebugLevelValue, "eventArgs.Device.Address is:" + eventArgs.Device.Address.ToString(), "");
				//			}

				//			if (String.IsNullOrEmpty(this.ConnectingDeviceInfo.ServerIP))
				//			{
				//				LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.ConnectingDeviceInfo.ServerIP is empty", "");
				//				return;
				//			}

				//			LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.ConnectingDeviceInfo.ServerIP is: " + this.ConnectingDeviceInfo.ServerIP, "");
				//			LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.ConnectingDeviceInfo.LastServerIP is: " + this.ConnectingDeviceInfo.LastServerIP, "");

				//			if (!this.ConnectingDeviceInfo.ServerIP.Equals(this.ConnectingDeviceInfo.LastServerIP))
				//			{
				//				LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.ConnectingDeviceInfo.ServerIP is different with this.ConnectingDeviceInfo.LastServerIP", "");
				//				if (null != this.ConnectingDeviceInfo)
				//				{
				//					this.ConnectingDeviceInfo.LastServerIP = this.ConnectingDeviceInfo.ServerIP;
				//					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Update this.ConnectingDeviceInfo.LastServerIP", "");

				//					this.ShowScreenCastingWindow();
				//					this.RestartScreen(this.ConnectingDeviceInfo, false);

				//					// Send the event to screen casting window
				//					this.SendAppEventAsync(new AppEventArgs(AppEventType.Reconnecting));
				//				}
				//			}
				//			else
				//			{
				//				if (eventArgs.Device.Services.Screencast.SupportedResolutions != null)
				//					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Device Update event Support Resoultuion Count" + eventArgs.Device.Services.Screencast.SupportedResolutions.Count.ToString(), "");
				//				if (this.ConnectingDeviceInfo.Device.Services.Screencast.SupportedResolutions != null)
				//					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Device Update Support Resoultuion Count" + this.ConnectingDeviceInfo.Device.Services.Screencast.SupportedResolutions.Count.ToString(), "");
				//				LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.ConnectingDeviceInfo.ServerIP is the same as this.ConnectingDeviceInfo.LastServerIP, to skip", "");
				//			}
				//		}
				//	}
				//	catch (Exception ex)
				//	{
				//		LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				//	}
				//}
			};

			#endregion DeviceUpdate Event Handler

			#region DeviceLost Event Handler

			///The function just for log Device Lost event. Don't need to anything.
			this._manager.UserDeviceManager.DeviceLost += delegate(object sender, DeviceLostEventArgs eventArgs)
			{
				LogtoFile.TrackEnter();

				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Get DeviceLost", "");
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Device Name: " + eventArgs.Device.DeviceName, "");

				LogtoFile.TrackLeave();
			};

			#endregion DeviceLost Event Handler

			#endregion Just for debug use Event, do nothing

			#endregion For AFManager
		}

		/// <summary>
		/// Sends the application event synchronous.
		/// </summary>
		/// <param name="args">The <see cref="AppEventArgs"/> instance containing the event data.</param>
		private void SendAppEventAsync(AppEventArgs args)
		{
			try
			{
				LogtoFile.TrackEnter();

				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Ready to send AppEvent: " + args.EventType, "");

				if (null != this.AppEvent)
				{
					var receivers = this.AppEvent.GetInvocationList();
					foreach (EventHandler<AppEventArgs> receiver in receivers)
					{
						//receiver.BeginInvoke(this, args, null, null);
						receiver.Invoke(this, args);
					}
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.AppEvent is null", "");
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
			finally
			{
				LogtoFile.TrackLeave();
			}
		}

		#endregion Event Handle

		#region Async methods

		private async Task DoStopScreenCastingDaemonAsync(AFUserDevice targetDevice)
		{
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Stop remote ADB screen casting", "");

			//Johnon 0528 add for resolve timing issue. If we want to stop screencast, we should disable detect screencast in tcp mode disconnection
			this._isShowTCPDisconnection = false;

			try
			{
				if (null != targetDevice)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "If VNC server is closed first, it will throw exception after calling stop", "");
					if (Configs.IsWithoutMiracast)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call stop async", "");
						bool stopResult = await this.StopScreenCastingDaemonAsync(targetDevice).ConfigureAwait(false);
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "<<<<<< After call stop async, await. stop result: " + stopResult, "");
					}
					else
					{
						if (targetDevice.HasSystemPermission.HasValue && targetDevice.HasSystemPermission.Value == false)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Call stop", "");
							bool stopResult = await this.StopScreenCastingDaemonAsync(targetDevice).ConfigureAwait(false);
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "<<<<<< After call stop async, await. stop result: " + stopResult, "");
						}
						else
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Not need to call stop", "");
						}
					}
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "targetDevice is null", "");
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		private async Task<InitialStatus> PlayScreenCastingDaemonAsync(AFUserDevice targetDevice, ScreencastControlInfo control)
		{
			try
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Enter PlayScreenCastingDaemonAsync", "");
				await targetDevice.ControlScreencastAsync(ScreencastControlInfo.ScreencastOperation.Play, control).ConfigureAwait(false);
				return InitialStatus.PlayScreencastSuccess;
			}
			catch (TimeoutException ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "PlayScreenCastingDaemonAsync Timout exception : " + ex.ToString(), "");
				return InitialStatus.PlayScreencastTimeout;
			}
			catch (WebServiceException ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "PlayScreenCastingDaemonAsync WebServiceException : " + ex.ToString(), "");
				return InitialStatus.PlayScreencastError;
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				return InitialStatus.PlayScreencastError;
			}
			finally
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "<<<<<< End PlayScreenCastingDaemonAsync", "");
			}
		}

		private async Task<bool> StopScreenCastingDaemonAsync(AFUserDevice targetDevice)
		{
			try
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> Enter StopScreenCastingDaemonAsync", "");
				await targetDevice.ControlScreencastAsync(ScreencastControlInfo.ScreencastOperation.Stop, new ScreencastControlInfo()).ConfigureAwait(false);
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "<<<<<< After call StopScreenCastingDaemonAsync, await.", "");
				return true;
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				return false;
			}
			finally
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "<<<<<< End StopScreenCastingDaemonAsync", "");
			}
		}

		private async Task<DeviceInfoStatus> RequestInformationAsync(AFUserDevice targetDevice)
		{
			try
			{
				await targetDevice.RequestInformationAsync().ConfigureAwait(false);
				return DeviceInfoStatus.Connectable;
			}
			catch (TimeoutException)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Request Remote Information Timout", "");
				return DeviceInfoStatus.GetDeviceInfoTimeout;
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				return DeviceInfoStatus.GetDeviceInfoFail;
			}
		}

		private async Task<AFUserDevice.PairingStatus> RequestPairingStatusAsync(AFUserDevice targetDevice)
		{
			return await targetDevice.RequestPairingStatusAsync().ConfigureAwait(false);
		}

		private async Task<ReconfirmAddressResult> ReconfirmAddressAsync(AFUserDevice targetDevice)
		{
			return await targetDevice.ReconfirmAddressAsync().ConfigureAwait(false);
		}

		private async Task<DaemonStatus> GetScreencastDaemonStatusAsync(AFUserDevice targetDevice)
		{
			return await targetDevice.GetScreencastDaemonStatusAsync().ConfigureAwait(false);
		}

		private async Task<bool> GetInputDaemonStatusAsync(AFUserDevice targetDevice)
		{
			return await targetDevice.GetInputDaemonStatusAsync().ConfigureAwait(false);
		}

		#endregion Async methods

		#endregion Private Methods

		#region Public Methods

		public BitmapImage GetQRCodeImage()
		{
			try
			{
				if (null == _qRCodeGenerator)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "_qRCodeGenerator is null", "");
					return null;
				}

				if (null == _manager)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "_manager is null", "");
					return null;
				}

				if (null == _manager.UserDeviceManager)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "_manager.UserDeviceManager is null", "");
					return null;
				}

				if (null == _manager.UserDeviceManager.LocalProfile)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "_manager.UserDeviceManager.LocalProfile is null", "");
					return null;
				}

				// 2015-03-19 Seoker: modify from ToBase64String() to ToJsonString() to simplify the QR code
				var test = _qRCodeGenerator.Encoder(_manager.UserDeviceManager.LocalProfile.ToJsonString());

				var width = _qRCodeGenerator.Width;
				var height = _qRCodeGenerator.Height;

				return test;
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				return null;
			}
		}

		public string GetComputerName()
		{
			try
			{
				if (null == _manager)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "_manager is null", "");
					return string.Empty;
				}

				if (null == _manager.UserDeviceManager)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "_manager.UserDeviceManager is null", "");
					return string.Empty;
				}

				if (null == _manager.UserDeviceManager.LocalProfile)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "_manager.UserDeviceManager.LocalProfile is null", "");
					return string.Empty;
				}

				string deviceName = _manager.UserDeviceManager.LocalProfile.DeviceName;
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> deviceName: " + deviceName, "");

				return deviceName;
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				return string.Empty;
			}
		}

		public void SetADBScreenRecoderParameter(BiteRateType biteratetype = BiteRateType.Others, KeyFrameValueType keyframevaluetype = KeyFrameValueType.Others)
		{
			#region Based on connection type to set keyframe Interval

			ConnectionType type = this.GetCurrentVideoProtocol();

			//Based on Meeting Discussion, we default set TCP keyframe interval is 15 and UDP keyframe interval is 4
			if (type == ConnectionType.TCP)
			{
				this._mKeyFrameInterval = 15;
			}
			else if (type == ConnectionType.UDP)
			{
				this._mKeyFrameInterval = 4;
			}

			#endregion Based on connection type to set keyframe Interval

			#region based on Biterate type to set biterate value

			//Set BiteRate
			if (biteratetype != BiteRateType.Others)
			{
				bool isX86 = true;
				object videoBiteRateValue = null;

				if (biteratetype == BiteRateType.High)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Registry request HighBiteRate", "");

					videoBiteRateValue = RegistryUtility.GetRegistry(RegistryHive.CurrentUser, Configs.RegVideoPath, Configs.RegVideoHighBiterate, RegistryView.Registry64, out isX86);
				}
				else if (biteratetype == BiteRateType.Low)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Registry request LowBiteRate", "");

					videoBiteRateValue = RegistryUtility.GetRegistry(RegistryHive.CurrentUser, Configs.RegVideoPath, Configs.RegVideoLowBiterate, RegistryView.Registry64, out isX86);
				}

				if (videoBiteRateValue != null)
				{
					this._mBitRate = Convert.ToUInt32(videoBiteRateValue);

					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Registry request BiteRate: " + Convert.ToUInt32(videoBiteRateValue).ToString(), "");
				}
				else if (biteratetype == BiteRateType.High)
				{
					this._mBitRate = 3000000;

					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Registry Query Fail, reset to default High BiteRate : 3000000", "");
				}
				else if (biteratetype == BiteRateType.Low)
				{
					this._mBitRate = 500000;

					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Registry Query Fail, reset to default Low BiteRate : 500000", "");
				}
				else
				{
					this._mBitRate = 500000;

					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Registry Query Fail, reset to default BiteRate : 500000", "");
				}
			}
			else
			{
				this._mBitRate = 500000;

				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Default Low BiteRate : 500000", "");
			}

			#endregion based on Biterate type to set biterate value

			#region based on registry value to reset keyframe interval value

			//Set KeyFrameInterval
			if (keyframevaluetype == KeyFrameValueType.Manual)
			{
				bool isX86 = false;
				object videoIntervalValue = RegistryUtility.GetRegistry(RegistryHive.CurrentUser, Configs.RegVideoPath, Configs.RegVideoIntervalKey, RegistryView.Registry64, out isX86);
				if (videoIntervalValue != null)
				{
					this._mKeyFrameInterval = Convert.ToUInt32(videoIntervalValue);

					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Registry request Frame Interval: " + Convert.ToUInt32(videoIntervalValue).ToString(), "");
				}
				else
				{
					if (type == ConnectionType.TCP)
						Configs.ApplicationReference._mKeyFrameInterval = 15;
					else if (type == ConnectionType.UDP)
						Configs.ApplicationReference._mKeyFrameInterval = 4;

					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Registry Query Fail, reset to default Frame Interval : 4", "");
				}
			}

			#endregion based on registry value to reset keyframe interval value
		}

		public void SetReceiveDataEvent()
		{
			bool _bReceiveTimerStarted = this._signalOfReceiveDataTimerStarted.Set();
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "SetReceiveDataEvent _signalOfReceiveDataTimerStarted value is : " + _bReceiveTimerStarted.ToString(), "");
		}

		//This funciton will reset adb command about Screencast resoultion, if we have a new connection,
		//need to call the funciton to reset dimesion
		public void ResetScreencastResolution()
		{
			this._mScreenCastHeight = 0;
			this._mScreenCastWidth = 0;
		}

		public void StartTimerToCheckReceiveData()
		{
			this.SendAppEventAsync(new AppEventArgs(AppEventType.InitialStatus, InitialStatus.InitailizationsCompleted));

			LogtoFile.TrackEnter();
			this.WaitForReceiveDataTimer.Stop();
			this._signalOfReceiveDataTimerStarted.Reset();
			if (null != this.AppScreenWindow)
			{
				// Start to timer for wairing for receving display data
				this.WaitForReceiveDataTimer.Interval = Configs.WaitForReceiveDataTimeoutPeriod;
				this.WaitForReceiveDataTimer.Start();
			}
			else
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.AppScreenWindow is null", "");
			}
			LogtoFile.TrackLeave();
		}

		public void ShowUpdateMessage(Version applicationVersion, Version minimumTargetVersion, Version RemoteDeviceapplicationVersion, Version RemoteDeviceminimumTargetVersion)
		{
			//Need to Update Android Deivice
			if (applicationVersion < RemoteDeviceminimumTargetVersion) // Need to Update Windows Version
			{
				string strAndroidUpdate = ReadingResoureceWording("txtUpdateWindowsDetail");//"Please update to the newest version on your computer to proceed.";
				string strAndroidUpdateDetail = ReadingResoureceWording("txtUpdate"); //"Update";
				this.Dispatcher.BeginInvoke(new Action(() =>
				{
					this._appScreenWindow.Close();

					CustomMessageBox WindowsUpdateMessageBox = new CustomMessageBox(new AirFusion.Windows.CustomMessageBox.CustomMessageBoxStructInfo(
																strAndroidUpdate, CustomMessageBox.CustomImageIcon.Update, strAndroidUpdateDetail), false, true);

					WindowsUpdateMessageBox.ShowDialog();

					if (WindowsUpdateMessageBox.MessageBoxResult == System.Windows.MessageBoxResult.OK)
					{
						this.UserCheckforUpdatesAsync();
					}
				}));
			}
			else if (minimumTargetVersion > RemoteDeviceminimumTargetVersion)
			{
				string strAndroidUpdate = ReadingResoureceWording("UpdateAndroidDetail");// "Please update to the newest version on your mobile device to proceed.";
				string strAndroidUpdateDetail = ReadingResoureceWording("txtPlayStore"); //"Google Play";
				this.Dispatcher.BeginInvoke(new Action(() =>
				{
					this._appScreenWindow.Close();

					CustomMessageBox androidUpdateMessageBox = new CustomMessageBox(new AirFusion.Windows.CustomMessageBox.CustomMessageBoxStructInfo(
																strAndroidUpdate, CustomMessageBox.CustomImageIcon.Update, strAndroidUpdateDetail), true, true);

					androidUpdateMessageBox.ShowDialog();

					if (androidUpdateMessageBox.MessageBoxResult == System.Windows.MessageBoxResult.OK)
					{
						this.LinkToAcerEXTENDPlayStore();
					}
				}));
			}
		}

		#endregion Public Methods

		#region Callbacks

		/// <summary>
		/// Handle Input control file transfer update event callback.
		/// </summary>
		/// <param name="e">The <see cref="VNC.FTUpdateEventArgs"/> instance containing the event data.</param>
		private void VNCClientFileTransferUpdateEventCallback(VNC.FTUpdateEventArgs e)
		{
			try
			{
				if (e.TransferType == VNC.VNC_FTType.Upload)
				{
					string[] infos = new string[2];
					infos[0] = e.Progress.ToString();
					infos[1] = Path.GetFileName(e.FilePath);

					this.SendAppEventAsync(new AppEventArgs(AppEventType.UploadFilePercentage, infos));

					if (e.Progress == 100)
					{
						++_nCurrentFinishFileCount;

						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Upload, e.Progress: 100, _nTotalFileCount: " + _nTotalFileCount + ", _nCurrentFinishFileCount: " + _nCurrentFinishFileCount, "");

						if (_nCurrentFinishFileCount == _nTotalFileCount)
						{
							// Send event to UI, to finish uploading
							this.SendAppEventAsync(new AppEventArgs(AppEventType.UploadFilePercentage));

							_nTotalFileCount = 0;
							_nCurrentFinishFileCount = 0;

							return;
						}
					}
				}
				else if (e.TransferType == VNC.VNC_FTType.Download)
				{
					if (e.Progress == 100)
					{
						// Send event to UI, to finish uploading
						this.SendAppEventAsync(new AppEventArgs(AppEventType.DownloadFilePercentage, e.FilePath));
						return;
					}

					string[] infos = new string[2];
					infos[0] = e.Progress.ToString();
					infos[1] = Path.GetFileName(e.FilePath);

					this.SendAppEventAsync(new AppEventArgs(AppEventType.DownloadFilePercentage, infos));
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		/// <summary>
		/// Handle Input control message event callback.
		/// </summary>
		/// <param name="e">The <see cref="VNC.VNCMessageEventArgs"/> instance containing the event data.</param>
		private void VNCClientVNCMessageEventCallback(VNC.VNCMessageEventArgs e)
		{
			try
			{
				if (VNC.VNC_ERR_CODE.EC_CreateFileFailed == e.ErrorCode || VNC.VNC_ERR_CODE.EC_InvalidFileSize == e.ErrorCode)
				{
					string msg = string.Format("MsgType: {0}; Function: {1}; ErrorCode: {2}", e.MsgType, e.Function, e.ErrorCode);
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "VNCClientVNCMessageEventCallback, " + msg, "");

					// Send event to UI, to show disconnected
					this.SendAppEventAsync(new AppEventArgs(AppEventType.VNCTransferFileError));
				}
				else if (VNC.VNC_ERR_CODE.EC_NoError != e.ErrorCode)
				{
					string msg = string.Format("MsgType: {0}; Function: {1}; ErrorCode: {2}", e.MsgType, e.Function, e.ErrorCode);
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "VNCClientVNCMessageEventCallback, " + msg, "");

					//if (null != this._connectingWorker)
					//{
					//	LogtoFile.WriteLog("", Configs.DebugLevelValue, "There is an existed connecting work process, no need to fire event", "");
					//	// If there is an existed connecting work process, no need to show disconnection dialog
					//	return;
					//}

					AppDeviceInfo disConnectDeviceInfo = this.ConnectingDeviceInfo;

					if (null != disConnectDeviceInfo)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Before clean, disConnectDeviceInfo is not null", "");
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Before clean, disConnectDeviceInfo is null", "");
					}

					//this.ClearAllConnection();

					//if (null != disConnectDeviceInfo)
					//{
					//	LogtoFile.WriteLog("", Configs.DebugLevelValue, "After clean, disConnectDeviceInfo is not null", "");
					//}
					//else
					//{
					//	LogtoFile.WriteLog("", Configs.DebugLevelValue, "After clean, disConnectDeviceInfo is null", "");
					//}

					// Send event to UI, to show disconnected
					this.SendAppEventAsync(new AppEventArgs(AppEventType.Disconnected, disConnectDeviceInfo));
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		/// <summary>
		/// RTSPs the controller RTSP event callback.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">The <see cref="RTSPControllerEventArgs"/> instance containing the event data.</param>
		private void RTSPControllerRTSPEventCallback(object sender, RTSPControllerEventArgs e)
		{
			try
			{
				if (RTSPControllerEventType.RTSPSocketDisconnected == e.EventType)
				{
					// Send event to UI, to show disconnected
					this.SendAppEventAsync(new AppEventArgs(AppEventType.Disconnected));

					if (null != this._connectingWorker)
					{
						this.ConnectingDeviceInfo = null;
						this.AbortConnectingThread();
					}
					else
					{
						this.ClearAllConnection();
					}
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		/// <summary>
		/// Systems the events session switch callback.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">The <see cref="SessionSwitchEventArgs"/> instance containing the event data.</param>
		private void SystemEventsSessionSwitchCallback(object sender, SessionSwitchEventArgs e)
		{
			try
			{
				LogtoFile.TrackEnter();

				switch (e.Reason)
				{
					case SessionSwitchReason.SessionLock:
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "OS is locked", "");
							this.SendAppEventAsync(new AppEventArgs(AppEventType.CloseWindowBySystemSleep));
						}
						break;
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
			finally
			{
				LogtoFile.TrackLeave();
			}
		}

		/// <summary>
		/// Systems the events(suspend) changed callback.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">The <see cref="PowerModeChangedEventArgs"/> instance containing the event data.</param>
		private void SystemEventsPowerModeChangedCallback(object sender, PowerModeChangedEventArgs e)
		{
			try
			{
				LogtoFile.TrackEnter();

				switch (e.Mode)
				{
					case PowerModes.Suspend:
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "OS is suspend", "");
							this.SendAppEventAsync(new AppEventArgs(AppEventType.CloseWindowBySystemSleep));
						}
						break;
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
			finally
			{
				LogtoFile.TrackLeave();
			}
		}

		/// <summary>
		/// Systems the events events thread shutdown callback, remove Session/PowerModeChange event
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
		private void SystemEventsEventsThreadShutdownCallback(object sender, EventArgs e)
		{
			try
			{
				LogtoFile.TrackEnter();

				// Unregister system event when event thead is shut down to avoid memory leak.
				SystemEvents.SessionSwitch -= this.SystemEventsSessionSwitchCallback;
				SystemEvents.PowerModeChanged -= this.SystemEventsPowerModeChangedCallback;
				SystemEvents.EventsThreadShutdown -= this.SystemEventsEventsThreadShutdownCallback;
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
			finally
			{
				LogtoFile.TrackLeave();
			}
		}

		#endregion Callbacks
	}
}