﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Data;
using System.Globalization;
using System.Windows.Controls;
using System.Windows;

namespace AirFusion
{
	/// <summary>
	/// Converts Boolean Value to Control.Visibility Value
	/// </summary>
	public class BooleanToVisibilityConverter : IValueConverter
	{
		public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
		{
			bool isYes = false;

			try
			{
				isYes = (bool)value;
			}
			catch
			{

			}

			if (isYes)
			{
				return System.Windows.Visibility.Visible;
			}
			else
			{
				return System.Windows.Visibility.Collapsed;
			}
		}

		public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
		{
			throw new NotImplementedException();
		}
	}

	/// <summary>
	/// Converts Boolean Value to Control.Visibility Value
	/// </summary>
	public class InverseBooleanToVisibilityConverter : IValueConverter
	{
		public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
		{
			bool isYes = false;

			try
			{
				isYes = (bool)value;
			}
			catch
			{

			}

			if (isYes)
			{
				return System.Windows.Visibility.Collapsed;

			}
			else
			{
				return System.Windows.Visibility.Visible;
			}
		}

		public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
		{
			throw new NotImplementedException();
		}
	}

	/// <summary>
	/// Converts to TabItem Width Value
	/// </summary>
	public class TabItemWidthConverter : IMultiValueConverter
	{
		public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			TabControl tabControl = values[0] as TabControl;
			double width = (tabControl.ActualWidth / tabControl.Items.Count) - 3;
			
			////Subtract 1, otherwise we could overflow to two rows.
			//return (width <= 1) ? 0 : (width - 1);

			return width;
		}

		public object[] ConvertBack(object value, Type[] targetTypes, object parameter,
			System.Globalization.CultureInfo culture)
		{
			throw new NotSupportedException();
		}
	}

	/// <summary>
	/// Converts to Unstable Network Visibility Value
	/// </summary>
	public class ScreenVisibilityToUnstableNetworkVisibilityConverter : IMultiValueConverter
	{
		public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			System.Windows.Visibility screenVisibility = System.Windows.Visibility.Collapsed;
			bool IsNetworkGood = true;

			try
			{
				screenVisibility = (System.Windows.Visibility)values[0];
				IsNetworkGood = (bool)values[1];
			}
			catch
			{

			}

			if (screenVisibility == System.Windows.Visibility.Visible)
			{
				if (!IsNetworkGood)
				{
					return System.Windows.Visibility.Visible;
				}
				else
				{
					return System.Windows.Visibility.Collapsed;
				}
			}
			else
			{
				return System.Windows.Visibility.Collapsed;
			}
		}

		public object[] ConvertBack(object value, Type[] targetTypes, object parameter,
			System.Globalization.CultureInfo culture)
		{
			throw new NotSupportedException();
		}
	}

	/// <summary>
	/// Does Monitoring PasswordBox
	/// </summary>
	public class PasswordBoxMonitor : DependencyObject
	{
		#region Properties

		public static readonly DependencyProperty IsMonitoringProperty =
			DependencyProperty.RegisterAttached("IsMonitoring", typeof(bool), typeof(PasswordBoxMonitor), new UIPropertyMetadata(false, OnIsMonitoringChanged));


		public static readonly DependencyProperty PasswordLengthProperty =
			DependencyProperty.RegisterAttached("PasswordLength", typeof(int), typeof(PasswordBoxMonitor), new UIPropertyMetadata(0));

		#endregion

		#region Public Methods

		public static bool GetIsMonitoring(DependencyObject obj)
		{
			return (bool)obj.GetValue(IsMonitoringProperty);
		}

		public static void SetIsMonitoring(DependencyObject obj, bool value)
		{
			obj.SetValue(IsMonitoringProperty, value);
		}

		public static int GetPasswordLength(DependencyObject obj)
		{
			return (int)obj.GetValue(PasswordLengthProperty);
		}

		public static void SetPasswordLength(DependencyObject obj, int value)
		{
			obj.SetValue(PasswordLengthProperty, value);
		}

		#endregion

		#region Callbacks

		private static void OnIsMonitoringChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			var pb = d as PasswordBox;
			if (pb == null)
			{
				return;
			}

			if ((bool)e.NewValue)
			{
				pb.PasswordChanged += PasswordChanged;
			}
			else
			{
				pb.PasswordChanged -= PasswordChanged;
			}
		}

		private static void PasswordChanged(object sender, RoutedEventArgs e)
		{
			var pb = sender as PasswordBox;
			if (pb == null)
			{
				return;
			}

			SetPasswordLength(pb, pb.Password.Length);
		}

		#endregion
	}
}
