﻿using AirFusion.Manager.DeviceManager;
using AirFusionCustomToast;
using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace AirFusion
{
	/// <summary>
	/// Enumeration for the arguments of initializing progressbar
	/// </summary>
	public enum InitializingProgressbarArguments
	{
		MaxValue,
		Value,
	}

	/// <summary>
	/// Enumeration for the status of initializing connection
	/// </summary>
	public enum InitialStatus
	{
		WaitForPairing,
		InitializeInputControl,
		InitializeInputControlFailed,

		//0514 Johnson add
		InitializeInputControlTimeout,

		//0515 Johnson add
		InitializeInputControlWebServiceException,

		RunInputControlFailed,
		ConnectToInputControlCompleted,
		InitializeScreenCasting,
		ConnectToRemoteServerFailed,
		InitailizationsCompleted,
		ReceiveDataFailed,
		ReceiveDataCompleted,
		DirectlyAborted,
		InitializeADBDeviceFailed,
		NoAvailableDevice,
		NoOnlineDevice,
		InitializeADBInputDaemonPullFailed,
		InitializeADBInputDaemonPushFailed,
		InitializeADBInputDaemonShellSetPrivilegeFailed,
		InitializeADBInputDaemonShellFailed,

		#region 0420 Johnson add for more detail debug info

		DandelionReconfirmIPFail,
		InitializeWakeUpFail,
		InitializeConnectToVNCFail,
		InitializeConnectTOVNC,
		InitializeCheckVNCDaemonStatus,
		InputDaemonNotRunning,
		InitializeCheckScreencastDaemonStatus,
		InitializeScreenCastDaemonFail,

		InitializePlayScreencast,

		//0515
		InitializeScreencastDaemonTimeout,

		InitializeScreencastDaemonWebServiceException,

		//0513 Johnson add
		DandelionGetIPFail,

		InitializeGetDeviceStatusFail,
		InitializeGetDeviceStatusTimeout,
		InitializeGetDeviceInformationFail,
		InitializeGetDeviceInformationTimeout,

		#endregion 0420 Johnson add for more detail debug info

		//0428 for Timer stop screencast event
		TimerStopScreenCast,

		VNCDisconnect,
		TCPDisconnect,

		//0513 Johnson add more debug use log
		PhoneWakeUp,

		GetDeviceStatus,
		GetDeviceInformation,

		PlayScreencastTimeout,
		PlayScreencastError,
		PlayScreencastSuccess,
	}

	public enum ErrorTypeMessage
	{
		DandelionReconfirmIPFail,
		InitializeWakeUpFail,
		InitializeConnectToVNCFail,
		InputDaemonNotRunning,
		InitializeScreenCastDaemonFail,
		TimerStopScreenCast,
		VNCDisconnect,
		TCPDisconnect,
		ReceiveDataFailed,
		InitializeInputControlFailed,
		HintScreenCastStoped,

		//0513 Johnon add more debug use log
		DandelionGetIPFail,

		InitializeGetDeviceStatusFail,
		InitializeGetDeviceInfoFail,

		//0514 Johnon add more debug use log
		InitializeGetDeviceStatusTimeout,

		InitializeGetDeviceInfoTimeout,
		InitializeGetInputDaemonStatusTimeout,

		//0515 Johnson add
		InitializeInputControlWebServiceException,

		InitializeScreencastDaemonTimeout,
		InitializeScreencastDaemonWebServiceException,

		//0521 Johnson add
		InitializePlayScreencastTimeout,

		InitializePlayScreencastFail,

		UnpairDevice,

		//0703 Johnson add
		ADBListDeviceError,
		ADBListDeviceNotFound,		
		ADBPullInputDaemonFail,		
		ADBPushInputDaemonFail,		
		ADBShellSetPrivilegeInputDaemonFail,		
		LaunchInputDaemonByADBException,	
		ADBPullScreencastDaemonFail,		
		ADBPushScreencastDaemonFail,		
		ADBShellSetPrivilegeScreencastDaemonFail,		
		LaunchScreencastDaemonByADBException,		
	}

	public enum WarningMessageType
	{
		UnableToConnect,
		NotOnTheSameNetWork,
		ScreencastingSuspended,
		ScreencastingStopped,
		ConnectionFailed,
		Reboot,
		UnpairDevice,
	}

	/// <summary>
	/// Enumeration for the status of device
	/// </summary>
	public enum DeviceInfoStatus
	{
		Connectable,
		Unconnectable,
		Unpair,
		WakeUpFail,
		WakeUpSuccess,
		ConfirmIPFail,
		DandelionFail,
		CheckDeviceStatusFail,
		CheckDeviceStatusTimeout,
		GetDeviceInfoFail,
		GetDeviceInfoTimeout,
	}

	/// <summary>
	/// Class for device in application
	/// </summary>
	public class AppDeviceInfo : System.ComponentModel.INotifyPropertyChanged
	{
		public event System.ComponentModel.PropertyChangedEventHandler PropertyChanged;

		protected virtual void OnPropertyChanged(string propertyName)
		{
			if (this.PropertyChanged != null)
			{
				this.PropertyChanged(this, new System.ComponentModel.PropertyChangedEventArgs(propertyName));
			}
		}

		#region Members

		//private PhoneCallToastManager _phoneCallManager = null; // 2015-05-20 Seoker: remove incoming call functions since it's defeatured
		private GeneralNotificationManager _generalNotificationManager = null;

		#endregion Members

		#region Binding Properties

		private AFUserDevice _device = null;

		public AFUserDevice Device
		{
			get
			{
				return _device;
			}

			set
			{
				_device = value;
				this.OnPropertyChanged("DeviceName");
			}
		}

		public string DeviceName
		{
			get
			{
				return _device.DeviceName;
			}
		}

		private bool _isSendNotificationOn = false;

		public bool IsSendNotificationOn
		{
			get
			{
				return _isSendNotificationOn;
			}

			set
			{
				this._isSendNotificationOn = value;
				if (null != this._generalNotificationManager)
				{
					if (null != this.Device)
					{
						this._generalNotificationManager.ConfigureNotificationStatus(
							this.Device.InstanceID.ToString(),
							this._isSendNotificationOn ? Manager.Notification.DeviceIgnoreStatus.Permit : Manager.Notification.DeviceIgnoreStatus.Ignore);
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.Device is null", "");
					}
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "this._generalNotificationManager is null", "");
				}

				RegistryUtility.SetRegistry(RegistryHive.CurrentUser, Configs.RegUISettingsPath + @"\" + _device.InstanceID.ToString(), Configs.RegNotificationKey, value, RegistryView.Registry64);
				this.OnPropertyChanged("IsSendNotificationOn");
			}
		}

		// 2015-05-20 Seoker: remove Auto-login & incoming call functions since it's defeatured
		/*
		private bool _isAnswerPhoneCallOn = false;
		public bool IsAnswerPhoneCallOn
		{
			get
			{
				return _isAnswerPhoneCallOn;
			}

			set
			{
				this._isAnswerPhoneCallOn = value;
				if (null != this._phoneCallManager)
				{
					if (null != this.Device)
					{
						this._phoneCallManager.EnablePhoneCallToast(this.Device.InstanceID, this._isAnswerPhoneCallOn);
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.Device is null", "");
					}
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "this._phoneCallManager is null", "");
				}

				RegistryUtility.SetRegistry(RegistryHive.CurrentUser, Configs.RegUISettingsPath + @"\" + _device.InstanceID.ToString(), Configs.RegAnswerPhoneCallKey, value, RegistryView.Registry64);
				this.OnPropertyChanged("IsAnswerPhoneCallOn");
			}
		}

		private bool _isAutoUnlockOn = false;
		public bool IsAutoUnlockOn
		{
			get
			{
				return _isAutoUnlockOn;
			}

			set
			{
				_isAutoUnlockOn = value;

				RegistryUtility.SetRegistry(RegistryHive.CurrentUser, Configs.RegUISettingsPath + @"\" + _device.InstanceID.ToString(), Configs.RegUnlockKey, value, RegistryView.Registry64);
				this.OnPropertyChanged("IsAutoUnlockOn");
			}
		}

		private string _loginAccount = string.Empty;
		public string LoginAccount
		{
			get
			{
				return _loginAccount;
			}

			set
			{
				if (_loginAccount != value)
				{
					_loginAccount = value;

					RegistryUtility.SetRegistry(RegistryHive.CurrentUser, Configs.RegUISettingsPath + @"\" + _device.InstanceID.ToString(), Configs.RegLoginAccountKey, value.ToString(), RegistryView.Registry64);
					this.OnPropertyChanged("LoginAccount");
				}
			}
		}

		private string _loginPassword = string.Empty;
		public string LoginPassword
		{
			get
			{
				return _loginPassword;
			}

			set
			{
				if (_loginPassword != value)
				{
					_loginPassword = value;

					RegistryUtility.SetRegistry(RegistryHive.CurrentUser, Configs.RegUISettingsPath + @"\" + _device.InstanceID.ToString(), Configs.RegLoginPWDKey, value.ToString(), RegistryView.Registry64);
					this.OnPropertyChanged("LoginPassword");
				}
			}
		}
		*/

		#endregion Binding Properties

		#region Properties

		public string ServerIP
		{
			get
			{
				if (null != _device)
				{
					if (null != _device.Address)
					{
						return _device.Address.ToString();
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "_device.Address is null", "");
						return string.Empty;
					}
				}
				else
				{
					return string.Empty;
				}
			}
		}

		private string _lastServerIP = string.Empty;

		public string LastServerIP
		{
			get
			{
				return _lastServerIP;
			}

			set
			{
				if (_lastServerIP != value)
				{
					_lastServerIP = value;
				}
			}
		}

		private int _serverRTSPPort = 0;

		public int ServerRTSPPort
		{
			get
			{
				return _serverRTSPPort;
			}

			set
			{
				_serverRTSPPort = value;
			}
		}

		private ReconfirmAddressResult _reconfirmIPresult = null;

		public ReconfirmAddressResult ReconfirmAddressResult
		{
			get;
			set;
		}

		private int _serverRTPPort = 0;

		public int ServerRTPPort
		{
			get
			{
				return _serverRTPPort;
			}

			set
			{
				_serverRTPPort = value;
			}
		}

		private uint _serverVNCPort = 0;

		public uint ServerVNCPort
		{
			get
			{
				return _serverVNCPort;
			}

			set
			{
				_serverVNCPort = value;
			}
		}

		private string _serverVNCPassword = string.Empty;

		public string ServerVNCPassword
		{
			get
			{
				return _serverVNCPassword;
			}

			set
			{
				_serverVNCPassword = value;
			}
		}

		#endregion Properties

		#region Constructors & Disposes

		// 2015-05-20 Seoker: remove incoming call functions since it's defeatured
		public AppDeviceInfo(/*PhoneCallToastManager phoneCallManager,*/GeneralNotificationManager generalNotificationManager)
		{
			//this._phoneCallManager = phoneCallManager;
			this._generalNotificationManager = generalNotificationManager;
		}

		#endregion Constructors & Disposes
	}

	/// <summary>
	/// Class for the video size wtih pixel in application
	/// </summary>
	public class VideoSize
	{
		private double _videoWidth = 0;

		public double VideoWidth
		{
			get
			{
				return _videoWidth;
			}

			set
			{
				_videoWidth = value;
			}
		}

		private double _videoHeight = 0;

		public double VideoHeight
		{
			get
			{
				return _videoHeight;
			}

			set
			{
				_videoHeight = value;
			}
		}
	}

	/// <summary>
	/// Class for the global settings in application
	/// </summary>
	public class GlobalSettings : System.ComponentModel.INotifyPropertyChanged
	{
		public event System.ComponentModel.PropertyChangedEventHandler PropertyChanged;

		protected virtual void OnPropertyChanged(string propertyName)
		{
			if (this.PropertyChanged != null)
			{
				this.PropertyChanged(this, new System.ComponentModel.PropertyChangedEventArgs(propertyName));
			}
		}

		#region Members

		private GeneralNotificationManager _generalNotificationManager = null;

		#endregion Members

		#region Binding Properties

		private bool _isMute = false;

		public bool IsMute
		{
			get
			{
				return _isMute;
			}

			set
			{
				_isMute = value;

				if (null != this._generalNotificationManager)
				{
					this._generalNotificationManager.MuteAllNotification(_isMute);
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "this._generalNotificationManager is null", "");
				}

				RegistryUtility.SetRegistry(RegistryHive.CurrentUser, Configs.RegUISettingsPath, Configs.RegMute, value, RegistryView.Registry64);
				this.OnPropertyChanged("IsMute");
			}
		}

		private bool _isAutoUpdate = false;

		public bool IsAutoUpdate
		{
			get
			{
				return _isAutoUpdate;
			}

			set
			{
				_isAutoUpdate = value;

				RegistryUtility.SetRegistry(RegistryHive.CurrentUser, Configs.RegUISettingsPath, Configs.RegAutoUpdate, value, RegistryView.Registry64);
				this.OnPropertyChanged("IsAutoUpdate");
			}
		}

		private bool _isStartUpLaunch = false;

		public bool IsStartUpLaunch
		{
			get
			{
				return _isStartUpLaunch;
			}

			set
			{
				_isStartUpLaunch = value;

				RegistryUtility.SetRegistry(RegistryHive.CurrentUser, Configs.RegUISettingsPath, Configs.RegStartUpLaunch, value, RegistryView.Registry64);
				this.OnPropertyChanged("IsStartUpLaunch");
				if (_isStartUpLaunch)
				{
					string strLaunchParameter = "\"" + Process.GetCurrentProcess().MainModule.FileName + "\" " + "\"bootLaunch\"";
					RegistryUtility.SetRegistry(RegistryHive.CurrentUser, Configs.RegStartUpLaunchPath, Configs.Product, strLaunchParameter, RegistryView.Registry64);
				}
				else
				{
					bool isDelete = false;
					RegistryUtility.DeleteRegistry(RegistryHive.CurrentUser, Configs.RegStartUpLaunchPath, Configs.Product, RegistryView.Registry64, isDelete);
				}
			}
		}

		#endregion Binding Properties

		#region Constructors & Disposes

		public GlobalSettings(GeneralNotificationManager generalNotificationManager)
		{
			this._generalNotificationManager = generalNotificationManager;
		}

		#endregion Constructors & Disposes
	}

	/// <summary>
	/// Enumerations for the video resolution
	/// </summary>
	public enum VideoResolutionOptions
	{
		LandscapeResolution1280x720,
		LandscapeResolution720x480,
		LandscapeResolution640x480,
		PortraitResolution720x1280,
		PortraitResolution480x720,
	}

	/// <summary>
	/// Enumerations for phone manufacturer detection
	/// </summary>
	public enum PhoneSystemPermissionDetectionRule
	{
		AutoDetect,
		ForceHas,
		ForceNotHas,
		NotDetect,
	}

	/// <summary>
	/// The enumeration for source filter COM method
	/// </summary>
	public enum ConnectionType
	{
		UDP = 1,
		TCP = 2,
		Others = 3
	}

	public enum BiteRateType
	{
		High = 1,
		Low = 2,
		Others = 3
	}

	public enum KeyFrameValueType
	{
		Manual = 1,
		Others = 2
	}

	/// <summary>
	/// The enumeration for video quality
	/// </summary>
	public enum VideoQuality
	{
		H264_SD_LOW,
		H264_SD_MEDIUM,
		H264_SD_HIGH,
		H264_HD_720P,
	}

	/// <summary>
	/// The enumeration for show DeviceManager Pairing or Paired
	/// </summary>
	public enum DeviceManagerTab
	{
		DeviceManager_Pairing,
		DeviceManager_Paired,
	}
}