﻿using AirFusion.Common;
using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;

namespace AirFusion
{
	public static class LogtoFile
	{
		private static string _logFileName = "AcerEXTEND";
		private static string _logFileExtensionName = ".log";

		//private static string _logFileNameWithPath = String.Format("MainUI_{1:yyyyMMddHHmmssfff}{2}", DateTime.UtcNow, _logFileExtensionName);
		private static string _logFileNameWithPath = String.Format("MainUI_{0:yyyyMMddHHmmssfff}{1}", DateTime.UtcNow, _logFileExtensionName);

		private static string _regDebug = Configs.Product;
		private static int _defaultDebugLevel = 0;   // Set it to 0 for opening log enable function, Set it to 1 for always open log
		private static string _defaultLogFilePath = null;
		private static Mutex _mut = new Mutex();

		public static string RegPathLog = Configs.RegPath + @"\Log";

		static LogtoFile()
		{
			try
			{
				// get default debug file path
				string sPath;
				sPath = System.IO.Path.Combine(Configs.LogPath, _logFileNameWithPath);
				_defaultLogFilePath = sPath;

				// get debug level from registry
				try
				{
					object oRK = Registry.CurrentUser.OpenSubKey(RegPathLog).GetValue(_regDebug);
					if (oRK != null)
					{
						_defaultDebugLevel = (int)oRK;
					}
				}
				catch (Exception ex)
				{
					Console.WriteLine("Error:" + ex.ToString());
				}

				if (_defaultDebugLevel != 0)
				{
					// initial other logger
					StaticLogger.SetParameter(1024, 52428800);
					StaticLogger.Initialize(System.IO.Path.Combine(Configs.LogPath, _logFileName + _logFileExtensionName), LogLevel.Trace);

					string sVncLogName = string.Format("VNCDebug_{0}.log", DateTime.Now.ToString("yyyyMMddHHmmss"));
					VNC.VNCClient.ControlVNCDebugLog(_defaultDebugLevel > 0, System.IO.Path.Combine(Configs.LogPath, sVncLogName));
				}
			}
			catch (Exception ex)
			{
				Console.WriteLine("Error:" + ex.ToString());
			}
		}

		private static int GetLogValue()
		{
			int defaultValue = 0;

			try
			{
				// get default debug file path
				string sPath;
				sPath = System.IO.Path.Combine(Configs.LogPath, _logFileNameWithPath);
				_defaultLogFilePath = sPath;

				// get debug level from registry
				object oRK = Registry.CurrentUser.OpenSubKey(RegPathLog).GetValue(_regDebug);
				if (oRK != null)
				{
					defaultValue = (int)oRK;
				}
			}
			catch (Exception ex)
			{
				Console.WriteLine("Error:" + ex.ToString());
			}

			return defaultValue;
		}

		public static void WriteLog(string sFileNameWithPath, DebugLevel debugLevel, string sFileContent, string sCaller)
		{
			//_DefaultDebugLevel = GetLogValue();
			if (_defaultDebugLevel == 0)
			{
				return;
			}

			try
			{
				_mut.WaitOne();
				if ((int)debugLevel >= _defaultDebugLevel)
				{
					switch (debugLevel)
					{
						case DebugLevel.OnlyDebugWrite:
							{
								Debug.WriteLine(sFileContent);
							}
							break;

						case DebugLevel.OnlyUI:
							{
								OnlyUI(sFileNameWithPath, sFileContent, sCaller);
							}
							break;

						case DebugLevel.OnlyAFAFStaticLogger:
							{
								StaticLogger.LogDebug(sFileContent);
							}
							break;

						case DebugLevel.All:
							{
								Debug.WriteLine(sFileContent);
								OnlyUI(sFileNameWithPath, sFileContent, sCaller);
								StaticLogger.LogDebug(sFileContent);
							}
							break;

						default:
							break;
					}
				}
			}
			catch
			{
				//Cannot do anything but makes web back to work ......
				//EventLog.WriteEntry("ASL", ex.Message);
				//return;
			}
			finally
			{
				_mut.ReleaseMutex();
			}
		}

		private static void OnlyUI(string sFileNameWithPath, string sFileContent, string sCaller)
		{
			FileInfo oFI = null;
			try
			{
				//string testFile = "d:\\abc.txt";
				if (sFileNameWithPath == string.Empty)
				{
					sFileNameWithPath = _defaultLogFilePath;
				}

				if (sCaller == string.Empty)
				{
					StackTrace st = new StackTrace(true);
					if (st.FrameCount > 1)
					{
						StackFrame sf = st.GetFrame(2);

						if (System.Threading.Thread.CurrentThread.ManagedThreadId == App.Current.Dispatcher.Thread.ManagedThreadId)
						{
							sCaller = string.Format("{0}, {1}, {2}", "MainThread", Path.GetFileName(sf.GetFileName()), sf.GetMethod().Name);
						}
						else
						{
							sCaller = string.Format("{0}, {1}, {2}", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString("X4"), Path.GetFileName(sf.GetFileName()), sf.GetMethod().Name);
						}
					}
				}

				oFI = new FileInfo(sFileNameWithPath);
				if (!oFI.Directory.Exists)
				{
					Directory.CreateDirectory(oFI.Directory.FullName);
				}

				oFI = null;

				using (StreamWriter oS = File.AppendText(sFileNameWithPath))
				{
					oS.WriteLine(string.Format("{0} [{1}] {2}", DateTime.Now.ToLocalTime().ToString("MM/dd/yyyy HH:mm:ss:fff"), sCaller, sFileContent));
					oS.Flush();
					oS.Close();
				}
			}
			catch (Exception _ex)
			{
				throw _ex;
			}
			finally
			{
				if (oFI != null)
				{
					oFI = null;
				}
			}
		}

		/// <summary>
		/// Tracks the activity of entering a function.
		/// </summary>
		/// <returns></returns>
		[DebuggerStepThrough]
		public static void TrackEnter(string methodName = null)
		{
			StackTrace trace = new StackTrace(false);
			if (methodName == null)
			{
				StackFrame frame = trace.GetFrame(1);
				MethodBase method = (frame != null) ? frame.GetMethod() : null;
				methodName = method != null ? (method.DeclaringType != null ? GetFullName(method.DeclaringType) + "." : "") + method.Name : "unknown method";
			}

			LogtoFile.WriteLog("", Configs.DebugLevelValue, ">>>>>> ========= Enter [" + methodName + "] ========= >>>>>>", "");
		}

		/// <summary>
		/// Tracks the activity of leaving a function.
		/// </summary>
		/// <returns></returns>
		[DebuggerStepThrough]
		public static void TrackLeave(string methodName = null)
		{
			StackTrace trace = new StackTrace(false);
			if (methodName == null)
			{
				StackFrame frame = trace.GetFrame(1);
				MethodBase method = (frame != null) ? frame.GetMethod() : null;
				methodName = method != null ? (method.DeclaringType != null ? GetFullName(method.DeclaringType) + "." : "") + method.Name : "unknown method";
			}

			LogtoFile.WriteLog("", Configs.DebugLevelValue, "<<<<<< ========= Leave [" + methodName + "] ========= <<<<<<", "");
		}

		/// <summary>
		/// Gets the full name of the specified type.
		/// </summary>
		/// <param name="type">The type.</param>
		/// <returns></returns>
		[DebuggerStepThrough]
		private static string GetFullName(Type type)
		{
			try
			{
				string fullName = "";
				if (type.IsGenericType)
				{
					Type[] genericParameters = type.GetGenericArguments();
					fullName = GetSpecifiedName(type.Name, ref genericParameters);

					if (type.DeclaringType != null)
					{
						fullName = (type.DeclaringType.ContainsGenericParameters ? GetSpecifiedName(type.DeclaringType.Name, ref genericParameters) : type.DeclaringType.Name) + "." + fullName;
					}
				}
				else
				{
					fullName = type.Name;
					if (type.DeclaringType != null)
					{
						fullName = type.DeclaringType.Name + "." + fullName;
					}
				}

				if (string.IsNullOrEmpty(type.Namespace) == false)
				{
					fullName = type.Namespace + "." + fullName;
				}

				return fullName;
			}
			catch
			{
				return type.FullName;
			}
		}

		[DebuggerStepThrough]
		private static string GetSpecifiedName(string name, ref Type[] additionalGenericParameters)
		{
			int used = 0;
			Match match = Regex.Match(name, "^(.*)`([0-9]+)$", RegexOptions.Compiled);
			if (match.Success)
			{
				used = Convert.ToInt32(match.Groups[2].Value);
				string fullName = match.Groups[1].Value + "<" + string.Join(",", additionalGenericParameters.Skip(additionalGenericParameters.Length - used).Take(used).Select(item => GetFullName(item))) + ">";
				additionalGenericParameters = additionalGenericParameters.Take(additionalGenericParameters.Length - used).ToArray();
				return fullName;
			}
			else
			{
				return name;
			}
		}
	}

	public enum DebugLevel
	{
		NotLog = 0,
		OnlyDebugWrite = 1,
		OnlyUI = 2,
		OnlyAFAFStaticLogger = 3,
		All = 4,
	}
}