﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net;
using System.Net.Sockets;
using System.Timers;

namespace AirFusion
{
	public class CustomSocket : Socket
	{
		#region Members

		private Timer _timer;
		private List<SocketEventHandler> onCloseHandlers = new List<SocketEventHandler>();

		#endregion

		#region Properties

		public bool EventsEnabled
		{
			set
			{
				if (value)
				{
					_timer.Start();
				}
				else
				{
					_timer.Stop();
				}
			}
		}

		public bool PollingConnected
		{
			get
			{
				bool part1 = this.Poll(1000, SelectMode.SelectRead);
				bool part2 = (this.Available == 0);

				return !(part1 && part2);
			}
		}

		#endregion

		#region Events

		public delegate void SocketEventHandler(Socket socket);
		public event SocketEventHandler SocketClosed
		{
			add { onCloseHandlers.Add(value); }
			remove { onCloseHandlers.Remove(value); }
		}

		#endregion

		#region Constructors

		public CustomSocket(AddressFamily addressFamily, SocketType socketType, ProtocolType protocolType)
			: base(addressFamily, socketType, protocolType)
		{
			_timer = new Timer(1000);
			_timer.Elapsed += delegate(Object source, ElapsedEventArgs e)
			{
				if (!PollingConnected)
				{
					foreach (var socketEventHandler in onCloseHandlers)
					{
						socketEventHandler.Invoke(this);
					}

				}
			};
		}

		#endregion
	}
}
