﻿using Gma.QrCodeNet.Encoding;
using Gma.QrCodeNet.Encoding.Windows.Render;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Media.Imaging;

namespace AirFusion
{
	public class QRCodeGenerator
	{
		private int qrWidth = 0;
		private int qrHeight = 0;

		public int Width { get { return this.qrWidth; } }
		public int Height { get { return this.qrHeight; } }

		public BitmapImage Encoder(string codeString)
		{
			if (codeString == null)
				return null;   // Parameter Fail;

			return StreamToBitmapConverter(QrEncoder(codeString));
		}

		private MemoryStream QrEncoder(string codeString, ErrorCorrectionLevel errorLevel = ErrorCorrectionLevel.L)
		{
			try
			{
				//設定錯誤修正容量
				/*
				  Level L (Low)      7%  of codewords can be restored.
				  Level M (Medium)   15% of codewords can be restored.
				  Level Q (Quartile) 25% of codewords can be restored.
				  Level H (High)     30% of codewords can be restored.
				*/
				QrEncoder encoder = new QrEncoder(errorLevel);

				//編碼
				QrCode code = encoder.Encode(codeString);

				//定義線條寬度
				int moduleSizeInPixels = 12;

				//繪二維條碼圖初始化
				GraphicsRenderer renderer = new GraphicsRenderer(new FixedModuleSize(moduleSizeInPixels, QuietZoneModules.Two), Brushes.Black, Brushes.White);

				//留白區大小
				System.Drawing.Point padding = new System.Drawing.Point(10, 16);

				//取得條碼圖大小
				DrawingSize dSize = renderer.SizeCalculator.GetSize(code.Matrix.Width);
				int imgWidth = dSize.CodeWidth + 2 * padding.X;
				int imgHeight = dSize.CodeWidth + 2 * padding.Y;
				qrWidth = imgWidth;
				qrHeight = imgHeight;

				//設定影像大小
				Image img = new Bitmap(imgWidth, imgHeight);
				//this.pictureBox1.Width = imgWidth;
				//this.pictureBox1.Height = imgHeight;
				////設定視窗大小 Kelsier
				//this.Width = imgWidth + 10;
				//this.Height = imgHeight + 28;
				//繪製二維條碼圖
				Graphics g = Graphics.FromImage(img);
				renderer.Draw(g, code.Matrix, padding);
				//this.pictureBox1.Image = img;

				//return img;
				MemoryStream outStream = new MemoryStream();
				renderer.WriteToStream(code.Matrix, ImageFormat.Bmp, outStream);
				return outStream;
			}
			catch (Exception)
			{ return null; }
		}

		private BitmapImage StreamToBitmapConverter(MemoryStream mStream)
		{
			if (mStream == null)
				return null;

			BitmapImage bitmap = new BitmapImage();
			bitmap.BeginInit();
			bitmap.StreamSource = mStream;
			bitmap.CacheOption = BitmapCacheOption.OnLoad;
			bitmap.EndInit();
			return bitmap;
		}
	}
}
