﻿using System;
using System.Runtime.InteropServices;
using System.Windows;
using System.Windows.Interop;

namespace AirFusion
{
	internal class WindowStater
	{
		#region WindowState

		private WindowState _windowState;
		private ResizeMode _resizeMode;
		private bool _topMost;
		private bool _isMaximized = false;
		private double _originalHeight;
		private double _originalWidth;
		private double _originalCenterX;
		private double _originalCenterY;
		private double _finalRatio;

		public void Maximize(Window targetWindow, int x, int y, double width, double height)
		{
			if (!_isMaximized)
			{
				_isMaximized = true;
				targetWindow.SizeToContent = SizeToContent.Manual;
				targetWindow.WindowState = WindowState.Maximized;
				targetWindow.ResizeMode = ResizeMode.NoResize;
				targetWindow.Background = System.Windows.Media.Brushes.Black;
			}

			targetWindow.Top = y;
			targetWindow.Left = x;
			targetWindow.Width = width;
			targetWindow.Height = height;

			if (!targetWindow.Topmost)
			{
				targetWindow.Topmost = true;
			}
		}

		public void Save(Window targetWindow, double finalRatio)
		{
			_windowState = targetWindow.WindowState;
			_resizeMode = targetWindow.ResizeMode;
			_topMost = targetWindow.Topmost;
			_originalCenterX = targetWindow.Left + targetWindow.ActualWidth / 2;
			_originalCenterY = targetWindow.Top + targetWindow.ActualHeight / 2;
			_finalRatio = finalRatio;
		}

		public void Restore(Window targetWindow)
		{
			targetWindow.Width = double.NaN;
			targetWindow.Height = double.NaN;
			targetWindow.SizeToContent = SizeToContent.WidthAndHeight;
			targetWindow.Background = System.Windows.Media.Brushes.Transparent;
			targetWindow.WindowState = _windowState;
			targetWindow.ResizeMode = _resizeMode;
			targetWindow.Topmost = _topMost;
			targetWindow.Top = _originalCenterY - targetWindow.ActualHeight * _finalRatio / 2;
			targetWindow.Left = _originalCenterX - targetWindow.ActualWidth * _finalRatio / 2;
			_isMaximized = false;
		}

		#endregion WindowState
	}
}