﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace AirFusion
{
	/// <summary>
	/// CustomProgressBar.xaml 的互動邏輯
	/// </summary>
	public partial class CustomProgressBar : UserControl
	{
		#region Properties

		private double _minimum = 0;
		private double _maximum = 100;
		private double _value = 0;

		private const string _progressUndoColor = "#FF929497";
		private const string _progressDoneColor = "#FFa5ea1d";

		private const string _progress0Color = "#00a5ea1d";
		private const string _progress20Color = "#33a5ea1d";
		private const string _progress40Color = "#66a5ea1d";
		private const string _progress60Color = "#99a5ea1d";
		private const string _progress80Color = "#CCa5ea1d";
		private const string _progress100Color = "#FFa5ea1d";

		public double Maximum
		{
			get
			{
				return _maximum;
			}
			set
			{
				if (this._maximum > 0)
					this._maximum = value;
			}
		}

		public double Value
		{
			get
			{
				return _value;
			}
			set
			{
				double tmp = value;

				if (tmp >= this._minimum && tmp <= this._maximum)
				{
					this._value = tmp;
					this.RefreshProgressbar();
				}
			}
		}

		#endregion Properties

		public CustomProgressBar()
		{
			InitializeComponent();

			Storyboard storyboard1 = new Storyboard();
			storyboard1.Children.Add(this.SetAnimation(this.e11));
			storyboard1.RepeatBehavior = RepeatBehavior.Forever;

			Storyboard storyboard2 = new Storyboard();
			storyboard2.Children.Add(this.SetAnimation(this.e22));
			storyboard2.RepeatBehavior = RepeatBehavior.Forever;

			Storyboard storyboard3 = new Storyboard();
			storyboard3.Children.Add(this.SetAnimation(this.e33));
			storyboard3.RepeatBehavior = RepeatBehavior.Forever;

			storyboard1.Begin();
			storyboard2.Begin();
			storyboard3.Begin();
		}

		private void RefreshProgressbar()
		{
			double progress = this._value * 3 / (this._maximum - this._minimum);

			if (progress == 0)
			{
				this.e11.Visibility = Visibility.Collapsed;
				this.e22.Visibility = Visibility.Collapsed;
				this.e33.Visibility = Visibility.Collapsed;

				this.e1.Stroke = (SolidColorBrush)(new BrushConverter().ConvertFrom(_progressUndoColor));
				this.e2.Stroke = (SolidColorBrush)(new BrushConverter().ConvertFrom(_progressUndoColor));
				this.e3.Stroke = (SolidColorBrush)(new BrushConverter().ConvertFrom(_progressUndoColor));
			}
			else if (progress < 1)
			{
				this.e11.Visibility = Visibility.Visible;
				this.e22.Visibility = Visibility.Collapsed;
				this.e33.Visibility = Visibility.Collapsed;

				this.e1.Stroke = (SolidColorBrush)(new BrushConverter().ConvertFrom(_progressUndoColor));
				this.e2.Stroke = (SolidColorBrush)(new BrushConverter().ConvertFrom(_progressUndoColor));
				this.e3.Stroke = (SolidColorBrush)(new BrushConverter().ConvertFrom(_progressUndoColor));
			}
			else if (progress < 2)
			{
				this.e22.Visibility = Visibility.Visible;
				this.e11.Visibility = Visibility.Collapsed;
				this.e33.Visibility = Visibility.Collapsed;

				this.e1.Stroke = (SolidColorBrush)(new BrushConverter().ConvertFrom(_progressDoneColor));
				this.e2.Stroke = (SolidColorBrush)(new BrushConverter().ConvertFrom(_progressUndoColor));
				this.e3.Stroke = (SolidColorBrush)(new BrushConverter().ConvertFrom(_progressUndoColor));
			}
			else if (progress < 3)
			{
				this.e33.Visibility = Visibility.Visible;
				this.e11.Visibility = Visibility.Collapsed;
				this.e22.Visibility = Visibility.Collapsed;

				this.e1.Stroke = (SolidColorBrush)(new BrushConverter().ConvertFrom(_progressDoneColor));
				this.e2.Stroke = (SolidColorBrush)(new BrushConverter().ConvertFrom(_progressDoneColor));
				this.e3.Stroke = (SolidColorBrush)(new BrushConverter().ConvertFrom(_progressUndoColor));
			}
			else
			{
				this.e11.Visibility = Visibility.Collapsed;
				this.e22.Visibility = Visibility.Collapsed;
				this.e33.Visibility = Visibility.Collapsed;

				this.e1.Stroke = (SolidColorBrush)(new BrushConverter().ConvertFrom(_progressDoneColor));
				this.e2.Stroke = (SolidColorBrush)(new BrushConverter().ConvertFrom(_progressDoneColor));
				this.e3.Stroke = (SolidColorBrush)(new BrushConverter().ConvertFrom(_progressDoneColor));
			}
		}

		private ObjectAnimationUsingKeyFrames SetAnimation(Ellipse e)
		{
			ObjectAnimationUsingKeyFrames objectAnimationUsingKeyFrames = new ObjectAnimationUsingKeyFrames();
			int timeSpan = 60;
			string[] orderedProgressColor = { _progress0Color, _progress20Color, _progress40Color, _progress60Color, _progress80Color, _progress100Color,
										   _progress80Color, _progress60Color, _progress40Color, _progress20Color, _progress100Color};

			for (int i = 0; i < 11; i++)
			{
				DiscreteObjectKeyFrame discreteObjectKeyFrame = new DiscreteObjectKeyFrame();
				discreteObjectKeyFrame.KeyTime = KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(i * timeSpan));
				discreteObjectKeyFrame.Value = (SolidColorBrush)(new BrushConverter().ConvertFrom(orderedProgressColor[i]));

				objectAnimationUsingKeyFrames.KeyFrames.Add(discreteObjectKeyFrame);
			}

			objectAnimationUsingKeyFrames.FillBehavior = FillBehavior.Stop;
			Storyboard.SetTarget(objectAnimationUsingKeyFrames, e);
			Storyboard.SetTargetProperty(objectAnimationUsingKeyFrames, new PropertyPath("Stroke"));

			return objectAnimationUsingKeyFrames;
		}
	}
}