﻿using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace AirFusion
{
	/// <summary>
	/// Interaction logic for GlobalSettingsPage.xaml
	/// </summary>
	public partial class GlobalSettingsPage : UserControl, INotifyPropertyChanged
	{
		#region Events

		public event EventHandler<UserControlEventArgs> UserControlEvent;

		public event System.ComponentModel.PropertyChangedEventHandler PropertyChanged;

		#endregion Events

		#region Properties

		private GlobalSettings _targetSettings = null;

		public GlobalSettings TargetSettings
		{
			get
			{
				return _targetSettings;
			}

			set
			{
				if (_targetSettings != value)
				{
					_targetSettings = value;
					this.OnPropertyChanged("TargetSettings");
				}
			}
		}

		public ICommand ClickHyperlinkCheckforUpdates { get; private set; }

		#endregion Properties

		#region Constructor & Dispose

		public GlobalSettingsPage()
		{
			// These commands needs to be declared and initialized before initializing UI components because there are UI components uses them with binding.
			ClickHyperlinkCheckforUpdates = new DelegateCommand(this.CheckforUpdates, null);

			InitializeComponent();
			this.DataContext = this;

			// Set version information
			this.txtbGlobalSettingsAboutDetail.Text = string.Format(Application.Current.TryFindResource("txtGlobalSettingsAboutItem") as string, Configs.ApplicationReference.ApplicationVersion);
			//this.txtbGlobalSettingsAboutDetail.Text = string.Format(Application.Current.TryFindResource("txtVersion") as String,
			//	Application.Current.TryFindResource("txtAppName") as String,
			//	Configs.ApplicationReference.Version);
		}

		#endregion Constructor & Dispose

		#region Private Methods

		private void CheckforUpdates(object parameter)
		{
			Configs.ApplicationReference.UserCheckforUpdatesAsync();
		}

		#endregion Private Methods

		#region Protected Methods

		protected virtual void OnPropertyChanged(string propertyName)
		{
			if (this.PropertyChanged != null)
			{
				this.PropertyChanged(this, new System.ComponentModel.PropertyChangedEventArgs(propertyName));
			}
		}

		#endregion Protected Methods

		#region UI Event Callbacks

		private void btnPageBack_Click(object sender, RoutedEventArgs e)
		{
			if (null != this.UserControlEvent)
			{
				this.UserControlEvent(this, new UserControlEventArgs(UserControlEventType.PageBack, PageType.GlobalSettings));
			}
		}

		#endregion UI Event Callbacks
	}
}