﻿using AirFusion.Common;
using System;
using System.Text;

namespace AirFusion.Manager.DeviceManager
{
	public partial class DnsSdHandler
	{
		/// <summary>
		/// Represents the callback arguments of RegisterService().
		/// </summary>
		public class RegistrarCallbackArgs : BaseCallbackArgs
		{
			/// <summary>
			/// Gets the instance name.
			/// </summary>
			public string InstanceName { get; internal set; }

			/// <summary>
			/// Gets the hostname that the service is registered.
			/// </summary>
			public string Hostname { get; internal set; }

			/// <summary>
			/// Gets the port number that the service is registered.
			/// </summary>
			public ushort? Port { get; internal set; }

			/// <summary>
			/// Gets the TXT record that the service is registered.
			/// </summary>
			public TxtRecord TxtRecord { get; internal set; }

			/// <summary>
			/// Gets the index of the interface that the service is registered.
			/// </summary>
			public uint? InterfaceIndex { get; internal set; }

			/// <summary>
			/// Initializes a new instance of the <see cref="RegistrarCallbackArgs"/> class.
			/// </summary>
			internal RegistrarCallbackArgs()
				: base()
			{
				this.InstanceName = null;
				this.Hostname = null;
				this.Port = null;
				this.TxtRecord = null;
				this.InterfaceIndex = null;
			}

			/// <summary>
			/// Returns a <see cref="System.String" /> that represents this instance.
			/// </summary>
			/// <returns>
			/// A <see cref="System.String" /> that represents this instance.
			/// </returns>
			public override string ToString()
			{
				StringBuilder builder = new StringBuilder();
				builder.AppendLine(this.GetType().GetFullName() + " (Base):\r\n" + base.ToString());
				builder.AppendLine(this.GetType().GetFullName() + ".InstanceName: " + this.InstanceName);
				builder.AppendLine(this.GetType().GetFullName() + ".Hostname: " + this.Hostname);
				builder.AppendLine(this.GetType().GetFullName() + ".Port: " + (this.Port.HasValue ? this.Port.Value.ToString() : "N/A"));
				builder.AppendLine(this.GetType().GetFullName() + ".TxtRecord: " + (this.TxtRecord != null ? "\r\n" + this.TxtRecord.ToString() : "N/A"));
				builder.AppendLine(this.GetType().GetFullName() + ".InterfaceIndex: " + (this.InterfaceIndex.HasValue ? this.InterfaceIndex.Value.ToString() : "N/A"));

				return builder.ToString().TrimEnd();
			}
		}
	}
}
