﻿using AirFusion.Common;
using System.Text;

namespace AirFusion.Manager.DeviceManager
{
	public partial class DnsSdHandler
	{
		/// <summary>
		/// Represents the data of a registrar.
		/// </summary>
		public class RegistrarData : ProcessorData
		{
			/// <summary>
			/// Gets the instance name.
			/// </summary>
			public string InstanceName { get; internal set; }

			/// <summary>
			/// Gets the hostname.
			/// </summary>
			public string Hostname { get; internal set; }

			/// <summary>
			/// Gets the port number.
			/// </summary>
			public ushort Port { get; internal set; }

			/// <summary>
			/// Gets the index of the interface.
			/// </summary>
			public uint InterfaceIndex { get; internal set; }

			/// <summary>
			/// Gets the TXT record.
			/// </summary>
			public TxtRecord TxtRecord { get; internal set; }

			/// <summary>
			/// Gets the callback function.
			/// </summary>
			internal RegistrarCallback Callback { get; set; }

			/// <summary>
			/// Initializes a new instance of the <see cref="RegistrarData"/> class.
			/// </summary>
			internal RegistrarData()
			{
				this.InstanceName = null;
				this.Hostname = null;
				this.Port = 0;
				this.InterfaceIndex = 0;
				this.TxtRecord = null;
				this.Callback = null;
			}

			/// <summary>
			/// Initializes a new instance of the <see cref="RegistrarData"/> class.
			/// </summary>
			/// <param name="instanceName">The instance name.</param>
			/// <param name="hostname">The hostname.</param>
			/// <param name="port">The port number.</param>
			/// <param name="interfaceIndex">The index of the interface.</param>
			/// <param name="txtRecord">The TXT record.</param>
			/// <param name="callback">The callback.</param>
			internal RegistrarData(string instanceName, string hostname, ushort port, uint interfaceIndex, TxtRecord txtRecord, RegistrarCallback callback)
			{
				this.InstanceName = instanceName;
				this.Hostname = hostname;
				this.Port = port;
				this.InterfaceIndex = interfaceIndex;
				this.TxtRecord = txtRecord;
				this.Callback = callback;
			}

			/// <summary>
			/// Returns a <see cref="System.String" /> that represents this instance.
			/// </summary>
			/// <returns>
			/// A <see cref="System.String" /> that represents this instance.
			/// </returns>
			public override string ToString()
			{
				StringBuilder builder = new StringBuilder();
				builder.AppendLine(this.GetType().GetFullName() + " (Base):\r\n" + base.ToString());
				builder.AppendLine(this.GetType().GetFullName() + ".InstanceName: " + this.InstanceName);
				builder.AppendLine(this.GetType().GetFullName() + ".Hostname: " + this.Hostname);
				builder.AppendLine(this.GetType().GetFullName() + ".Port: " + this.Port);
				builder.AppendLine(this.GetType().GetFullName() + ".InterfaceIndex: " + this.InterfaceIndex);
				builder.AppendLine(this.GetType().GetFullName() + ".TxtRecord: " + (this.TxtRecord != null ? "\r\n" + this.TxtRecord.ToString() : "N/A"));

				return builder.ToString().TrimEnd();
			}
		}
	}
}
