﻿using AirFusion.Common;
using System.Text;

namespace AirFusion.Manager.DeviceManager
{
	public partial class DnsSdHandler
	{
		/// <summary>
		/// Represents the data of a resolver.
		/// </summary>
		public class ResolverData : ProcessorData
		{
			/// <summary>
			/// Gets the instance name.
			/// </summary>
			public string InstanceName { get; internal set; }

			/// <summary>
			/// Gets the index of the interface.
			/// </summary>
			public uint InterfaceIndex { get; internal set; }

			/// <summary>
			/// Gets a value indicating whether the resolver forces a multicast query.
			/// </summary>
			public bool ForceMulticast { get; internal set; }

			/// <summary>
			/// Gets the callback function.
			/// </summary>
			internal ResolverCallback Callback { get; set; }

			/// <summary>
			/// Initializes a new instance of the <see cref="ResolverData"/> class.
			/// </summary>
			internal ResolverData()
			{
				this.InstanceName = null;
				this.InterfaceIndex = 0;
				this.ForceMulticast = false;
				this.Callback = null;
			}

			/// <summary>
			/// Initializes a new instance of the <see cref="ResolverData"/> class.
			/// </summary>
			/// <param name="instanceName">The instance name.</param>
			/// <param name="interfaceIndex">The index of the interface.</param>
			/// <param name="forceMulticast">A value indicating whether the resolver forces a multicast query.</param>
			/// <param name="callback">The callback.</param>
			internal ResolverData(string instanceName, uint interfaceIndex, bool forceMulticast, ResolverCallback callback)
			{
				this.InstanceName = instanceName;
				this.InterfaceIndex = interfaceIndex;
				this.ForceMulticast = forceMulticast;
				this.Callback = callback;
			}

			/// <summary>
			/// Returns a <see cref="System.String" /> that represents this instance.
			/// </summary>
			/// <returns>
			/// A <see cref="System.String" /> that represents this instance.
			/// </returns>
			public override string ToString()
			{
				StringBuilder builder = new StringBuilder();
				builder.AppendLine(this.GetType().GetFullName() + " (Base):\r\n" + base.ToString());
				builder.AppendLine(this.GetType().GetFullName() + ".InstanceName: " + this.InstanceName);
				builder.AppendLine(this.GetType().GetFullName() + ".InterfaceIndex: " + this.InterfaceIndex);
				builder.AppendLine(this.GetType().GetFullName() + ".ForceMulticast: " + this.ForceMulticast);

				return builder.ToString().TrimEnd();
			}
		}
	}
}
