﻿using AirFusion.Common;
using Bonjour;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace AirFusion.Manager.DeviceManager
{
	public partial class DnsSdHandler
	{
		/// <summary>
		/// Provides the TXT record functionality.
		/// </summary>
		public class TxtRecord
		{
			private Dictionary<string, string> _txtRecord;

			/// <summary>
			/// Initializes the instance of TXT record.
			/// </summary>
			public TxtRecord()
			{
				this._txtRecord = new Dictionary<string, string>();
			}

			/// <summary>
			/// Determines if the TXT record contains the specified key.
			/// </summary>
			/// <param name="key">The key.</param>
			/// <returns></returns>
			public bool ContainsKey(string key)
			{
				return this._txtRecord.ContainsKey(key);
			}

			/// <summary>
			/// Gets total number of key/value pairs.
			/// </summary>
			/// <returns></returns>
			public uint GetCount()
			{
				return (uint)this._txtRecord.Count;
			}

			/// <summary>
			/// Gets key at the specified index.
			/// </summary>
			/// <param name="index">The index.</param>
			/// <returns></returns>
			public string GetKeyAtIndex(uint index)
			{
				return this._txtRecord.ElementAt((int)index).Key;
			}

			/// <summary>
			/// Gets value at the specified index.
			/// </summary>
			/// <param name="index">The index.</param>
			/// <returns></returns>
			public string GetValueAtIndex(uint index)
			{
				return this._txtRecord.ElementAt((int)index).Value;
			}

			/// <summary>
			/// Gets value for the specified key.
			/// </summary>
			/// <param name="key">The key.</param>
			/// <returns></returns>
			public string GetValueForKey(string key)
			{
				return this._txtRecord[key];
			}

			/// <summary>
			/// Removes value with the specified key.
			/// </summary>
			/// <param name="key">The key.</param>
			public void RemoveValue(string key)
			{
				this._txtRecord.Remove(key);
			}

			/// <summary>
			/// Sets key value.
			/// </summary>
			/// <param name="key">The key.</param>
			/// <param name="value">The value.</param>
			public void SetValue(string key, string value)
			{
				this._txtRecord[key] = value;
			}

			/// <summary>
			/// Returns a <see cref="System.String"/> that represents this instance.
			/// </summary>
			/// <returns>
			/// A <see cref="System.String"/> that represents this instance.
			/// </returns>
			public override string ToString()
			{
				StringBuilder builder = new StringBuilder();
				foreach (var entry in this._txtRecord)
				{
					builder.AppendLine(entry.Key + "=" + entry.Value);
				}

				return builder.ToString().TrimEnd();
			}
		}
	}

	#region Extensions

	/// <summary>
	/// Extensions.
	/// </summary>
	internal static partial class AFExtensions
	{
		internal static Bonjour.TXTRecord Transform(this DnsSdHandler.TxtRecord txtRecord)
		{
			if (txtRecord == null)
			{
				return null;
			}

			// construct txt record
			Bonjour.TXTRecord newRecord = new Bonjour.TXTRecord();
			for (uint index = 0; index < txtRecord.GetCount(); ++index)
			{
				try
				{
					newRecord.SetValue(txtRecord.GetKeyAtIndex(index), Encoding.UTF8.GetBytes(txtRecord.GetValueAtIndex(index)));
				}
				catch (Exception ex)
				{
					StaticLogger.LogWarn("Failed setting TXT record: " + ex.Message);
				}
			}

			return newRecord;
		}

		internal static DnsSdHandler.TxtRecord Transform(this TXTRecord txtRecord)
		{
			if (txtRecord == null)
			{
				return null;
			}

			// construct txt record
			DnsSdHandler.TxtRecord newRecord = new DnsSdHandler.TxtRecord();
			for (uint index = 0; index < txtRecord.GetCount(); ++index)
			{
				try
				{
					newRecord.SetValue(txtRecord.GetKeyAtIndex(index), Encoding.UTF8.GetString((byte[])txtRecord.GetValueAtIndex(index)));
				}
				catch (Exception ex)
				{
					StaticLogger.LogWarn("Failed setting TXT record: " + ex.Message);
				}
			}

			return newRecord;
		}

		internal static string GetRepresentation(this DnsSdHandler.TxtRecord txtRecord, string prefix = null)
		{
			if (txtRecord == null)
			{
				return null;
			}

			StringBuilder builder = new StringBuilder();
			for (uint index = 0; index < txtRecord.GetCount(); ++index)
			{
				builder.AppendLine(prefix + "" + txtRecord.GetKeyAtIndex(index) + "=" + txtRecord.GetValueAtIndex(index));
			}

			return builder.ToString();
		}

		internal static string GetRepresentation(this TXTRecord txtRecord, string prefix = null)
		{
			if (txtRecord == null)
			{
				return null;
			}

			StringBuilder builder = new StringBuilder();
			for (uint index = 0; index < txtRecord.GetCount(); ++index)
			{
				try
				{
					builder.AppendLine(prefix + "" + txtRecord.GetKeyAtIndex(index) + "=" + Encoding.UTF8.GetString((byte[])txtRecord.GetValueAtIndex(index)));
				}
				catch (Exception ex)
				{
					StaticLogger.LogWarn("Failed printing TXT record: " + ex.Message);
				}
			}

			return builder.ToString();
		}
	}

	#endregion
}
