﻿using AirFusion.Common;
using System;
using System.Text;

namespace AirFusion.Manager.DeviceManager
{
	/// <summary>
	/// Represents the event arguments of initialized event.
	/// </summary>
	public sealed class InitializedEventArgs : EventArgs
	{
		#region Properties

		/// <summary>
		/// The initialization time, in milliseconds.
		/// </summary>
		public long InitializedTime { get; private set; }

		#endregion Properties

		#region Methods

		/// <summary>
		/// Initializes a new instance of the <see cref="InitializedEventArgs"/> class.
		/// </summary>
		internal InitializedEventArgs()
		{
			this.InitializedTime = 0;
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="InitializedEventArgs"/> class.
		/// </summary>
		/// <param name="initializedTime">The initialization time, in milliseconds.</param>
		internal InitializedEventArgs(long initializedTime)
		{
			this.InitializedTime = initializedTime;
		}

		/// <summary>
		/// Returns a <see cref="System.String" /> that represents this instance.
		/// </summary>
		/// <returns>
		/// A <see cref="System.String" /> that represents this instance.
		/// </returns>
		public override string ToString()
		{
			StringBuilder builder = new StringBuilder();
			builder.AppendLine(this.GetType().GetFullName() + ".InitializedTime: " + this.InitializedTime);

			return builder.ToString().TrimEnd();
		}

		#endregion Methods
	}
}