﻿using AirFusion.Common;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirFusion.Manager.DeviceManager
{
	/// <summary>
	/// Represnets the pairing response.
	/// </summary>
	public sealed class PairingResponse
	{
		#region Properties

		/// <summary>
		/// Gets a value indicating whether this pairing response is intended to accept the pairing request.
		/// </summary>
		public bool IsAccepted { get; set; }

		/// <summary>
		/// Gets the session ID.
		/// </summary>
		/// <value>
		/// The session ID.
		/// </value>
		public ISessionId SessionId { get; set; }

		#endregion Properties

		#region Private/Protected/Internal Methods

		/// <summary>
		/// Prevents a default instance of the <see cref="PairingResponse"/> class from being created.
		/// </summary>
		private PairingResponse()
		{
			this.IsAccepted = true;
			this.SessionId = null;
		}

		#endregion Private/Protected/Internal Methods

		#region Override Object

		/// <summary>
		/// Returns a <see cref="System.String" /> that represents this instance.
		/// </summary>
		/// <returns>
		/// A <see cref="System.String" /> that represents this instance.
		/// </returns>
		public override string ToString()
		{
			StringBuilder builder = new StringBuilder();
			builder.AppendLine(this.GetType().GetFullName() + ".IsAccepted: " + this.IsAccepted);
			builder.AppendLine(this.GetType().GetFullName() + ".SessionId: " + (this.SessionId != null ? this.SessionId.ToString() : "N/A"));

			return builder.ToString().TrimEnd();
		}

		#endregion Override Object

		#region Public Static Methods

		/// <summary>
		/// Creates a pairing response with the specified accept value.
		/// </summary>
		/// <param name="accept">Determines whether to accept the pairing response.</param>
		/// <returns></returns>
		public static PairingResponse CreatePairingResponse(bool accept)
		{
			PairingResponse response = new PairingResponse();
			response.IsAccepted = accept;

			return response;
		}

		#endregion Public Static Methods
	}
}