﻿using AirFusion.Common;
using Newtonsoft.Json;
using System.Text;

namespace AirFusion.WebService.Adapters
{
	internal partial class DeviceInformation
	{
		public partial class ServicesInfo
		{
			/// <summary>
			/// Represents the information of screencast service.
			/// </summary>
			public partial class ScreencastInfo
			{
				/// <summary>
				/// Gets the port number of screen cast.
				/// </summary>
				[JsonProperty("Port")]
				public ushort? Port { get; internal set; }

				/// <summary>
				/// Gets the supported resolutions.
				/// </summary>
				[JsonProperty("SupportedResolutions")]
				public ScreenResolution[] SupportedResolutions { get; internal set; }

				#region Private/Protected/Internal

				/// <summary>
				/// Initializes a new instance of the <see cref="ScreencastInfo"/> class.
				/// </summary>
				internal ScreencastInfo()
				{
					this.Port = null;
					this.SupportedResolutions = null;
				}

				#endregion Private/Protected/Internal

				#region Override

				/// <summary>
				/// Returns a <see cref="System.String" /> that represents this instance.
				/// </summary>
				/// <returns>
				/// A <see cref="System.String" /> that represents this instance.
				/// </returns>
				public override string ToString()
				{
					StringBuilder builder = new StringBuilder();
					builder.AppendLine(this.GetType().GetFullName() + ".Port: " + (this.Port.HasValue ? this.Port.Value.ToString() : "N/A"));
					builder.AppendLine(this.GetType().GetFullName() + ".SupportedResolutions: " + (this.SupportedResolutions != null ? string.Join("; ", this.SupportedResolutions) : "N/A"));

					return builder.ToString().TrimEnd();
				}

				#endregion Override
			}
		}
	}
}
