﻿using Newtonsoft.Json;
using System;
using System.Net.Http;

namespace AirFusion.WebService.Adapters
{
	/// <summary>
	/// Represents the web service of launch request (POST /launch).
	/// </summary>
	internal sealed class LaunchRequestAdapter : WebServiceAdapter<LaunchRequestAdapter.LaunchRequestInfo, DBNull>
	{
		#region Definitions

		internal const string WebServiceUri = "/launch";
		internal static HttpMethod WebServiceMethod = HttpMethod.Post;

		#endregion Definitions

		#region Classes

		/// <summary>
		/// Represents the data of launch request.
		/// </summary>
		public sealed class LaunchRequestInfo
		{
			/// <summary>
			/// Represents the instance ID of the requester.
			/// </summary>
			[JsonProperty("InstanceID")]
			public Guid? InstanceID { get; set; }
		}

		#endregion Classes

		#region Methods

		/// <summary>
		/// Initializes a new instance of the <see cref="LaunchRequestAdapter"/> class.
		/// </summary>
		/// <param name="webServiceManager">The web service manager.</param>
		public LaunchRequestAdapter(WebServicesManager webServiceManager)
			: base(webServiceManager, WebServiceUri, WebServiceMethod)
		{
		}

		#endregion Methods
	}
}
