﻿using System;
using System.Diagnostics;
using System.Linq;
using System.Net.Http;

namespace AirFusion.WebService.Adapters
{
	/// <summary>
	/// Represents the exception of a version not matched during a web service request.
	/// </summary>
	public sealed class WebServiceVersionNotMatchedException : WebServiceException
	{
		internal const string ApplicationVersionKey = "Application-Version";
		internal const string MinimumTargetVersionKey = "Min-Target-Version";

		/// <summary>
		/// Gets the application version.
		/// </summary>
		public Version ApplicationVersion
		{
			get
			{
				Trace.Assert(this.ResponseMessage != null, "this.ResponseMessage = null");
				Trace.Assert(this.ResponseMessage.Headers != null, "this.ResponseMessage.Headers = null");
				try
				{
					var values = this.ResponseMessage.Headers.GetValues(ApplicationVersionKey);
					return (values.Count() > 0) ? new Version(values.First()) : null;
				}
				catch
				{
					return null;
				}
			}
		}

		/// <summary>
		/// Gets the minimum version of target device.
		/// </summary>
		public Version MinimumTargetVersion
		{
			get
			{
				Trace.Assert(this.ResponseMessage != null, "this.ResponseMessage = null");
				Trace.Assert(this.ResponseMessage.Headers != null, "this.ResponseMessage.Headers = null");
				try
				{
					var values = this.ResponseMessage.Headers.GetValues(MinimumTargetVersionKey);
					return (values.Count() > 0) ? new Version(values.First()) : null;
				}
				catch
				{
					return null;
				}
			}
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="WebServiceVersionNotMatchedException"/> class.
		/// </summary>
		/// <param name="responseMessage">The response message.</param>
		public WebServiceVersionNotMatchedException(HttpResponseMessage responseMessage)
			: base(responseMessage, "Your client version does not match the minimum required version.")
		{
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="WebServiceVersionNotMatchedException"/> class.
		/// </summary>
		///<param name="responseMessage">The response message.</param>
		///<param name="message">The message that describes the error.</param>
		public WebServiceVersionNotMatchedException(HttpResponseMessage responseMessage, string message)
			: base(responseMessage, message)
		{
		}
	}
}
