﻿using AirFusion.Common;
using AirFusion.Common.Network;
using AirFusion.Manager;
using AirFusion.Manager.DeviceManager;
using AirFusion.Manager.Notification;
using AirFusion.Toast;
using AirFusion.WebService;
using AirFusionCustomToast;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows;
using System.Windows.Controls;

namespace AirFusion.ToastDemo
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		internal AirFusionManager _manager;

		//internal AFToastManager _toastManager;
		internal GeneralNotificationManager _customToastManager;

		public MainWindow()
		{
			this.InitializeComponent();

			//this._toastManager = new AFToastManager();
			this._manager = new AirFusionManager();
			this._customToastManager = new GeneralNotificationManager(this._manager.NotificationManager);

			// Notification event register
			//this._manager.NotificationManager.NotificationRemoved += NotificationManager_NotificationItemRemoved;
			//this._manager.NotificationManager.NotificationReceived += NotificationManager_NotificationItemUpdated;

			// Toast event register (only on Win8/8.1)
			/*
			this._toastManager.ToastActivated += delegate(object sender, AFToastManager.AFToastActionActivateEventArgs args)
			{
			};
			this._toastManager.ToastDismissed += delegate(object sender, AFToastManager.AFToastActionDismissEventArgs args)
			{
			};
			this._toastManager.ToastFailed += delegate(object sender, AFToastManager.AFToastActionFailEventArgs args)
			{
			};
			*/

			// Custom event register (for win7)
			this._customToastManager.NotificationEvent += delegate(object sender, ToastEventArgs e)
			{
				switch (e.EventType)
				{
					case ToastEventType.ToastActivated:
						{
							if (e.Intent == ToastIntent.ShowScreenCasting)
							{
							}
							if (null != e.Data)
							{
							}
						}
						break;

					case ToastEventType.ToastLaunched:
						{
							if (null != e.Data)
							{
								string notificationKey = e.Data as string;
								this._manager.NotificationManager.NotificationLaunch(notificationKey);
							}
						}
						break;

					case ToastEventType.ToastIgnored:
						{
							if (null != e.Data)
							{
								string notificationKey = e.Data as string;
								this._manager.NotificationManager.NotificationIgnore(notificationKey);
							}
						}
						break;

					case ToastEventType.ToastActionLaunched:
						{
							if (null != e.Data && null != e.Args)
							{
								string notificationKey = e.Data as string;
								int qaId = Convert.ToInt32(e.Args);
								this._manager.NotificationManager.QuickActionLaunch(notificationKey, qaId);
							}
						}
						break;
				}
			};

			#region pairing / device manager

			//this._manager.UserDeviceManager.ConfigFilePath = CommonFuncs.ConvertPath(@"%APPDATA%\AirFusion\DeviceManager.ini");

			this._manager.UserDeviceManager.DiscoveryEvent += delegate(object sender, DiscoveryEventArgs eventArgs)
			{
			};

			this._manager.UserDeviceManager.PairingRequested += delegate(object sender, PairingRequestedEventArgs eventArgs)
			{
				eventArgs.Device.PairingResponseAsync(PairingResponse.CreatePairingResponse(true));
			};

			#endregion pairing / device manager

			this._manager.Initialize();

			// add paired device(s)
			//foreach (var device in this._manager.UserDeviceManager.DeviceStore.GetPairedDevices().ToArray())
			//{
			//    this.AddPairedDevice(device);
			//    device.StartMonitor(); // start monitor the device
			//}

			// enable discoverable
			this._manager.UserDeviceManager.EnableDiscoverable();

			//this.Title += "ToastDemo - (" + NetworkHelper.GetFirstLocalIpV4Address() + ":" + this._manager.WebServiceManager.Port + ")";
		}

		public void NotificationManager_NotificationItemUpdated(object sender, AFNotificationManager.AFNotificationItemUpdateEventArgs args)
		{
			string notificationKey = args.NotificationKey;
			ToastItem item = args.Toast;

			//this._toastManager.CreateAndShowToast(notificationKey, item);
			Dispatcher.Invoke(System.Windows.Threading.DispatcherPriority.Normal, new Action(
			() =>
			{
				this._customToastManager.CreateAndShowToast(notificationKey, args.DeviceName, item);
			}));

			/*
			AFCustomNotification.NotificationUIElement inNofication = new AFCustomNotification.NotificationUIElement();
			Dispatcher.Invoke(System.Windows.Threading.DispatcherPriority.Normal, new Action(
			() =>
			{
				AppItemListBox.Items.Add(inNofication);
				AppItemListBox.Items.Refresh();

				AFCustomNotification.ToastPopUp toast = new AFCustomNotification.ToastPopUp(inNofication);

				//toast = new ToastPopUp(title, text1, text2, notificationType);
				toast.ClosedByUser += this.ToastClosedByUser;
				toast.PressedEvent += this.ShowKey;
				toast.Show();
			}));
			*/
		}

		public void NotificationManager_NotificationItemRemoved(object sender, AFNotificationManager.AFNotificationItemRemoveEventArgs args)
		{
			string notificationKey = args.NotificationKey;
			foreach (var item in AppItemListBox.Items)
			{
				GeneralNotificationUIElement element = item as GeneralNotificationUIElement;
				if (element.NotificationKey.Equals(notificationKey))
				{
					Dispatcher.Invoke((System.Action)delegate()
					{
						AppItemListBox.Items.Remove(item);
						AppItemListBox.Items.Refresh();
					});

					break;
				}
			}
		}

		private void Window_Closed(object sender, EventArgs e)
		{
		}

		/// <summary>
		/// Toasts the closed by user.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
		private void ToastClosedByUser(object sender, EventArgs e)
		{
			MessageBox.Show("User closed the toast.");
		}

		private void AppItemListBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
		{
			//do something
			AirFusionCustomToast.GeneralNotificationUIElement selectedItem = (sender as ListBox).SelectedItem as AirFusionCustomToast.GeneralNotificationUIElement;
			btnRemoveToast.IsEnabled = selectedItem.Removable;
		}

		private void btnRemoveToast_Click(object sender, RoutedEventArgs e)
		{
			GeneralNotificationUIElement selectedItem = AppItemListBox.SelectedItem as GeneralNotificationUIElement;
			if (selectedItem.Removable)
				this._manager.NotificationManager.NotificaitonRemove(selectedItem.NotificationKey);
			else
				MessageBox.Show("This notification could not be removed!!");
		}

		private void AppItemListBox_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			GeneralNotificationUIElement selectedItem = ((sender as ListBox).SelectedItem as GeneralNotificationUIElement);
			if (selectedItem.Type == Manager.Notification.NotificationType.BasicWithLaunch)
				this._manager.NotificationManager.NotificationLaunch(selectedItem.NotificationKey);
			else
				MessageBox.Show("This notification could not be launched!!");
		}

		private void btnIgnore_Click(object sender, RoutedEventArgs e)
		{
			this._customToastManager.ConfigureNotificationStatus("92208075-1ce1-5198-5254-875b1bfd0000", DeviceIgnoreStatus.Ignore);
		}

		private void btnPermit_Click(object sender, RoutedEventArgs e)
		{
			this._customToastManager.ConfigureNotificationStatus("92208075-1ce1-5198-5254-875b1bfd0000", DeviceIgnoreStatus.Permit);
		}
	}
}