//  Copyright (C) 2000 Tridia Corporation. All Rights Reserved.
//  Copyright (C) 1999 AT&T Laboratories Cambridge. All Rights Reserved.
//
//  This file is part of the VNC system.
//
//  The VNC system is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
// TightVNC distribution homepage on the Web: http://www.tightvnc.com/
//
// If the source code for the VNC system is not available from the place
// whence you received this file, check http://www.uk.research.att.com/vnc or contact
// the authors on vnc@uk.research.att.com for information on obtaining it.

#ifndef CLIENTCONNECTION_H__
#define CLIENTCONNECTION_H__

#pragma once

#include "stdhdrs.h"
#ifdef UNDER_CE
#include "omnithreadce.h"
#else
#include "omnithread.h"
#endif

#include "VNCHandler.h"

#include "CapsContainer.h"
#include "VNCOptions.h"

////#include "VNCviewerApp.h"
#include "KeyMap.h"
////#include "ConnectingDialog.h"
#include "FileTransfer.h"
#include "RfbKeySym.h"

#include "zlib/zlib.h"

extern "C" {
#include "libjpeg/jpeglib.h"
}

#define SETTINGS_KEY_NAME "Software\\ORL\\VNCviewer\\Settings"
#define MAX_HOST_NAME_LEN 250
#define MAX_SIZE_PASSWORD 32

#define ZLIBHEX_DECOMP_UNINITED (-1)

#define TIGHT_ZLIB_BUFFER_SIZE 512

#include "Log.h"

extern Log vnclog;

struct Point
{
	Point() : x(0), y(0) {}
	Point(int x_, int y_) : x(x_), y(y_) {}

	inline void clear() { x = 0; y = 0; }
	inline void setPoint(int x_, int y_) { x = x_; y = y_; }
	inline void move(int deltaX, int deltaY) { x += deltaX; y += deltaY; }
	bool isEqualTo(const Point *other) const { return x == other->x &&
		y == other->y; }

	int x;
	int y;
};

class ClientConnection;
typedef void (ClientConnection:: *tightFilterFunc)(int);

class ClientConnection : public omni_thread
{
public:
	ClientConnection(int index);
	////ClientConnection(LPTSTR host, int port);

	virtual ~ClientConnection();

	int m_vnc_index;		//[New Member]

	////ClientConnection(VNCviewerApp *pApp);
	////ClientConnection(VNCviewerApp *pApp, SOCKET sock);
	////ClientConnection(VNCviewerApp *pApp, LPTSTR host, int port);
	////ClientConnection(VNCviewerApp *pApp, LPTSTR configFile);
	ClientConnectMsgFun MessageCallback;

	void Run();
	void SetConnectInfo(LPTSTR host, int port, LPTSTR pw);	// [New Function]
	void KillThread();
	////void CopyOptions(ClientConnection *source);
	////int  LoadConnection(char *fname, bool sess);
	////void UnloadConnection() { m_opts.m_configSpecified = false; }

	void Close();	// [New Function] -> Close connection

	int m_port;
	char m_host[MAX_HOST_NAME_LEN];
	HWND m_hSess;

	LRESULT CALLBACK WndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam);
	void SendClientInputText(char *str, size_t len);	// [New Function]
	bool InitClipboard(HWND hWnd);						// [New Function]
	static void RemoveClipboard(HWND hWnd);					// [New Function]
	void SendFileToServer(wchar_t* wsFilePath, int iFileId);			// [New Function]

	void ReadExact(char *buf, int bytes);		// [private -> public]
	void ReadString(char *buf, int length);		// [private -> public]
	void WriteExact(char *buf, int bytes);		// [private -> public]

	void SetFileTransferUpdateCallback(VNCUploadFileProgessFun upload_callback, VNCDownloadFileProgessFun download_callback);	// [New Function]
	void SetCoordinateTransformationInfo(int iWidth, int iHeight, int iDeltaX, int iDeltaY, float fDpiRate);	// [New Function]
	void SetDownloadFolderPath(wchar_t* folder_path);		// [New Function]

	void sendKeyboardEvent(bool downFlag, UINT32 key);	// [New Function]
	void sendVirtualKey(unsigned short usVirKeyCode);	// [New Function]
	void pasteClipboardText();							// [New Function]
	void sendTouchDown(long lx, long ly);
	void sendTouchMove(long lx, long ly);
	void sendTouchUp(long lx, long ly);

	void DropDownNotification();	// [New Function] Simulate user drop down notification on Android phone
	void SwipeLeftToRight();		// [New Function] Simulate user swipe page from left to right on Android phone
	void SwipeRightToLeft();		// [New Function] Simulate user swipe page from right to left on Android phone

private:

	void SendVncMessage(int msg_type, int fun_code, int err_code, char* msg);
	////static LRESULT CALLBACK WndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam);
	////static LRESULT CALLBACK WndProc1(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam);
	////static LRESULT CALLBACK Proc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam);
	////static LRESULT CALLBACK ScrollProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam);
	////void DoBlit();
	////VNCviewerApp *m_pApp;
	////ConnectingDialog *m_connDlg;

	char m_password[MAX_SIZE_PASSWORD];	// [New Member] Store the password

	bool m_enableFileTransfers;
	bool m_fileTransferDialogShown;
	friend class FileTransfer;
	FileTransfer *m_pFileTransfer;

	int m_iWinWidth;		// [New Member] Store the current size of AirFusion UI
	int m_iWinHeight;		// [New Member] Store the current size of AirFusion UI
	int m_iDeltaX;
	int m_iDeltaY;
	float m_fWidthRate;		// [New Member] Coordination translation
	float m_fHeightRate;	// [New Member] Coordination translation
	float m_fDPIRate;		// [New Member] Coordination translation
	bool m_bIsLandscapeMode;	// [New Member] True = Landscape; False = Portrait
	bool m_bIsClintCutData;		// [New Member] True = Clinet cuts the data; False = No Clinet cuts the data

	SOCKET m_sock;
	bool m_serverInitiated;
	HWND m_hwnd;		// [ => Main Windows (WPF)]
	////HWND m_hwnd, m_hbands, m_hwnd1,
	////	 m_hToolbar, m_hwndscroll;

	////void Init(VNCviewerApp *pApp);
	void Init();
	void InitCapabilities();
	void InitWinSocket();			// [New Function]

	////void CreateDisplay();
	////HWND CreateToolbar();
	////void GetConnectDetails();
	void Connect();
	void SetSocketOptions();
	void NegotiateProtocolVersion();
	void PerformAuthentication();
	int ReadSecurityType();
	int SelectSecurityType();
	void SetupTunneling();
	void PerformAuthenticationTight();
	void Authenticate(CARD32 authScheme);
	bool AuthenticateNone(char *errBuf, int errBufSize);
	bool AuthenticateVNC(char *errBuf, int errBufSize);
	void ReadServerInit();
	void ReadInteractionCaps();
	void ReadCapabilityList(CapsContainer *caps, int count);
	void SendClientInit();
	////void CreateLocalFramebuffer();
	////void SaveConnection();
	////void SaveConnectionHistory();
	////
	////void SetupPixelFormat();
	////void SetFormatAndEncodings();
	////void SendSetPixelFormat(rfbPixelFormat newFormat);

	////void SendIncrementalFramebufferUpdateRequest();
	////void SendFullFramebufferUpdateRequest();
	////void SendAppropriateFramebufferUpdateRequest();
	////void SendFramebufferUpdateRequest(int x, int y, int w, int h, bool incremental);
	////
	////void ProcessPointerEvent(int x, int y, DWORD keyflags, UINT msg);
	////void SubProcessPointerEvent(int x, int y, DWORD keyflags);
	void SendPointerEvent(int x, int y, int buttonMask);
	////void ProcessKeyEvent(int virtkey, DWORD keyData);
	void SendKeyEvent(CARD32 key, bool down);

	// [New Keyboard Handler] TightVNC v2.7.10 =============================
	// bool DesktopWindow::onKey(...)
	bool m_ctrlDown;
	bool m_altDown;
	bool onKey(WPARAM wParam, LPARAM lParam);

	// keyboard support
	friend class RfbKeySym;
	std::auto_ptr<RfbKeySym> m_rfbKeySym;

	// [ToDo][New Mouse Handler] TightVNC v2.7.10 ==========================
	//bool DesktopWindow::onMouse(...)
	bool m_bSwapMouse;	// True = Swap the right and left buttons (Default value is False)

	bool onMouse(unsigned char mouseButtons, unsigned short wheelSpeed, POINT position);

	// This variable contained previously state of mouse-button and position of cursor.
	unsigned char m_previousMouseState;
	Point m_previousMousePos;

	static const int MOUSE_LDOWN  = 1;
	static const int MOUSE_MDOWN  = 2;
	static const int MOUSE_RDOWN  = 4;
	static const int MOUSE_WUP    = 8;
	static const int MOUSE_WDOWN  = 16;

	// This flag is used for checking if the pervious LMouseDown has receives one LMouseUp event
	// - True : LMouseUp evnet has not happened after one LMouseDown had triggerred
	// - False: No RMouseDown event occurs
	bool m_bFlag_OnceLMouseDown;
	// =====================================================================

	////void SwitchOffKey();

	////void ReadScreenUpdate();
	////void Update(RECT *pRect);
	////void SizeWindow(bool centered);
	////void PositionChildWindow();
	////bool ScrollScreen(int dx, int dy);
	////void UpdateScrollbars();
	////void EnableFullControlOptions();
	////void EnableAction(int id, bool enable);

	////void InvalidateScreenRect(const RECT *pRect);

	////void ReadRawRect(rfbFramebufferUpdateRectHeader *pfburh);
	////void ReadCopyRect(rfbFramebufferUpdateRectHeader *pfburh);
	////void ReadRRERect(rfbFramebufferUpdateRectHeader *pfburh);
	////void ReadCoRRERect(rfbFramebufferUpdateRectHeader *pfburh);
	////void ReadHextileRect(rfbFramebufferUpdateRectHeader *pfburh);
	////void HandleHextileEncoding8(int x, int y, int w, int h);
	////void HandleHextileEncoding16(int x, int y, int w, int h);
	////void HandleHextileEncoding32(int x, int y, int w, int h);
	////void ReadZlibRect(rfbFramebufferUpdateRectHeader *pfburh);

	////void ReadNewFBSize(rfbFramebufferUpdateRectHeader *pfburh);

	////// ClientConnectionTight.cpp
	////void ReadTightRect(rfbFramebufferUpdateRectHeader *pfburh);
	////int ReadCompactLen();
	////int InitFilterCopy (int rw, int rh);
	////int InitFilterGradient (int rw, int rh);
	////int InitFilterPalette (int rw, int rh);
	////void FilterCopy8 (int numRows);
	////void FilterCopy16 (int numRows);
	////void FilterCopy24 (int numRows);
	////void FilterCopy32 (int numRows);
	////void FilterGradient8 (int numRows);
	////void FilterGradient16 (int numRows);
	////void FilterGradient24 (int numRows);
	////void FilterGradient32 (int numRows);
	////void FilterPalette (int numRows);
	////void DecompressJpegRect(int x, int y, int w, int h);

	////// ClientConnectionZlibHex.cpp
	////void HandleZlibHexEncoding8(int x, int y, int w, int h);
	////void HandleZlibHexEncoding16(int x, int y, int w, int h);
	////void HandleZlibHexEncoding32(int x, int y, int w, int h);
	////void HandleZlibHexSubencodingStream8(int x, int y, int w, int h, int subencoding);
	////void HandleZlibHexSubencodingStream16(int x, int y, int w, int h, int subencoding);
	////void HandleZlibHexSubencodingStream32(int x, int y, int w, int h, int subencoding);
	////void HandleZlibHexSubencodingBuf8(int x, int y, int w, int h, int subencoding, unsigned char * buffer);
	////void HandleZlibHexSubencodingBuf16(int x, int y, int w, int h, int subencoding, unsigned char * buffer);
	////void HandleZlibHexSubencodingBuf32(int x, int y, int w, int h, int subencoding, unsigned char * buffer);
	////void ReadZlibHexRect(rfbFramebufferUpdateRectHeader *pfburh);

	////bool zlibDecompress(unsigned char *from_buf, unsigned char *to_buf, unsigned int count, unsigned int size, z_stream *decompressor);

	////void ReadRBSRect(rfbFramebufferUpdateRectHeader *pfburh);
	////BOOL DrawRBSRect8(int x, int y, int w, int h, CARD8 **pptr);
	////BOOL DrawRBSRect16(int x, int y, int w, int h, CARD8 **pptr);
	////BOOL DrawRBSRect32(int x, int y, int w, int h, CARD8 **pptr);

	////// ClientConnectionCursor.cpp
	////bool prevCursorSet;
	////HDC m_hSavedAreaDC;
	////HBITMAP m_hSavedAreaBitmap;
	////COLORREF *rcSource;
	////bool *rcMask;
	////int rcHotX, rcHotY, rcWidth, rcHeight;
	////int rcCursorX, rcCursorY;
	////int rcLockX, rcLockY, rcLockWidth, rcLockHeight;
	////bool rcCursorHidden, rcLockSet;

	////void ReadCursorShape(rfbFramebufferUpdateRectHeader *pfburh);
	////void ReadCursorPos(rfbFramebufferUpdateRectHeader *pfburh);
	////void SoftCursorLockArea(int x, int y, int w, int h);
	////void SoftCursorUnlockScreen();
	////void SoftCursorMove(int x, int y);
	////void SoftCursorFree();
	////bool SoftCursorInLockedArea();
	////void SoftCursorSaveArea();
	////void SoftCursorRestoreArea();
	////void SoftCursorDraw();
	////void SoftCursorToScreen(RECT *screenArea, POINT *cursorOffset);

	////// ClientConnectionFullScreen.cpp
	////void SetFullScreenMode(bool enable);
	////bool InFullScreenMode();
	////void RealiseFullScreenMode(bool suppressPrompt);
	////bool BumpScroll(int x, int y);

	// ClientConnectionClipboard.cpp
	void ProcessLocalClipboardChange();
	void UpdateLocalClipboard(char *buf, size_t len);
	bool IsTextClipboardFormat();
	void SendClientCutText(char *str, size_t len);
	void ReadServerCutText();

	void ReadSetColourMapEntries();
	////void ReadBell();

	////void SendRFBMsg(CARD8 msgType, void* data, int length);
	////void ReadExact(char *buf, int bytes);
	////void ReadString(char *buf, int length);
	////void WriteExact(char *buf, int bytes);
	char *ReadFailureReason();

	// This is what controls the thread
	void * run_undetached(void* arg);
	bool m_bKillThread;

	// Utilities

	////	// These draw a solid rectangle of colour on the bitmap
	////	// They assume the bitmap is already selected into the DC, and the
	////	// DC is locked if necessary.
	////#ifndef UNDER_CE
	////	// Normally this is an inline call to a GDI method.
	////	inline void FillSolidRect(RECT *pRect, COLORREF color) {
	////		COLORREF oldbgcol = SetBkColor(m_hBitmapDC, color);
	////		// This is the call MFC uses for FillSolidRect. Who am I to argue?
	////		::ExtTextOut(m_hBitmapDC, 0, 0, ETO_OPAQUE, pRect, NULL, 0, NULL);
	////	};
	////#else
	////	// Under WinCE this is a manual insert into a pixmap,
	////	// and is a little too complicated for an inline.
	////	void FillSolidRect(RECT *pRect, COLORREF color);
	////#endif // UNDER_CE

	////inline void FillSolidRect(int x, int y, int w, int h, COLORREF color) {
	////	RECT r;
	////	r.left = x;		r.right = x + w;
	////	r.top = y;		r.bottom = y + h;
	////	FillSolidRect(&r, color);
	////};

	//// how many other windows are owned by this process?
	//unsigned int CountProcessOtherWindows();

	//// Buffer for network operations
	void CheckBufferSize(size_t bufsize);
	char *m_netbuf;
	size_t m_netbufsize;
	omni_mutex m_bufferMutex, m_clipMutex, m_readMutex, m_writeMutex, m_sockMutex, m_callbackMutex;
	////m_bitmapdcMutex, m_cursorMutex;

	////// Buffer for zlib decompression.
	////void CheckZlibBufferSize(size_t bufsize);
	////unsigned char *m_zlibbuf;
	////size_t m_zlibbufsize;

	////// zlib decompression state
	////bool m_decompStreamInited;
	////z_stream m_decompStream;
	////z_stream m_decompStreamRaw;
	////z_stream m_decompStreamEncoded;

	// Variables used by tight encoding:

	// Separate buffer for tight-compressed data.
	////char m_tightbuf[TIGHT_ZLIB_BUFFER_SIZE];

	////// Four independent compression streams for zlib library.
	////z_stream m_tightZlibStream[4];
	////bool m_tightZlibStreamActive[4];

	////// Tight filter stuff. Should be initialized by filter initialization code.
	////tightFilterFunc m_tightCurrentFilter;
	////bool m_tightCutZeros;
	////int m_tightRectWidth, m_tightRectColors;
	////COLORREF m_tightPalette[256];
	////CARD8 m_tightPrevRow[2048*3*sizeof(CARD16)];

	////// Bitmap for local copy of screen, and DC for writing to it.
	////HBITMAP m_hBitmap;
	////HDC		m_hBitmapDC;
	////HPALETTE m_hPalette;

#ifdef UNDER_CE
	// Under WinCE this points to the DIB pixels.
	BYTE* m_bits;
#endif

	//// Keyboard mapper
	//KeyMap m_keymap;

	// RFB settings
	VNCOptions m_opts;

	// Protocol capabilities
	CapsContainer m_tunnelCaps;		// known tunneling/encryption methods
	CapsContainer m_authCaps;		// known authentication schemes
	CapsContainer m_serverMsgCaps;	// known non-standard server messages
	CapsContainer m_clientMsgCaps;	// known non-standard client messages
	CapsContainer m_encodingCaps;	// known encodings besides Raw

	TCHAR *m_desktopName;
	unsigned char m_encPasswd[8];
	bool m_passwdSet;
	int m_authScheme;
	rfbServerInitMsg m_si;
	rfbPixelFormat m_myFormat, m_pendingFormat;
	// protocol version in use.
	int m_minorVersion;
	bool m_tightVncProtocol;
	bool m_threadStarted, m_running;
	// mid-connection format change requested
	bool m_pendingFormatChange;
	// Display connection info;
	void ShowConnInfo();

	// Window may be scrollable - these control the scroll position
	int m_hScrollPos, m_hScrollMax, m_vScrollPos, m_vScrollMax;

	// The size of the current client area
	int m_cliwidth, m_cliheight;
	// The size of a window needed to hold entire screen without scrollbars
	int m_fullwinwidth, m_fullwinheight;
	int m_winwidth, m_winheight;
	// The size of the CE CommandBar
	int m_barheight;

	// Dormant basically means minimized; updates will not be requested
	// while dormant.
	void SetDormant(bool newstate);
	bool m_dormant;

	// The number of bytes required to hold at least one pixel.
	unsigned int m_minPixelBytes;
	// Next window in clipboard chain
	static HWND sHwnd_NextViewer;
	//bool m_initialClipboardSeen;
	static bool sbFlag_AddClipboardChain;	// [Workaround] To fix the issue that it couldn't remove the window from Clipboard Chain

	////// Are we waiting on a timer for emulating three buttons?
	////bool m_waitingOnEmulateTimer;
	////// Or are we emulating the middle button now?
	////bool m_emulatingMiddleButton;
	////// Emulate 3 buttons mouse timer:
	////UINT m_emulate3ButtonsTimer;
	////// Buttons pressed, waiting for timer in emulating 3 buttons:
	////DWORD m_emulateKeyFlags;
	////int m_emulateButtonPressedX;
	////int m_emulateButtonPressedY;
};

#ifdef UNDER_CE
#define SETPIXEL(b,x,y,c) SetPixel((b),(x),(y),(c))
#else
#define SETPIXEL(b,x,y,c) SetPixelV((b),(x),(y),(c))
#endif

#endif // CLIENTCONNECTION_H__