//  Copyright (C) 1999 AT&T Laboratories Cambridge. All Rights Reserved.
//
//  This file is part of the VNC system.
//
//  The VNC system is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
// TightVNC distribution homepage on the Web: http://www.tightvnc.com/
//
// If the source code for the VNC system is not available from the place
// whence you received this file, check http://www.uk.research.att.com/vnc or
// contact the authors on vnc@uk.research.att.com for information on obtaining it.
//

#include "stdhdrs.h"
////#include "vncviewer.h"
#include "ClientConnection.h"
#include "Exception.h"

// This file contains the code for getting text from, and putting text into
// the Windows clipboard.

//
// ProcessClipboardChange
// Called by ClientConnection::WndProc.
// We've been informed that the local clipboard has been updated.
// If it's text we want to send it to the server.
//

void ClientConnection::ProcessLocalClipboardChange()
{
	vnclog.Print(2, _T("[VNC] ClientConnection::ProcessLocalClipboardChange(): Clipboard changed\n"));

	HWND hOwner = GetClipboardOwner();

	// P_O_b Remote Windows  Copy r]Yݾާ@qƻsr
	if(hOwner == this->m_hwnd){
		//=> Yes (hOwner = m_hwnd): ơA]| Copy O ServerAM Server |^ǽƻsnr
		vnclog.Print(2, _T("[VNC] ClientConnection::ProcessLocalClipboardChange(): We changed it - ignore!\n"));
	} else if (!ClientConnection::sbFlag_AddClipboardChain) {
		// pGO@}l Initial State hAñN Initial Flag ] true
		vnclog.Print(2, _T("[VNC] ClientConnection::ProcessLocalClipboardChange(): Don't send initial clipboard!\n"));
		ClientConnection::sbFlag_AddClipboardChain = true;
	} else if (!m_opts.m_DisableClipboard) {
		// oO User bۤvq䥦{ UI ƻsr]Ob Remote Windows ƻs^-> Ns Copy rǵ  Server
		// The clipboard should not be modified by more than one thread at once
		omni_mutex_lock l(m_clipMutex);

		if (OpenClipboard(this->m_hwnd)) {
			HGLOBAL hglb = GetClipboardData(CF_UNICODETEXT);
			if (hglb == NULL) {
				CloseClipboard();
			} else {
				LPWSTR lpwstr = (LPWSTR) GlobalLock(hglb);

				WCHAR *winContentsW = new WCHAR[wcslen(lpwstr) + 1];
				wcscpy(winContentsW, lpwstr);
				GlobalUnlock(hglb);
				CloseClipboard();

				int len = WideCharToMultiByte (CP_UTF8, 0, (LPCWSTR)winContentsW , -1, NULL, 0, NULL, NULL);
				char* unixContents = new char[len + 1];
				ZeroMemory (unixContents, len + 1);
				WideCharToMultiByte (CP_UTF8, 0, (LPCWSTR)winContentsW, -1, unixContents, len, NULL, NULL);

				try
				{
					SendClientCutText(unixContents, len + 1);
					m_bIsClintCutData = true;
				}
				catch (VncException &e)
				{
					if(e.m_ErrType == EXP_SocketError)
						SendVncMessage(MT_ErrorMessage, FC_ClipboardText, EC_SocketError, nullptr);
					else
						SendVncMessage(MT_ErrorMessage, FC_ClipboardText, EC_OtherError, nullptr);
				}
				catch (WarningException &e)
				{
					vnclog.Print(0, _T("[ERROR] ClientConnection::ProcessLocalClipboardChange(): Exception while sending clipboard text : %s\n"), e.m_info);
					////DestroyWindow(m_main_hwnd);	// [ToDo]
				}

				delete [] winContentsW;
				delete [] unixContents;
			}
		}
	}

	if( ClientConnection::sHwnd_NextViewer != NULL)
	{
		// Pass the message to the next window in clipboard viewer chain
		::SendMessage( ClientConnection::sHwnd_NextViewer, WM_DRAWCLIPBOARD , 0,0);
	}
}

// We've read some text from the remote server, and
// we need to copy it into the local clipboard.
// Called by ClientConnection::ReadServerCutText()

void ClientConnection::UpdateLocalClipboard(char *buf, size_t len) {
	if (m_opts.m_DisableClipboard)
	{
		vnclog.Print(5, "[VNC] ClientConnection::UpdateLocalClipboard(): Not support; m_DisableClipboard = true !!\n");
		return;
	}

	WCHAR* winContentsW = new WCHAR[len + 1];
	ZeroMemory (winContentsW, sizeof(WCHAR) * (len + 1));
	MultiByteToWideChar(CP_UTF8, 0, buf, -1, winContentsW, len);

	int iWslen = wcslen(winContentsW);
	vnclog.Print(5, "[VNC] ClientConnection::UpdateLocalClipboard(): Set clipboard content (text length = %d)\n", iWslen);

	// The clipboard should not be modified by more than one thread at once
	{
		omni_mutex_lock l(m_clipMutex);

		if (!OpenClipboard(m_hwnd)) {
			vnclog.Print(1, "[ERROR] ClientConnection::UpdateLocalClipboard(): Failed to open clipboard !!\n");
			throw WarningException("Failed to open clipboard\n");
		}
		if (! ::EmptyClipboard()) {
			vnclog.Print(1, "[ERROR] ClientConnection::UpdateLocalClipboard(): Failed to empty clipboard !!\n");
			throw WarningException("Failed to empty clipboard\n");
		}

		// Allocate a global memory object for the text.
		HGLOBAL hglbCopy = GlobalAlloc(GMEM_DDESHARE, sizeof(WCHAR) * (iWslen + 1));
		if (hglbCopy != NULL) {
			// Lock the handle and copy the text to the buffer.
			wchar_t* lptWstrCopy = (wchar_t*) GlobalLock(hglbCopy);
			memcpy(lptWstrCopy, winContentsW, sizeof(WCHAR) * (iWslen + 1));
			lptWstrCopy[iWslen] = (WCHAR) 0;    // null character
			GlobalUnlock(hglbCopy);          // Place the handle on the clipboard.
			SetClipboardData(CF_UNICODETEXT, hglbCopy);
			m_bIsClintCutData = true;
		}

		delete [] winContentsW;

		if (! ::CloseClipboard()) {
			throw WarningException("Failed to close clipboard\n");
		}
	}
}

bool ClientConnection::IsTextClipboardFormat() {
	vnclog.Print(2, _T("[VNC] ClientConnection::IsTextClipboardFormat() >>>\n"));
	bool isText=false;

	if (OpenClipboard(this->m_hwnd)) {
		HGLOBAL hglb = GetClipboardData(CF_UNICODETEXT);
		if (hglb != NULL) {
			vnclog.Print(2, _T("[VNC] ClientConnection::IsTextClipboardFormat(): true\n"));
			isText = true;
		}
		CloseClipboard();
	}

	vnclog.Print(2, _T("[VNC] ClientConnection::IsTextClipboardFormat() <<<\n"));
	return isText;
}